<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Block\Adminhtml\System\Config\Field;

use Magento\Config\Block\System\Config\Form\Field;
use Magento\Framework\Data\Form\Element\AbstractElement;
use Magento\Backend\Block\Template\Context;
use Magento\Framework\View\Helper\SecureHtmlRenderer;
use WriteTextAI\WriteTextAI\Model\SettingManager;

class MinMax extends Field
{
    /**
     * @var SettingManager
     */
    protected $settingManager;

    /**
     * @param Context $context
     * @param SettingManager $settingManager
     * @param array $data
     * @param SecureHtmlRenderer|null $secureRenderer
     */
    public function __construct(
        Context $context,
        SettingManager $settingManager,
        array $data = [],
        ?SecureHtmlRenderer $secureRenderer = null
    ) {
        $this->settingManager = $settingManager;
        parent::__construct(
            $context,
            $data,
            $secureRenderer
        );
    }

    /**
     * @inheritdoc
     */
    protected function _getElementHtml(AbstractElement $element)
    {
        $rules = $this->getRules();

        $minOutputWords = $rules['minOutputWords'] ?? '';
        $maxOutputWords = $rules['maxOutputWords'] ?? '';
        $element->setData('type', 'number');

        $html = parent::_getElementHtml($element);
        
        $html .= '<script type="text/javascript">
            require(["jquery", "jquery/ui"], function ($) {
                $(document).ready(function () {
                    var id = "' . $element->getHtmlId() . '";

                    $("#" + id).on("change", function () {
                        var value = $(this).val(),
                            max = ' . $maxOutputWords . ',
                            min = ' . $minOutputWords . ';

                        switch (id) {
                            case "writetextai_settings_description_min":
                                max = Number($("#writetextai_settings_description_max").val()) - 1;
                                break;
                            case "writetextai_settings_description_max":
                                min = Number($("#writetextai_settings_description_min").val()) + 1;
                                break;
                            case "writetextai_settings_excerpt_min":
                                max = Number($("#writetextai_settings_excerpt_max").val()) - 1;
                                break;
                            case "writetextai_settings_excerpt_max":
                                min = Number($("#writetextai_settings_excerpt_min").val()) + 1;
                                break;
                            default:
                                break;
                        }

                        if (value < min) {
                            $(this).val(min);
                        } else if (value > max) {
                            $(this).val(max);
                        }
                    });
                });
            });
        </script>';
        
        return $html;
    }

    /**
     * Get rules
     *
     * @return array
     */
    public function getRules()
    {
        return $this->settingManager->getSettings('rules');
    }
}
