<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Edit\Audience;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use WriteTextAI\WriteTextAI\ViewModel\Premium;

class Get extends Action
{
    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * Constructor
     *
     * @param Context $context
     * @param ApiManager $apiManager
     * @param Premium $premium
     */
    public function __construct(
        Context $context,
        ApiManager $apiManager,
        Premium $premium
    ) {
        $this->apiManager = $apiManager;
        $this->premium = $premium;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        if (!$this->premium->getIsPremium()) {
            $response->setData([
                'success' => false,
                'is_premium' => false,
                'message' => __('Premium is required to do this action.')
            ]);

            return $response;
        }
        
        try {
            $productId = $this->getRequest()->getParam('product_id');
            $storeId = $this->getRequest()->getParam('store_id');
            $keywords = $this->getRequest()->getParam('keywords');

            $keywordsTexts = array_column($keywords, 'text');
            if (!empty($keywordsTexts)) {
                $audience = $this->apiManager->getSuggestedAudience($storeId, $productId, $keywordsTexts, 'true');
            }
            
            $result = [
                'success' => true,
                'selected' => $audience['selected'] ?? [],
                'values' => $audience['values'] ?? []
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage()
            ];
        }

        $response->setData($result);

        return $response;
    }
}
