<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Edit;

use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Filter\FilterManager;
use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;
use Magento\Store\Model\ScopeInterface;
use WriteTextAI\WriteTextAI\Helper\Data;
use WriteTextAI\WriteTextAI\Helper\Html;
use WriteTextAI\WriteTextAI\Helper\Image;
use WriteTextAI\WriteTextAI\Helper\Generate as GenerateHelper;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use WriteTextAI\WriteTextAI\Model\OptionSource\Filter\Fields;
use WriteTextAI\WriteTextAI\Model\GeneratedText;
use WriteTextAI\WriteTextAI\Model\AiProductManager;
use WriteTextAI\WriteTextAI\Helper\Product as ProductHelper;
use Magento\Framework\UrlInterface;
use WriteTextAI\WriteTextAI\ViewModel\Premium;
use WriteTextAI\WriteTextAI\ViewModel\Settings;
use Magento\Catalog\Helper\Output as OutputHelper;

class Generate extends Action
{
    public const ADMIN_RESOURCE = 'WriteTextAI_WriteTextAI::generate';

    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var RemoteAddress
     */
    protected $remoteAddress;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var ProductRepositoryInterface
     */
    protected $productRepository;

    /**
     * @var GeneratedText
     */
    protected $generatedText;

    /**
     * @var AiProductManager
     */
    protected $aiProductManager;

    /**
     * @var Html
     */
    protected $htmlHelper;

    /**
     * @var Image
     */
    protected $imageHelper;

    /**
     * @var GenerateHelper
     */
    protected $generateHelper;

    /**
     * @var ProductHelper
     */
    protected $productHelper;

    /**
     * @var FilterManager
     */
    protected $filterManager;

    /**
     * @var UrlInterface
     */
    protected $urlBuilder;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * @var Settings
     */
    protected $settings;

    /**
     * @var OutputHelper
     */
    protected $outputHelper;

    /**
     * Constructor
     *
     * @param Context $context
     * @param ApiManager $apiManager
     * @param RemoteAddress $remoteAddress
     * @param ScopeConfigInterface $scopeConfig
     * @param Data $helper
     * @param ProductRepositoryInterface $productRepository
     * @param GeneratedText $generatedText
     * @param AiProductManager $aiProductManager
     * @param Html $htmlHelper
     * @param Image $imageHelper
     * @param GenerateHelper $generateHelper
     * @param ProductHelper $productHelper
     * @param FilterManager $filterManager
     * @param UrlInterface $urlBuilder
     * @param Premium $premium
     * @param Settings $settings
     * @param OutputHelper $outputHelper
     */
    public function __construct(
        Context $context,
        ApiManager $apiManager,
        RemoteAddress $remoteAddress,
        ScopeConfigInterface $scopeConfig,
        Data $helper,
        ProductRepositoryInterface $productRepository,
        GeneratedText $generatedText,
        AiProductManager $aiProductManager,
        Html $htmlHelper,
        Image $imageHelper,
        GenerateHelper $generateHelper,
        ProductHelper $productHelper,
        FilterManager $filterManager,
        UrlInterface $urlBuilder,
        Premium $premium,
        Settings $settings,
        OutputHelper $outputHelper
    ) {
        $this->apiManager = $apiManager;
        $this->remoteAddress = $remoteAddress;
        $this->scopeConfig = $scopeConfig;
        $this->helper = $helper;
        $this->productRepository = $productRepository;
        $this->generatedText = $generatedText;
        $this->aiProductManager = $aiProductManager;
        $this->htmlHelper = $htmlHelper;
        $this->imageHelper = $imageHelper;
        $this->generateHelper = $generateHelper;
        $this->productHelper = $productHelper;
        $this->filterManager = $filterManager;
        $this->urlBuilder = $urlBuilder;
        $this->premium = $premium;
        $this->settings = $settings;
        $this->outputHelper = $outputHelper;
        parent::__construct($context);
    }

    /**
     * Execute action
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);
        
        try {
            $tones = $this->getRequest()->getPost('tones', explode(",", (string) $this->helper->getTones()));
            $style = $this->getRequest()->getPost('styles');
            $audience = $this->getRequest()->getPost('audience');
            $storeId = $this->getRequest()->getPost('store_id');
            $productId = $this->getRequest()->getPost('product_id');
            $descriptionMin = $this->getRequest()->getPost('desc_min', "");
            $descriptionMax = $this->getRequest()->getPost('desc_max', "");
            $excerptMin = $this->getRequest()->getPost('exc_min', "");
            $excerptMax = $this->getRequest()->getPost('exc_max', "");
            $attributes = $this->getRequest()->getPost('product_attributes', []);
            $attributes_selected = $this->getRequest()->getPost('product_attributes_selected', []);
            $otherProductDetails = $this->getRequest()->getPost('other_product_details', "");
            $selectedFields = $this->getRequest()->getPost('selected_fields', []);
            $selectedImages = $this->getRequest()->getPost('selected_images', []);
            $gallery = $this->getRequest()->getPost('images', []);
            $customTone = $this->getRequest()->getPost('custom_tone', "");
            $customStyle = $this->getRequest()->getPost('custom_style', "");
            $referenceProductId = $this->getRequest()->getPost('reference_product', "");
            $referenceProductChecked = filter_var(
                $this->getRequest()->getPost('reference_product_checked'),
                FILTER_VALIDATE_BOOLEAN
            );
            $rewrite = filter_var($this->getRequest()->getPost('rewrite'), FILTER_VALIDATE_BOOLEAN);
            $keywordAnalysisViews = $this->getRequest()->getPost('keyword_analysis_views');
            
            $product = $this->productRepository->getById($productId, false, $storeId);
            $productName = $this->outputHelper->productAttribute(
                $product,
                $product->getName(),
                'name'
            );
            $mappingSettings = $this->helper->getMappingSettings();
            $locale =  $this->scopeConfig->getValue(
                'general/locale/code',
                ScopeInterface::SCOPE_STORE,
                $storeId
            );
            $language = str_replace('_', '-', $locale);
            $ipAddress = $this->remoteAddress->getRemoteAddress();

            // get keywords and semantic keywords
            $keywords = $this->apiManager->getSemanticKeywords($storeId, $productId, $productName);
            $semanticKeywords = $keywords['selected'];
            $texts = array_map(function ($text) {
                return $text['text'];
            }, $keywords['texts']);

            // keywords
            $targetKeywords = array_filter($texts, function ($keyword) use ($productName) {
                return $keyword !== $productName;
            });
            $targetKeywords = array_values($targetKeywords);

            $rules = $this->settings->getSettings('rules');

            if (!empty($selectedFields) || !empty($selectedImages)) {
                $maxOutputWords = $rules['maxOutputWords'] ?? '';
                $maxReferenceTextLength = $rules['maxReferenceTextLength'] ?? '';
                $additionalReferenceTextLength = $rules['additionalReferenceTextLength'] ?? '';
                $maxAttributes = $rules['maxAttributes'] ?? '';
                $maxAttributeValueLength = $rules['maxAttributeValueLength'] ?? '';

                $credits = $this->settings->getSettings('credits');
                $wordsPerCredit = $credits['wordsPerCredit'] ?? '';
                // phpcs:ignore Generic.Files.LineLength.TooLong
                $maxInputChar = ((($maxOutputWords / $wordsPerCredit) - 1) * $additionalReferenceTextLength) + $maxReferenceTextLength;

                // get attributes
                $attributesData = [];
                $images = [];
                foreach ($attributes as $attribute) {
                    if (!in_array($attribute['attribute_code'], $attributes_selected)) {
                        continue;
                    }
                    
                    if ($maxAttributes <= count($attributesData)) {
                        continue;
                    }

                    if (strtolower($attribute['attribute']) !== 'thumbnail') {
                        $attributeValue = str_replace("<br />", "\n", $attribute['details']);
                        $attributeValue = $this->filterManager->stripTags($attributeValue);
                        $attributeValue = substr($attributeValue, 0, $maxAttributeValueLength);
                        $attributesData[] = [
                            "name" => $attribute['attribute'],
                            "value" => $attributeValue,
                            "isCustom" => "false"
                        ];
                    } else {
                        if (!$this->premium->isPremium()) {
                            continue;
                        }

                        try {
                            $images[] = $this->imageHelper->getImageApiId(
                                $storeId,
                                $ipAddress,
                                $language,
                                $attribute['value'],
                                $attribute['details']
                            );
                        // phpcs:ignore Magento2.CodeAnalysis.EmptyBlock.DetectedCatch
                        } catch (\WriteTextAI\WriteTextAI\Exception\ImageUnsupported $e) {
                            // do nothing
                        // phpcs:ignore Magento2.CodeAnalysis.EmptyBlock.DetectedCatch
                        } catch (\WriteTextAI\WriteTextAI\Exception\ImageInvalid $e) {
                            // do nothing
                        }
                    }
                }

                if ($referenceProductChecked) {
                    $referenceProduct = $this->productRepository->getById($referenceProductId, false, $storeId);
                }

                // fields to generate
                $fieldConfig = [
                    'page_title' => Fields::PAGE_TITLE,
                    'page_description' => Fields::PAGE_DESCRIPTION,
                    'product_description' => Fields::PRODUCT_DESCRIPTION,
                    'short_product_description' => Fields::EXCERPT,
                    'open_graph' => Fields::OPEN_GRAPH,
                ];

                $fields = [];
                foreach ($fieldConfig as $fieldName => $fieldConstant) {
                    if (in_array($fieldName, $selectedFields)) {
                        $currentValue = $product->getData($mappingSettings[$fieldName]);
                        $field = [
                            "field" => $fieldConstant,
                            "currentValue" => $currentValue
                        ];

                        if ($fieldName == 'product_description') {
                            $field["minWords"] = $descriptionMin;
                            $field["maxWords"] = $descriptionMax;
                        } elseif ($fieldName == 'short_product_description') {
                            $field["minWords"] = $excerptMin;
                            $field["maxWords"] = $excerptMax;
                        }

                        if ($referenceProductChecked) {
                            $referenceText = $this->generateHelper->concatText(
                                $referenceProduct->getData($mappingSettings[$fieldName]),
                                $maxInputChar,
                            );
                            $field['referenceText'] = $this->htmlHelper->removeDisallowedHtml($referenceText);
                            $field['referenceRecordId'] = $referenceProduct->getId();
                        }

                        if ($rewrite && $currentValue) {
                            $rewriteText = $this->generateHelper->concatText(
                                $currentValue,
                                $maxInputChar,
                            );
                            $field['rewriteText'] = $this->htmlHelper->removeDisallowedHtml($rewriteText);
                            $fields[] = $field;
                        }

                        if (!$rewrite) {
                            $fields[] = $field;
                        }
                    }
                }
                
                // get custom audience
                $texts = array_map(function ($text) {
                    return $text['text'];
                }, $keywords['texts']);

                if (!empty($texts)) {
                    $selectedAudience = $this->apiManager->getSuggestedAudience(
                        $storeId,
                        $productId,
                        implode(',', $texts)
                    );
                } else {
                    $selectedAudience = [];
                }
                
                $textData = [
                    "name" => $productName,
                    "recordId" => $productId,
                    "sku" => $product->getSku(),
                    "options" => 1,
                    "autoselectFirst" => true,
                    "attributes" => $attributesData,
                    "otherDetails" => $this->filterManager->stripTags($otherProductDetails),
                    "fields" => $fields,
                    "keywords" => $targetKeywords,
                    "semanticKeywords" => $semanticKeywords,
                    "keywordAnalysisViews" => $keywordAnalysisViews
                ];

                if ($this->premium->isPremium()) {
                    if (!empty($images)) {
                        $textData["images"] = $images;
                    }
                    
                    $imageAltTextParams = $this->getAltTextParam(
                        $selectedImages,
                        $gallery,
                        $rewrite
                    );
                    if ($imageAltTextParams) {
                        $textData["imageAltTexts"] = $imageAltTextParams;
                    }
                }
                
                $customAudience = isset($selectedAudience['selected']) ?
                    array_shift($selectedAudience['selected']) : '';
                if ($customAudience) {
                    $textData['customAudience'] = $customAudience;
                }

                if (!empty($audience)) {
                    $textData['audiences'] = $audience;
                }

                if (in_array('custom', $tones)) {
                    $textData['customTone'] = $customTone;
                } else {
                    $textData['tones'] = $tones;
                }

                if ($style == 'custom') {
                    $textData['customStyle'] = $customStyle;
                } else {
                    $textData['style'] = $style;
                }
                
                $params = [
                    "type" => "Product",
                    "storeId" => $storeId,
                    "language" => $language,
                    "countries" => $this->helper->getSelectedCountries(),
                    "ipAddress" => $ipAddress,
                    "texts" => [$textData]
                ];

                $apiResponse = $this->apiManager->generate(json_encode($params));
            }
            
            $this->aiProductManager->saveDate($productId, $storeId, 'generated');
            $this->aiProductManager->saveDate($productId, $storeId, 'edited');

            $result = [
                'success' => true,
                'reviewed' => $this->productHelper->getReviewed($productId, $storeId),
                'response' => $apiResponse ?? [],
                'is_premium' => $this->premium->isPremium(),
                'params' => $params ?? []
            ];
        } catch (\WriteTextAI\WriteTextAI\Exception\UnauthorizedException $e) {
            $result = [
                'success' => false,
                'unauthorized' => true,
                'login_url' => $this->urlBuilder->getUrl('writetextai/setup/index'),
                'message' => $e->getMessage()
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'is_premium' => $this->premium->isPremium(),
                'message' => $e->getMessage(),
                'params' => $params ?? []
            ];
        }

        $response->setData($result);

        return $response;
    }

    /**
     * Generate alt text for images
     *
     * @param array $selectedImages
     * @param array $gallery
     * @param bool $rewrite
     *
     * @return array
     */
    private function getAltTextParam(
        $selectedImages,
        $gallery,
        $rewrite
    ) {
        if (empty($selectedImages)) {
            return [];
        }

        $images = [];
        foreach ($gallery as $image) {
            if (in_array($image['id'], $selectedImages)) {
                if ($rewrite && !$image['alt']) {
                    continue;
                }
                $images[] = [
                    "imageId" => $image['id'],
                    "currentValue" => $image['alt']
                ];
            }
        }

        return $images;
    }
}
