<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Edit;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use WriteTextAI\WriteTextAI\Model\Api\Session as ApiSession;
use WriteTextAI\WriteTextAI\Model\GeneratedText;
use Magento\Store\Model\ScopeInterface;
use WriteTextAI\WriteTextAI\Model\KeywordIdeaFiltersManager;
use WriteTextAI\WriteTextAI\Model\UserFieldsSelectedManager;
use WriteTextAI\WriteTextAI\Model\UserProductsManager;
use WriteTextAI\WriteTextAI\Model\GenerateOptionsManager;
use WriteTextAI\WriteTextAI\Model\AiProductManager;
use Magento\User\Model\ResourceModel\User\CollectionFactory;
use WriteTextAI\WriteTextAI\Helper\Data as HelperData;
use WriteTextAI\WriteTextAI\Helper\Fields as FieldHelper;
use WriteTextAI\WriteTextAI\Model\OptionSource\Filter\Stores as StoresFilter;
use Magento\Catalog\Model\Product\Visibility;
use WriteTextAI\WriteTextAI\Helper\Product as ProductHelper;
use WriteTextAI\WriteTextAI\Helper\Generate as GenerateHelper;
use WriteTextAI\WriteTextAI\ViewModel\Premium;
use WriteTextAI\WriteTextAI\ViewModel\Settings;
use WriteTextAI\WriteTextAI\ViewModel\UserFieldsSelected;
use WriteTextAI\WriteTextAI\Helper\ProductAttributes as ProductAttributesHelper;
use Magento\Catalog\Helper\Output as OutputHelper;
use WriteTextAI\WriteTextAI\Helper\ReviewStatus as ReviewHelper;

class Index extends Action
{
    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var ApiSession
     */
    protected $apiSession;
    
    /**
     * @var GeneratedText
     */
    protected $generatedText;

    /**
     * @var KeywordIdeaFiltersManager
     */
    protected $keywordIdeaFiltersManager;

    /**
     * @var UserFieldsSelected
     */
    protected $userFieldsSelected;

    /**
     * @var GenerateOptionsManager
     */
    protected $generateOptionsManager;

    /**
     * @var AiProductManager
     */
    protected $aiProductManager;

    /**
     * @var HelperData
     */
    protected $helper;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var FieldHelper
     */
    protected $fieldHelper;

    /**
     * @var array
     */
    protected $directivePatterns;

    /**
     * @var StoresFilter
     */
    protected $storesFilter;

    /**
     * @var ProductHelper
     */
    protected $productHelper;

    /**
     * @var GenerateHelper
     */
    protected $generateHelper;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * @var Settings
     */
    protected $settings;

    /**
     * @var ProductAttributesHelper
     */
    protected $productAttributesHelper;

    /**
     * @var OutputHelper
     */
    protected $outputHelper;

    /**
     * @var ReviewHelper
     */
    protected $reviewHelper;

    /**
     * @var CollectionFactory
     */
    protected $userCollectionFactory;

    /**
     * @var UserProductsManager
     */
    protected $userProductsManager;

    /**
     * Constructor
     *
     * @param Context $context
     * @param ApiManager $apiManager
     * @param ApiSession $apiSession
     * @param CollectionFactory $userCollectionFactory
     * @param GeneratedText $generatedText
     * @param KeywordIdeaFiltersManager $keywordIdeaFiltersManager
     * @param UserFieldsSelected $userFieldsSelected
     * @param UserProductsManager $userProductsManager
     * @param GenerateOptionsManager $generateOptionsManager
     * @param AiProductManager $aiProductManager
     * @param HelperData $helper
     * @param ScopeConfigInterface $scopeConfig
     * @param FieldHelper $fieldHelper
     * @param StoresFilter $storesFilter
     * @param ProductHelper $productHelper
     * @param GenerateHelper $generateHelper
     * @param Premium $premium
     * @param Settings $settings
     * @param ProductAttributesHelper $productAttributesHelper
     * @param OutputHelper $outputHelper
     * @param ReviewHelper $reviewHelper
     * @param array $directivePatterns
     */
    public function __construct(
        Context $context,
        ApiManager $apiManager,
        ApiSession $apiSession,
        CollectionFactory $userCollectionFactory,
        GeneratedText $generatedText,
        KeywordIdeaFiltersManager $keywordIdeaFiltersManager,
        UserFieldsSelected $userFieldsSelected,
        UserProductsManager $userProductsManager,
        GenerateOptionsManager $generateOptionsManager,
        AiProductManager $aiProductManager,
        HelperData $helper,
        ScopeConfigInterface $scopeConfig,
        FieldHelper $fieldHelper,
        StoresFilter $storesFilter,
        ProductHelper $productHelper,
        GenerateHelper $generateHelper,
        Premium $premium,
        Settings $settings,
        ProductAttributesHelper $productAttributesHelper,
        OutputHelper $outputHelper,
        ReviewHelper $reviewHelper,
        $directivePatterns = []
    ) {
        $this->apiManager = $apiManager;
        $this->apiSession = $apiSession;
        $this->userCollectionFactory = $userCollectionFactory;
        $this->generatedText = $generatedText;
        $this->keywordIdeaFiltersManager = $keywordIdeaFiltersManager;
        $this->userFieldsSelected = $userFieldsSelected;
        $this->userProductsManager = $userProductsManager;
        $this->generateOptionsManager = $generateOptionsManager;
        $this->aiProductManager = $aiProductManager;
        $this->helper = $helper;
        $this->scopeConfig = $scopeConfig;
        $this->fieldHelper = $fieldHelper;
        $this->directivePatterns = $directivePatterns;
        $this->storesFilter = $storesFilter;
        $this->productHelper = $productHelper;
        $this->generateHelper = $generateHelper;
        $this->premium = $premium;
        $this->settings = $settings;
        $this->productAttributesHelper = $productAttributesHelper;
        $this->outputHelper = $outputHelper;
        $this->reviewHelper = $reviewHelper;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);
        try {
            $productId = $this->getRequest()->getParam('id');
            $storeId = $this->getRequest()->getParam('store_id');

            $frontendStoreId = $this->productHelper->getFrontendStoreId($storeId);
            $currentUserId = $this->apiSession->getCurrentUser()->getId();

            $product = $this->productHelper->getProduct($productId, $storeId);
            $productName = $this->outputHelper->productAttribute(
                $product,
                $product->getName(),
                'name'
            );

            $filters = $this->keywordIdeaFiltersManager->getKeywordIdeasFilters(
                $productId,
                $storeId
            );

            $keywords = $this->getKeywords($productId, $storeId, $productName, $filters);

            $keywordsTexts = array_column($keywords['texts'], 'text');
            if (!empty($keywordsTexts)) {
                $audience = $this->getAudience($storeId, $productId, $keywordsTexts);
            }
            
            $localeCode =  $this->scopeConfig->getValue(
                'general/locale/code',
                ScopeInterface::SCOPE_STORE,
                $storeId
            );
            $language = str_replace('_', '-', $localeCode);

            $aiProduct = $this->aiProductManager->getProduct($productId, $storeId);
            
            $currDateTime = (new \DateTime())->format(\Magento\Framework\Stdlib\DateTime::DATETIME_PHP_FORMAT);

            $customOpenGraph = $this->helper->getCustomOpenGraph();

            $pageTitle = $this->fieldHelper->getAttributesValues($product, 'page_title');
            $pageDescription = $this->fieldHelper->getAttributesValues($product, 'page_description');
            $productDescription = $this->fieldHelper->getAttributesValues($product, 'product_description');
            $shortProductDescription = $this->fieldHelper->getAttributesValues(
                $product,
                'short_product_description'
            );
            $openGraph = $this->fieldHelper->getAttributesValues($product, 'open_graph');

            $tones = $this->generateHelper->updateDefault(
                $this->generateOptionsManager->getOptions('tones'),
                $this->userFieldsSelected->getUserFieldsSelected(
                    UserFieldsSelectedManager::ACTION_TYPE_TONES
                )
            );

            $userStyles = $this->userFieldsSelected->getUserFieldsSelected(
                UserFieldsSelectedManager::ACTION_TYPE_STYLE
            );
            $styles = $this->generateHelper->updateDefault(
                $this->generateOptionsManager->getOptions('styles'),
                $userStyles ? [$userStyles] : []
            );

            $audiences = $this->generateHelper->updateDefault(
                $this->generateOptionsManager->getOptions('audience'),
                $this->userFieldsSelected->getUserFieldsSelected(
                    UserFieldsSelectedManager::ACTION_TYPE_AUDIENCES
                )
            );

            $generatedText = $this->generatedText->getGenerated($productId, $storeId);

            $mapping = $this->helper->getMappingSettings();

            $result = [
                'success' => true,
                'is_premium' => $this->premium->getIsPremium(),
                'total_credits' => $this->premium->getTotalCredits(),
                'product_name' => $productName,
                'sku' => $product->getSku(),
                'product_description' => $productDescription,
                'product_short_description' => $shortProductDescription,
                'product_meta_title' => $pageTitle,
                'product_meta_description' => $pageDescription,
                'open_graph' => $customOpenGraph ? $openGraph : '',
                'use_default' => [
                    'page_title' => $this->fieldHelper->isUsingDefaultValues(
                        $product,
                        $storeId,
                        $mapping['page_title']
                    ),
                    'page_description' => $this->fieldHelper->isUsingDefaultValues(
                        $product,
                        $storeId,
                        $mapping['page_description']
                    ),
                    'product_description' => $this->fieldHelper->isUsingDefaultValues(
                        $product,
                        $storeId,
                        $mapping['product_description']
                    ),
                    'short_product_description' => $this->fieldHelper->isUsingDefaultValues(
                        $product,
                        $storeId,
                        $mapping['short_product_description']
                    ),
                    'open_graph' => isset($mapping['open_graph']) ?
                        $this->fieldHelper->isUsingDefaultValues($product, $storeId, $mapping['open_graph']) : false
                ],
                'textfields_statuses' => $this->generatedText->getTextFieldsStatuses($productId, $storeId),
                'link_preview_url' => $product->setStoreId($frontendStoreId)->getUrlInStore(),
                'keywords' => $keywords,
                'product_attributes' => $this->productAttributesHelper->getProductAttributes($product),
                'product_attributes_selected' => $this->productAttributesHelper
                    ->getProductAttributesSelected($aiProduct),
                'other_details_checked' => (bool) $aiProduct->getOtherDetailsChecked(),
                'other_details' => $aiProduct->getOtherDetails(),
                'preview_changes_url' => $product->setStoreId($frontendStoreId)->getUrlInStore([
                    '_query' => [
                        'wta-preview' => base64_encode($currentUserId)
                    ]
                ]),
                'text_fields' => $this->generatedText->getTextFields($productId, $storeId, false, $generatedText),
                'reviewed' => $this->productHelper->getReviewed($productId, $storeId, $generatedText),
                'refresh' => $aiProduct->getId() ? ($aiProduct->getRefreshAt() ?
                    $aiProduct->getRefreshAt() <= $currDateTime : false) : false,
                'users' => $this->getUsers(),
                'product_id' => $productId,
                'visibility' => $product->getVisibility() != Visibility::VISIBILITY_NOT_VISIBLE,
                'store_id' => $storeId,
                'store_scope' => $this->productHelper->getStoreScope($storeId),
                'stores' => array_values($this->storesFilter->toOptionArray()),
                'selected_audience' => $audience['selected'] ?? [],
                'suggested_audience' => $audience['values'] ?? [],
                'guide_steps' => filter_var($this->userFieldsSelected->getUserFieldsSelected(
                    UserFieldsSelectedManager::ACTION_TYPE_GUIDE_STEPS
                ), FILTER_VALIDATE_BOOLEAN),
                'selected_fields' => $this->userFieldsSelected->getUserFieldsSelected(
                    UserFieldsSelectedManager::ACTION_TYPE_SINGLE_EDITOR_FIELDS
                ),
                'highlighted' => $this->userFieldsSelected->getUserFieldsSelected(
                    UserFieldsSelectedManager::ACTION_TYPE_HIGHLIGHTED
                ),
                'highlight_pronouns' => $this->userFieldsSelected->getUserFieldsSelected(
                    UserFieldsSelectedManager::ACTION_TYPE_HIGHLIGHT_PRONOUNS
                ),
                'keyword_ideas_filters' => $filters,
                'custom_tone' => $this->userFieldsSelected->getUserFieldsSelected(
                    UserFieldsSelectedManager::ACTION_TYPE_CUSTOM_TONE
                ),
                'custom_style' => $this->userFieldsSelected->getUserFieldsSelected(
                    UserFieldsSelectedManager::ACTION_TYPE_CUSTOM_STYLE
                ),
                'gallery' => $this->userFieldsSelected->getUserFieldsSelected(
                    UserFieldsSelectedManager::ACTION_TYPE_GALLERY
                ),
                'tones' => $tones,
                'styles' => $styles,
                'audience' => $audiences,
                'formal_language_support' => $this->settings->getSettings('formalLanguageSupport'),
                'formal_languages' => $this->settings->getFormalLanguages(explode('_', $localeCode)[0]),
                'disallowed_combinations' => $this->settings->getSettings('disallowedCombinations'),
                'desc_min' => $this-> helper->getDescriptionSettings('min'),
                'desc_max' => $this-> helper->getDescriptionSettings('max'),
                'exc_min' => $this->helper->getExcerptSettings('min'),
                'exc_max' => $this->helper->getExcerptSettings('max'),
                'credits' => $this->settings->getSettings('credits'),
                'language' => $this->getLanguage($localeCode),
                'directiveRegex' => $this->getDirectivePatterns(),
                'product_description_directive_exists' => $this->fieldHelper->isDirectiveExists(
                    $product,
                    'product_description'
                ),
                'product_short_description_directive_exists' => $this->fieldHelper->isDirectiveExists(
                    $product,
                    'short_product_description'
                ),
                'images' => $this->productHelper->getImages($product, $storeId, $language),
                'review_status' => $this->reviewHelper->getReviewStatus($productId, $storeId),
                'images_selected' => $this->userProductsManager->getUserProductsImagesSelected(
                    $productId
                ),
                'images_selected_available' => $this->userProductsManager->getUserProductsImagesSelectedAvailable(
                    $productId
                ),
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => __(
                    'A system error has occurred. Please try again. If the issue persists,'
                    . ' please contact our support team at support@writetext.ai.'
                ),
                'error' => $e->getMessage(),
                'trace' => $e->getTrace()
            ];
        }

        $response->setData($result);

        return $response;
    }

    /**
     * Get language
     *
     * @param string $localeCode
     * @return string
     */
    public function getLanguage($localeCode)
    {
        $languageCode = explode('_', $localeCode)[0];
        $languageCode = strtoupper($languageCode);

        return $languageCode;
    }

    /**
     * Get keywords
     *
     * @param int $productId
     * @param int $storeId
     * @param string $productName
     * @param array $filters
     * @return array
     */
    protected function getKeywords($productId, $storeId, $productName, $filters)
    {
        $keywords = $this->apiManager->getSemanticKeywords($storeId, $productId, $productName);
        $selected = $keywords['selected'] ?? [];
        $texts = $keywords['texts'] ?? [];
        
        $ideas = $this->getIdeas($productId, $storeId, array_column($texts, 'text'), $filters);

        $data = array_map(function ($keyword) {
            unset($keyword['searchVolume']);
            unset($keyword['difficulty']);
            return $keyword;
        }, $ideas);
        $this->mergeArraysByProperty($texts, $data, 'text');

        $result = [
            'selected' => $selected,
            'texts' => $texts
        ];

        return $result;
    }

    /**
     * Merge arrays by property
     *
     * @param array $array1
     * @param array $array2
     * @param string $matchingProperty
     * @return void
     */
    private function mergeArraysByProperty(&$array1, $array2, $matchingProperty)
    {
        $tempArray = [];
    
        foreach ($array2 as $item) {
            $tempArray[$item[$matchingProperty]] = $item;
        }
    
        foreach ($array1 as &$item) {
            $id = $item[$matchingProperty];
            if (isset($tempArray[$id])) {
                $item += $tempArray[$id];
            }
        }
    }

    /**
     * Get ideas
     *
     * @param int $productId
     * @param int $storeId
     * @param array $keywords
     * @param array $filters
     *
     * @return array
     */
    protected function getIdeas($productId, $storeId, $keywords, $filters)
    {
        $location           = $filters['location'];
        $filterByCompetition = $filters['competition'] ?? [''];
        $filterBySearchVolumeMinimum = $filters['search_volume_minimum'];
        $filterBySearchVolumeMaximum = $filters['search_volume_maximum'];
        $competitionSort    = $filters['competition_sort'];
        $volumeSort         = $filters['volume_sort'];
        $keywordSort        = $filters['keyword_sort'];
        $page               = 1;
        $pageSize           = 10;
        $noGenerate         = true;
        $refresh            = false;

        $ideas = $this->apiManager->getKeywordIdeas(
            $storeId,
            $productId,
            $keywords,
            $location,
            $filterByCompetition ?? [''],
            $filterBySearchVolumeMinimum,
            $filterBySearchVolumeMaximum,
            $competitionSort,
            $volumeSort,
            $keywordSort,
            $page,
            $pageSize,
            $refresh,
            $noGenerate
        );

        $data = [];
        if (isset($ideas['keywords'])) {
            foreach ($ideas['keywords'] as $keyword) {
                $data[] = [
                    'text' => $keyword['keyword'],
                    'searchVolume' => $keyword['search_volume'] ?? __('No data'),
                    'difficulty' => $keyword['competition'] ?? __('No data')
                ];
            }
        }

        return $data;
    }
    
    /**
     * Get users
     *
     * @return array
     */
    private function getUsers()
    {
        $collection = $this->userCollectionFactory->create();
        $users = [];

        foreach ($collection as $user) {
            $users[] = [
                'id' => $user->getUserId(),
                'name' => $user->getFirstname() . ' ' . $user->getLastname()
            ];
        }

        return $users;
    }

    /**
     * Get audience
     *
     * @param int $storeId
     * @param int $productId
     * @param array $keywords
     * @return array
     */
    private function getAudience($storeId, $productId, $keywords)
    {
        return $this->apiManager->getSuggestedAudience($storeId, $productId, $keywords);
    }

    /**
     * Get directives regular expression
     *
     * @return array
     * @see \Magento\Framework\Filter\Template
     */
    private function getDirectivePatterns()
    {
        $patterns = [];
        foreach ($this->directivePatterns as $pattern) {
            $pattern = ltrim($pattern, '/');
            $pattern = rtrim($pattern, '/si');
            $patterns[] = $pattern;
        }

        $patterns[] = FieldHelper::MEDIA_PATTERN;
        return $patterns;
    }
}
