<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Edit\Keyword;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use WriteTextAI\WriteTextAI\Model\KeywordIdeaFiltersManager;
use WriteTextAI\WriteTextAI\ViewModel\Premium;

class Add extends Action
{
    public const ADMIN_RESOURCE = 'WriteTextAI_WriteTextAI::add_keywords';

    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var KeywordIdeaFiltersManager
     */
    protected $keywordIdeaFiltersManager;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * Constructor
     *
     * @param Context $context
     * @param ApiManager $apiManager
     * @param KeywordIdeaFiltersManager $keywordIdeaFiltersManager
     * @param Premium $premium
     */
    public function __construct(
        Context $context,
        ApiManager $apiManager,
        KeywordIdeaFiltersManager $keywordIdeaFiltersManager,
        Premium $premium
    ) {
        $this->apiManager = $apiManager;
        $this->keywordIdeaFiltersManager = $keywordIdeaFiltersManager;
        $this->premium = $premium;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        if (!$this->premium->getIsPremium()) {
            $response->setData([
                'success' => false,
                'is_premium' => false,
                'total_credits' => $this->premium->getTotalCredits(),
                'message' => __('Premium is required to do this action.')
            ]);

            return $response;
        }

        try {
            $keywords = $this->getRequest()->getParam('keywords');
            $keyword = $this->getRequest()->getParam('keyword');
            $productId = $this->getRequest()->getParam('product_id');
            $storeId = $this->getRequest()->getParam('store_id');

            $newKeywords = $keywords;
            array_push($newKeywords, $keyword);
            $semanticKeywords = $this->apiManager->getSemanticKeywords($storeId, $productId, $newKeywords);
            /**
             * commented to temporary fix too many requets
             */
            // $ideas = $this->getIdeas($productId, $storeId, $newKeywords, $keyword);

            $texts = $semanticKeywords['texts'];
            $textIndex = array_search($keyword, array_column($texts, 'text'));
            $data = [
                'text' => $keyword,
                'values' => $texts[$textIndex]['values'] ?? []
            ];

            /**
             * commented to temporary fix too many requets
             */
            // if (isset($ideas['searchVolume'])) {
            //     $data['searchVolume'] = $ideas['searchVolume'];
            // }
            // if (isset($ideas['difficulty'])) {
            //     $data['difficulty'] = $ideas['difficulty'];
            // }

            $keywordsTexts = array_column($texts, 'text');
            if (!empty($keywordsTexts)) {
                $audience = $this->apiManager->getSuggestedAudience($storeId, $productId, $keywordsTexts);
            }
            
            $result = [
                'success' => true,
                'data' => $data,
                'selected_audience' => $audience['selected'] ?? [],
                'suggested_audience' => $audience['values'] ?? []
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage(),
                'api_response' => $semanticKeywords
            ];
        }

        $result['is_premium'] = $this->premium->getIsPremium();
        $result['total_credits'] = $this->premium->getTotalCredits();

        $response->setData($result);

        return $response;
    }

    /**
     * Get ideas
     *
     * @param int $productId
     * @param int $storeId
     * @param array $newKeywords
     * @param string $inputKeyword
     *
     * @return array
     */
    protected function getIdeas($productId, $storeId, $newKeywords, $inputKeyword)
    {
        $params             = $this->getRequest()->getParams();
        $location           = $params['location'];
        $filterByCompetition = $params['competition_filter'] ?? [''];
        $filterBySearchVolumeMinimum = $params['search_volume_minimum'];
        $filterBySearchVolumeMaximum = $params['search_volume_maximum'];
        $competitionSort    = $params['competition_sort'];
        $volumeSort         = $params['volume_sort'];
        $keywordSort        = $params['keyword_sort'];
        $page               = $params['page'];
        $pageSize           = $params['page_size'];
        $noGenerate         = filter_var($params['nogenerate'], FILTER_VALIDATE_BOOLEAN);
        $refresh            = filter_var($params['refresh'], FILTER_VALIDATE_BOOLEAN);

        $ideas = $this->apiManager->getKeywordIdeas(
            $storeId,
            $productId,
            $newKeywords,
            $location,
            $filterByCompetition ?? [''],
            $filterBySearchVolumeMinimum,
            $filterBySearchVolumeMaximum,
            $competitionSort,
            $volumeSort,
            $keywordSort,
            $page,
            $pageSize,
            $refresh,
            $noGenerate
        );

        $result = [];
        
        if (isset($ideas['keywords'])) {
            foreach ($ideas['keywords'] as $keyword) {
                if ($keyword['keyword'] == $inputKeyword) {
                    $result = [
                        'keyword' => $keyword['keyword'],
                        'searchVolume' => $keyword['search_volume'],
                        'difficulty' => $keyword['competition']
                    ];
                }
            }
        }

        $this->keywordIdeaFiltersManager->saveProductKeywordIdeasFilters(
            $storeId,
            $productId,
            $location,
            $filterByCompetition,
            $filterBySearchVolumeMinimum,
            $filterBySearchVolumeMaximum,
            $competitionSort,
            $volumeSort,
            $keywordSort
        );

        return $result;
    }
}
