<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Edit\Keyword;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use WriteTextAI\WriteTextAI\ViewModel\Premium;

class Delete extends Action
{
    public const ADMIN_RESOURCE = 'WriteTextAI_WriteTextAI::add_keywords';

    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * Constructor
     *
     * @param Context $context
     * @param ApiManager $apiManager
     * @param Premium $premium
     */
    public function __construct(
        Context $context,
        ApiManager $apiManager,
        Premium $premium
    ) {
        $this->apiManager = $apiManager;
        $this->premium = $premium;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        if (!$this->premium->getIsPremium()) {
            $response->setData([
                'success' => false,
                'is_premium' => false,
                'total_credits' => $this->premium->getTotalCredits(),
                'message' => __('Premium is required to do this action.')
            ]);

            return $response;
        }
        
        try {
            $remainingKeywords = $this->getRequest()->getParam('remaining_keywords');
            $productId = $this->getRequest()->getParam('product_id');
            $storeId = $this->getRequest()->getParam('store_id');
            $clear = count($remainingKeywords) > 1 ? 'false' : 'true';

            $semanticKeywords = $this->apiManager->getSemanticKeywords(
                $storeId,
                $productId,
                array_column($remainingKeywords, 'text'),
                $clear
            );

            $keywordsTexts = array_column($remainingKeywords, 'text');
            if (!empty($keywordsTexts)) {
                $audience = $this->apiManager->getSuggestedAudience($storeId, $productId, $keywordsTexts);
            }
            
            $result = [
                'success' => true,
                'selected' => $semanticKeywords['selected'],
                'selected_audience' => $audience['selected'] ?? [],
                'suggested_audience' => $audience['values'] ?? []
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage()
            ];
        }

        $result['is_premium'] = $this->premium->getIsPremium();
        $result['total_credits'] = $this->premium->getTotalCredits();

        $response->setData($result);

        return $response;
    }
}
