<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Edit\Mark;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use WriteTextAI\WriteTextAI\Model\GeneratedText;
use WriteTextAI\WriteTextAI\Helper\Data;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;
use WriteTextAI\WriteTextAI\Model\OptionSource\Filter\Fields;
use WriteTextAI\WriteTextAI\ViewModel\Premium;

class Reviewed extends Action
{
    public const ADMIN_RESOURCE = 'WriteTextAI_WriteTextAI::review';

    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var GeneratedText
     */
    protected $generatedText;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * Constructor
     *
     * @param Context $context
     * @param ApiManager $apiManager
     * @param GeneratedText $generatedText
     * @param Data $helper
     * @param ScopeConfigInterface $scopeConfig
     * @param Premium $premium
     */
    public function __construct(
        Context $context,
        ApiManager $apiManager,
        GeneratedText $generatedText,
        Data $helper,
        ScopeConfigInterface $scopeConfig,
        Premium $premium
    ) {
        $this->apiManager = $apiManager;
        $this->generatedText = $generatedText;
        $this->helper = $helper;
        $this->scopeConfig = $scopeConfig;
        $this->premium = $premium;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);
        try {
            $productId = $this->getRequest()->getParam('product_id');
            $storeId = $this->getRequest()->getParam('store_id');
            $reviewed = filter_var($this->getRequest()->getParam('reviewed'), FILTER_VALIDATE_BOOLEAN);

            $ids = $this->generatedText->getTextIds($productId, $storeId);
            $textfields = $this->generatedText->getTextFields($productId, $storeId, true);

            if (!empty($ids)) {
                $fields = [];
                $fieldMappings = [
                    'page_title' => Fields::PAGE_TITLE,
                    'page_description' => Fields::PAGE_DESCRIPTION,
                    'product_description' => Fields::PRODUCT_DESCRIPTION,
                    'short_product_description' => Fields::EXCERPT,
                    'open_graph' => Fields::OPEN_GRAPH,
                ];

                foreach ($fieldMappings as $key => $value) {
                    if (isset($ids[$key]) &&
                        !$textfields['status'][$key] &&
                        ($key != 'open_graph' || $this->helper->getCustomOpenGraph())
                    ) {
                        $fields[] = [
                            "textId" => $ids[$key],
                            "field" => $value,
                            "platform" => "Magento",
                            "value" => $textfields[$key],
                            "reviewed" => $reviewed,
                            "publish" => false
                        ];
                    }
                }
    
                $locale =  $this->scopeConfig->getValue(
                    'general/locale/code',
                    ScopeInterface::SCOPE_STORE,
                    $storeId
                );
    
                $params = [
                    'type' => 'Product',
                    'language' => str_replace('_', '-', $locale),
                    'recordId' => $productId,
                    'storeId' => $storeId,
                    'fields' => $fields
                ];
    
                $apiResponse = $this->apiManager->save($params);
    
                $result = [
                    'success' => true,
                    'response' => $apiResponse,
                    'params' => $params
                ];
            } else {
                $result = [
                    'success' => false,
                    'message' => __("You haven't generated any text yet.")
                ];
            }
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage()
            ];
        }

        $result['is_premium'] = $this->premium->getIsPremium();
        $result['total_credits'] = $this->premium->getTotalCredits();

        $response->setData($result);

        return $response;
    }
}
