<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Edit;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use Magento\Catalog\Model\ResourceModel\Product\Action as ProductAction;
use WriteTextAI\WriteTextAI\Model\GeneratedText;
use WriteTextAI\WriteTextAI\Helper\Data;
use WriteTextAI\WriteTextAI\Model\AiProductManager;
use WriteTextAI\WriteTextAI\Helper\Fields as FieldHelper;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use WriteTextAI\WriteTextAI\Helper\Product as ProductHelper;
use WriteTextAI\WriteTextAI\ViewModel\Premium;
use WriteTextAI\WriteTextAI\Helper\Fields as FieldsHelper;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use WriteTextAI\WriteTextAI\Model\OptionSource\Filter\Fields;
use WriteTextAI\WriteTextAI\ViewModel\Settings;
use WriteTextAI\WriteTextAI\Helper\ReviewStatus as ReviewHelper;
use WriteTextAI\WriteTextAI\Model\ReviewStatus;

class Transfer extends Action
{
    public const ADMIN_RESOURCE = 'WriteTextAI_WriteTextAI::transfer';
    
    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var ProductAction
     */
    protected $productAction;

    /**
     * @var GeneratedText
     */
    protected $generatedText;

    /**
     * @var AiProductManager
     */
    protected $aiProductManager;

    /**
     * @var FieldHelper
     */
    protected $fieldHelper;

    /**
     * @var ProductRepositoryInterface
     */
    protected $productRepository;

    /**
     * @var ProductHelper
     */
    protected $productHelper;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * @var FieldsHelper
     */
    protected $fieldsHelper;

    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var Settings
     */
    protected $settings;

    /**
     * @var ReviewHelper
     */
    protected $reviewHelper;

    /**
     * @var ReviewStatus
     */
    protected $reviewStatus;

    /**
     * Constructor
     *
     * @param Context $context
     * @param Data $helper
     * @param ProductAction $productAction
     * @param GeneratedText $generatedText
     * @param AiProductManager $aiProductManager
     * @param FieldHelper $fieldHelper
     * @param ProductRepositoryInterface $productRepository
     * @param ProductHelper $productHelper
     * @param Premium $premium
     * @param FieldsHelper $fieldsHelper
     * @param ApiManager $apiManager
     * @param Settings $settings
     * @param ReviewHelper $reviewHelper
     * @param ReviewStatus $reviewStatus
     */
    public function __construct(
        Context $context,
        Data $helper,
        ProductAction $productAction,
        GeneratedText $generatedText,
        AiProductManager $aiProductManager,
        FieldHelper $fieldHelper,
        ProductRepositoryInterface $productRepository,
        ProductHelper $productHelper,
        Premium $premium,
        FieldsHelper $fieldsHelper,
        ApiManager $apiManager,
        Settings $settings,
        ReviewHelper $reviewHelper,
        ReviewStatus $reviewStatus
    ) {
        $this->helper = $helper;
        $this->productAction = $productAction;
        $this->generatedText = $generatedText;
        $this->aiProductManager = $aiProductManager;
        $this->fieldHelper = $fieldHelper;
        $this->productRepository = $productRepository;
        $this->productHelper = $productHelper;
        $this->premium = $premium;
        $this->fieldsHelper = $fieldsHelper;
        $this->apiManager = $apiManager;
        $this->settings = $settings;
        $this->reviewHelper = $reviewHelper;
        $this->reviewStatus = $reviewStatus;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        try {
            $storeFilter = $this->getRequest()->getPost('store_id');
            $productId = $this->getRequest()->getPost('product_id');
            $selectedStores = $this->getRequest()->getPost('selected_stores', []);
            $selectedFields = $this->getRequest()->getPost('selected_fields', []);
            $field = $this->getRequest()->getParam('field');
            $value = $this->getRequest()->getParam('value');
            
            try {
                $product = $this->productRepository->getById($productId, false, $storeFilter);
            } catch (NoSuchEntityException $e) {
                $result = [
                    'success' => false,
                    'is_premium' => $this->premium->isPremium(),
                    'total_credits' => $this->premium->getTotalCredits(),
                    'message' => $e->getMessage()
                ];
            }

            if (!empty($selectedFields) || ($field && $value)) {
                $mappingSettings = $this->helper->getMappingSettings();

                if (!empty($selectedFields)) {
                    $pageTitle = $this->getRequest()->getParam('page_title');
                    $pageDescription = $this->getRequest()->getParam('page_description');
                    $productDescription = $this->getRequest()->getParam('product_description');
                    $productShortDescription = $this->getRequest()->getParam('product_short_description');
                    $openGraph = $this->getRequest()->getParam('open_graph');

                    /**
                     * comment for now, this is the reason for mismatch render during transfer
                     */
                    // if (strpos($productDescription, "<br />") !== false) {
                    //     $productDescription = preg_replace("/(\r\n|\n\r|\n|\r)/", "<br />", $productDescription);
                    // }

                    // if (strpos($productShortDescription, "<br />") !== false) {
                    //     $productShortDescription = preg_replace(
                    //         "/(\r\n|\n\r|\n|\r)/",
                    //         "<br />",
                    //         $productShortDescription
                    //     );
                    // }

                    $this->generatedText->saveText(
                        $storeFilter,
                        $productId,
                        $pageTitle,
                        $pageDescription,
                        $productDescription,
                        $productShortDescription,
                        $openGraph,
                        $selectedFields
                    );
                
                    $fields = [
                        'page_title' => $pageTitle,
                        'page_description' => $pageDescription,
                        'product_description' => $productDescription,
                        'short_product_description' => $productShortDescription,
                        'open_graph' => $openGraph
                    ];

                    if ($this->helper->getCustomOpenGraph() && !empty($openGraph)) {
                        $fields['open_graph'] = $openGraph;
                    }
                    $toReview = [];
                    foreach ($selectedFields as $selectedField) {
                        if (!empty($fields[$selectedField])) {
                            $toReview[$selectedField] = $fields[$selectedField];
                        }
                    }
                } else {
                    /**
                     * comment for now, this is the reason for mismatch render during transfer
                     */
                    // if (in_array($field, ['product_description', 'short_product_description'])) {
                    //     if (strpos($value, "<br />") !== false) {
                    //         $value = preg_replace("/(\r\n|\n\r|\n|\r)/", "<br />", $value);
                    //     }
                    // }

                    $selectedFields = [$field];
                    $this->generatedText->saveText(
                        $storeFilter,
                        $productId,
                        $field === 'page_title' ? $value : null,
                        $field === 'page_description' ? $value : null,
                        $field === 'product_description' ? $value : null,
                        $field === 'short_product_description' ? $value : null,
                        $field === 'open_graph' ? $value : null,
                        $selectedFields
                    );

                    $fields = [
                        $field => $value
                    ];
                }
        
                foreach ($selectedStores as $storeId) {
                    $toUpdate = [];
                    foreach ($fields as $field => $value) {
                        if (isset($mappingSettings[$field]) && in_array($field, $selectedFields)) {
                            $toUpdate[$mappingSettings[$field]] = $value;
                        }
                    }
            
                    $this->productAction->updateAttributes(
                        [$productId],
                        $toUpdate,
                        $storeId
                    );
                    
                    $this->aiProductManager->saveDate($productId, $storeId, 'transferred');
                }

                $resultFields = [];
                $directives = [];

                try {
                    $product = $this->productRepository->getById($productId, false, $storeFilter, true);
                } catch (NoSuchEntityException $e) {
                    $result = [
                        'success' => false,
                        'is_premium' => $this->premium->isPremium(),
                        'message' => $e->getMessage()
                    ];
                }

                foreach ($fields as $field => $value) {
                    $fieldValue = $this->fieldHelper->getAttributesValues($product, $field);
                    $resultFields[$field] = $fieldValue;
                    $resultFields[$field . '_char'] = $fieldValue ? strlen($fieldValue) : 0;
                    $resultFields[$field . '_words'] = $fieldValue ? count(array_filter(
                        explode(' ', $fieldValue),
                        function ($n) {
                            return $n !== '';
                        }
                    )) : 0;

                    if (in_array($field, ['product_description', 'short_product_description'])) {
                        $directives[$field] = $this->fieldHelper->isDirectiveExists($product, $field);
                    }
                }
            }

            $language = $this->fieldsHelper->getStoreLanguage($storeFilter);

            $this->transferImageAltTexts(
                $productId,
                $selectedStores,
                $language
            );

            try {
                $product = $this->productRepository->getById($productId, false, $storeFilter, true);
            } catch (NoSuchEntityException $e) {
                $result = [
                    'success' => false,
                    'message' => $e->getMessage()
                ];
            }

            if (!empty($fields)) {
                /**
                 * $toReview is set if transfered from transfer all button
                 */
                if (isset($toReview) && !empty($toReview)) {
                    $fields = $toReview;
                }
                $this->reviewStatus->updateReview(array_keys($fields), $productId, $selectedStores);
            }
            
            $useDefault = $this->getUseDefault($storeFilter, $product, $mappingSettings);

            $updatedReview = $this->reviewHelper->getReviewStatus($productId, $storeFilter);
            $result = [
                'success' => true,
                'reviewed' => $this->productHelper->getReviewed($productId, $storeFilter),
                'fields' => $resultFields ?? [],
                'directives' => $directives ?? [],
                'images' => $this->productHelper->getImages($product, $storeFilter, $language),
                'updatedReview' => $updatedReview,
                'use_default' => $useDefault
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage()
            ];
        }
        
        $result['is_premium'] = $this->premium->getIsPremium();
        $result['total_credits'] = $this->premium->getTotalCredits();

        $response->setData($result);

        return $response;
    }

    /**
     * Get use default
     *
     * @param int $storeId
     * @param \Magento\Catalog\Api\Data\ProductInterface $product
     * @param array $mappingSettings
     *
     * @return array
     */
    public function getUseDefault($storeId, $product, $mappingSettings)
    {
        $useDefault = [
            'page_title' => $this->fieldHelper->isUsingDefaultValues(
                $product,
                $storeId,
                $mappingSettings['page_title']
            ),
            'page_description' => $this->fieldHelper->isUsingDefaultValues(
                $product,
                $storeId,
                $mappingSettings['page_description']
            ),
            'product_description' => $this->fieldHelper->isUsingDefaultValues(
                $product,
                $storeId,
                $mappingSettings['product_description']
            ),
            'short_product_description' => $this->fieldHelper->isUsingDefaultValues(
                $product,
                $storeId,
                $mappingSettings['short_product_description']
            ),
            'open_graph' => isset($mappingSettings['open_graph']) ?
                $this->fieldHelper->isUsingDefaultValues($product, $storeId, $mappingSettings['open_graph']) : false
        ];

        return $useDefault;
    }

    /**
     * Transfer image alt texts
     *
     * @param int $productId
     * @param array $selectedStores
     * @param string $language
     * @return void
     */
    public function transferImageAltTexts($productId, $selectedStores, $language)
    {
        $selectedImages = $this->getRequest()->getParam('selected_images', []);
        $images = $this->getRequest()->getParam('images', []);
        
        if (empty($selectedImages)) {
            return;
        }

        $rules = $this->settings->getSettings('rules');
        $selectedImagesChunk = array_chunk($selectedImages, $rules['maxImageAltTextPerRequest']);

        foreach ($selectedStores as $storeId) {
            foreach ($selectedImagesChunk as $selectedImageChunk) {
                try {
                    $apiImages = $this->apiManager->getImageList([
                        "language" => $language,
                        "images" => $selectedImageChunk,
                        "storeId" => $storeId,
                        "includeUpdateHistory" => "true"
                    ]);
                } catch (\WriteTextAI\WriteTextAI\Exception\ApiException $e) {
                    if ($e->getCode() === 404) {
                        $apiImages = [];
                    }
                }

                foreach ($images as $image) {
                    if (!in_array($image['id'], $selectedImageChunk)
                        || empty($image['writetext_alt'])) {
                        continue;
                    }
                    
                    $product = $this->productRepository->getById($productId, true, $storeId);
                    $mediaGalleryEntries = $product->getMediaGalleryEntries();
                    
                    foreach ($mediaGalleryEntries as $mediaGalleryEntry) {
                        if ($mediaGalleryEntry->getId() == $image['id']) {
                            $mediaGalleryEntry->setLabel($image['writetext_alt']);
                            break;
                        }
                    }

                    $product->setMediaGalleryEntries($mediaGalleryEntries);
                    $this->productRepository->save($product);
                    $this->aiProductManager->saveDate($productId, $storeId, 'transferred');
                    $filteredImages = array_filter(
                        $apiImages['images'],
                        function ($apiImage) use ($image) {
                            return $apiImage['imageId'] == $image['id'];
                        }
                    );
                    
                    if (!empty($filteredImages)) {
                        $apiImage = reset($filteredImages);
                        if (isset($apiImage['altText']['id'])) {
                            $saveParams = [
                                "storeId" => $storeId,
                                "language" => $language,
                                "imageId" => $image['id'],
                                "textId" => $apiImage['altText']['id'],
                                "value" => $image['writetext_alt'],
                                "publish" => true
                            ];
                            $this->apiManager->updateAltText(json_encode($saveParams));
                        }
                    }
                }
            }
        }
    }
}
