<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Grid;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use Magento\Config\Model\Config\Factory;
use WriteTextAI\WriteTextAI\Helper\Data;
use WriteTextAI\WriteTextAI\Helper\Settings;

class Country extends Action
{
    /**
     * @var Factory
     */
    protected $configFactory;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * Constructor
     *
     * @param Context $context
     * @param Factory $configFactory
     * @param Data $helper
     */
    public function __construct(
        Context $context,
        Factory $configFactory,
        Data $helper
    ) {
        $this->configFactory = $configFactory;
        $this->helper = $helper;
        parent::__construct($context);
    }

    /**
     * Execute action
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);
        $result = [
            'success' => true
        ];

        try {
            $configData = [
                'section' => Settings::SECTION_NAME
            ];

            $countries = $this->getRequest()->getPost('country_id');
            $configData['groups']['country']['fields']['selected_countries']['value'] = $countries;

            /** @var \Magento\Config\Model\Config $configModel */
            $configModel = $this->configFactory->create(['data' => $configData]);
            $configModel->save();

            $this->helper->flushConfigCache();

            $result = [
                'success' => true,
                'countries' => $countries
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage(),
                'post' => $this->getRequest()->getPost('country_id')
            ];
        }

        $response->setData($result);

        return $response;
    }
}
