<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Grid\Transfer;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use WriteTextAI\WriteTextAI\Model\BulkTransferRequestsManager;

class Cancel extends Action
{
    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var BulkTransferRequestsManager
     */
    protected $bulkTransferRequestsManager;

    /**
     * Constructor
     *
     * @param Context $context
     * @param ApiManager $apiManager
     * @param BulkTransferRequestsManager $bulkTransferRequestsManager
     */
    public function __construct(
        Context $context,
        ApiManager $apiManager,
        BulkTransferRequestsManager $bulkTransferRequestsManager
    ) {
        $this->apiManager = $apiManager;
        $this->bulkTransferRequestsManager = $bulkTransferRequestsManager;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);
        try {
            $id = $this->getRequest()->getParam('id');
            $user = $this->getRequest()->getParam('user');
            $this->bulkTransferRequestsManager->delete($user);
            $result = [
                'success' => true,
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage()
            ];
        }

        $response->setData($result);

        return $response;
    }
}
