<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

declare(strict_types=1);

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Settings;

use WriteTextAI\WriteTextAI\Model\Api\CurlBuilder;
use WriteTextAI\WriteTextAI\Helper\Data;
use Magento\Framework\Event\ManagerInterface as EventManager;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultInterface;
use WriteTextAI\WriteTextAI\Model\Api\Session as ApiSession;

class Index extends \Magento\Backend\App\Action
{
    /**
     * @var CurlBuilder
     */
    protected $curlBuilder;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var EventManager
     */
    protected $eventManager;

    /**
     * @var ApiSession
     */
    protected $apiSession;

    /**
     * Constructor
     *
     * @param Context $context
     * @param CurlBuilder $curlBuilder
     * @param Data $helper
     * @param EventManager $eventManager
     * @param ApiSession $apiSession
     */
    public function __construct(
        Context $context,
        CurlBuilder $curlBuilder,
        Data $helper,
        EventManager $eventManager,
        ApiSession $apiSession
    ) {
        $this->curlBuilder = $curlBuilder;
        $this->helper = $helper;
        $this->eventManager = $eventManager;
        $this->apiSession = $apiSession;
        parent::__construct($context);
    }

    /**
     * Index action
     *
     * @return ResultInterface
     */
    public function execute()
    {
        $resultRedirect = $this->resultRedirectFactory->create();

        try {
            if (!$this->isLoggedIn()) {
                return $resultRedirect->setPath('writetextai/setup/index');
            } else {
                $this->eventManager->dispatch('writetextai_prepare_settings');
                $locale = $this->apiSession->getCurrentUser()->getInterfaceLocale();
                if (strpos($locale, 'en_') !== 0) {
                    $this->messageManager->addNotice(
                        __("Translation of extension help text is ongoing. Please stay tuned.")
                    );
                }
                
                return $resultRedirect->setPath('adminhtml/system_config/edit/section/writetextai_settings');
            }
        } catch (\WriteTextAI\WriteTextAI\Exception\UnauthorizedException $e) {
            return $resultRedirect->setPath('writetextai/setup/index');
        } catch (\WriteTextAI\WriteTextAI\Exception\VersionException $e) {
            return $resultRedirect->setPath('writetextai/grid/index');
        }
    }

    /**
     * Check if atleast logged in
     *
     * @return bool
     */
    private function isLoggedIn()
    {
        $accountToken = $this->helper->getAccountToken();
        $region = $this->helper->getRegion();

        /**
         * check if account token and region is set
         * then verify account token
         */
        if ($accountToken &&
            $region &&
            $this->curlBuilder->getConnectToken($accountToken)
        ) {
            return true;
        }

        return false;
    }
}
