<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

declare(strict_types=1);

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Setup;

class Reset extends \Magento\Backend\App\Action
{
    /**
     * @var \Magento\Framework\Controller\ResultFactory
     */
    protected $resultFactory;

    /**
     * @var \Magento\Framework\Message\ManagerInterface
     */
    protected $messageManager;

    /**
     * @var \Magento\Config\Model\ResourceModel\Config\Data\CollectionFactory
     */
    protected $configCollectionFactory;

    /**
     * @var \Magento\Config\Model\ResourceModel\Config\Data
     */
    protected $configResource;

    /**
     * @var \Magento\Framework\App\Cache\Manager
     */
    protected $cacheManager;

    /**
     * Constructor
     *
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Magento\Framework\Message\ManagerInterface $messageManager
     * @param \Magento\Config\Model\ResourceModel\Config\Data\CollectionFactory $configCollection
     * @param \Magento\Config\Model\ResourceModel\Config\Data $configResource
     * @param \Magento\Framework\App\Cache\Manager $cacheManager
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\Message\ManagerInterface $messageManager,
        \Magento\Config\Model\ResourceModel\Config\Data\CollectionFactory $configCollection,
        \Magento\Config\Model\ResourceModel\Config\Data $configResource,
        \Magento\Framework\App\Cache\Manager $cacheManager
    ) {
        $this->messageManager = $messageManager;
        $this->configCollectionFactory = $configCollection;
        $this->configResource = $configResource;
        $this->cacheManager = $cacheManager;
        parent::__construct($context);
    }

    /**
     * Reset action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $collection = $this->configCollectionFactory->create();
        $collection->addPathFilter('writetextai_settings');

        foreach ($collection as $config) {
            $this->configResource->delete($config);
        }

        $this->cacheManager->clean(['config']);

        $this->messageManager->addSuccess(__('Reset successful!'));

        $resultRedirect = $this->resultRedirectFactory->create();

        return $resultRedirect->setPath('writetextai/setup/index');
    }
}
