<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

declare(strict_types=1);

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Setup;

use Magento\Framework\App\Action\HttpPostActionInterface as HttpPostActionInterface;
use WriteTextAI\WriteTextAI\Helper\Settings;

class Save extends \Magento\Backend\App\Action implements HttpPostActionInterface
{
    /**
     * Backend Config Model Factory
     *
     * @var \Magento\Config\Model\Config\Factory
     */
    protected $configFactory;

    /**
     * @var \WriteTextAI\WriteTextAI\Helper\Data
     */
    protected $helper;

    /**
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Magento\Config\Model\Config\Factory $configFactory
     * @param \WriteTextAI\WriteTextAI\Helper\Data $helper
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Config\Model\Config\Factory $configFactory,
        \WriteTextAI\WriteTextAI\Helper\Data $helper
    ) {
        parent::__construct($context);
        
        $this->configFactory = $configFactory;
        $this->helper = $helper;
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Redirect
     */
    public function execute()
    {
        $step = $this->getRequest()->getPost('step');

        if ($step == 2) {
            $this->saveStep2();
            $this->helper->flushConfigCache();
        } elseif ($step == 3) {
            $this->saveStep3();
            $this->helper->flushConfigCache();
        } elseif ($step == 4) {
            $this->saveStep4();
            $this->helper->flushConfigCache();
        }

        return $this->resultRedirectFactory->create()->setPath('writetextai/*/');
    }

    /**
     * Save step 2
     *
     * @return void
     */
    protected function saveStep2()
    {
        try {
            $configData = [
                'section' => Settings::SECTION_NAME
            ];

            $fields = array_keys($this->helper->getDataMappingFields());

            foreach ($fields as $field) {
                $value = $this->getRequest()->getPost($field);
                $configData['groups']['mapping']['fields'][$field]['value'] = $value;
            }

            /**
             * Save custom open graph
             */
            $customOpenGraph = $this->getRequest()->getPost('custom_open_graph');
            $configData['groups']['mapping']['fields']['custom_open_graph']['value'] = $customOpenGraph ?? 0;
            if (!$customOpenGraph) {
                $configData['groups']['mapping']['fields']['open_graph']['value'] = '';
            }

            /** @var \Magento\Config\Model\Config $configModel */
            $configModel = $this->configFactory->create(['data' => $configData]);
            $configModel->save();
        } catch (\Magento\Framework\Exception\LocalizedException $e) {
            $messages = explode("\n", $e->getMessage());
            foreach ($messages as $message) {
                $this->messageManager->addError($message);
            }
        } catch (\Exception $e) {
            $this->messageManager->addException(
                $e,
                $e->getMessage()
            );
        }
    }
    
    /**
     * Save step 3
     *
     * @return void
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Exception
     */
    protected function saveStep3()
    {
        $request = $this->getRequest();
        $configData = [
            'section' => Settings::SECTION_NAME
        ];

        // tones/styles/audience
        foreach (['tones', 'styles', 'audiences'] as $field) {
            if ($field != 'styles') {
                $value = implode(",", $request->getPost($field, []));
            } else {
                $value = $this->getRequest()->getPost($field) ;
            }

            $configData['groups']['general']['fields'][$field]['value'] = $value;
        }

        // attributes
        $attributes = implode(",", $request->getPost('attributes', []));
        $configData['groups']['attributes']['fields']['attributes']['value'] = $attributes;

        // min/max
        foreach (['min', 'max'] as $field) {
            $description    = $request->getPost('description_'. $field);
            $excerpt        = $request->getPost('excerpt_'. $field);

            $configData['groups']['description']['fields'][$field]['value'] = $description;
            $configData['groups']['excerpt']['fields'][$field]['value'] = $excerpt;
        }

        try {
            /** @var \Magento\Config\Model\Config $configModel */
            $configModel = $this->configFactory->create(['data' => $configData]);
            $configModel->save();
        } catch (\Magento\Framework\Exception\LocalizedException $e) {
            $messages = explode("\n", $e->getMessage());
            foreach ($messages as $message) {
                $this->messageManager->addError($message);
            }
        } catch (\Exception $e) {
            $this->messageManager->addException(
                $e,
                $e->getMessage()
            );
        }
    }

    /**
     * Save step 4
     *
     * @return void
     */
    protected function saveStep4()
    {
        try {
            $configData = ['section' => Settings::SECTION_NAME];
            $finalized = $this->getRequest()->getPost('finalized');
            $configData['groups']['setup']['fields']['finalized']['value'] = $finalized;

            /** @var \Magento\Config\Model\Config $configModel */
            $configModel = $this->configFactory->create(['data' => $configData]);
            $configModel->save();
        } catch (\Magento\Framework\Exception\LocalizedException $e) {
            $messages = explode("\n", $e->getMessage());
            foreach ($messages as $message) {
                $this->messageManager->addError($message);
            }
        } catch (\Exception $e) {
            $this->messageManager->addException(
                $e,
                $e->getMessage()
            );
        }
    }
}
