<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Helper;

use Magento\Store\Model\StoreManagerInterface;
use WriteTextAI\WriteTextAI\Model\GeneratedText;
use WriteTextAI\WriteTextAI\Model\Config\Source\DefaultValues\Store;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;

class ReviewStatus extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var GeneratedText
     */
    protected $generatedText;

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var TimezoneInterface
     */
    protected $timezone;

    /**
     * Constructor
     *
     * @param GeneratedText $generatedText
     * @param StoreManagerInterface $storeManager
     * @param TimezoneInterface $timezone
     */
    public function __construct(
        GeneratedText $generatedText,
        StoreManagerInterface $storeManager,
        TimezoneInterface $timezone
    ) {
        $this->generatedText = $generatedText;
        $this->storeManager = $storeManager;
        $this->timezone = $timezone;
    }

    /**
     * Get review status
     *
     * @param int $productId
     * @param int $storeId
     * @return array
     */
    public function getReviewStatus($productId, $storeId)
    {
        $reviews = [];
        if ((int)$storeId !== Store::DEFAULT_STORE_ID) {
            $review = $this->generatedText->getReviewStatus(
                $productId,
                $storeId
            );

            if (!empty($review)) {
                $review = $this->formatDate($review);
                $reviews[] = [
                    'id' => $storeId,
                    'name' => '',
                    'reviews' => $review
                ];
            }
        } else {
            $stores = $this->storeManager->getStores();
            foreach ($stores as $store) {
                $review = $this->generatedText->getReviewStatus(
                    $productId,
                    $store->getId()
                );
                if (!empty($review)) {
                    $review = $this->formatDate($review);
                    $reviews[] = [
                        'id' => $store->getId(),
                        'name' => $store->getName(),
                        'reviews' => $review
                    ];
                }
            }
        }

        return $reviews;
    }

    /**
     * Format date
     *
     * @param array $reviews
     * @return string
     */
    protected function formatDate($reviews)
    {
        $formattedReviews = [];
        
        foreach ($reviews as $review) {
            $date = $review['date'];

            $timestampObj = new \DateTime($date);
            $timestamp = $timestampObj->format('Y-m-d H:i:s');
            $formattedDate = $this->timezone->date($timestamp);
            $formattedDate = $formattedDate->format('F j, Y \a\t g:i:s A');

            $review['date'] = $formattedDate;

            $formattedReviews[] = $review;
        }

        return $formattedReviews;
    }
}
