<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

declare(strict_types=1);

namespace WriteTextAI\WriteTextAI\Model;

use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;
use WriteTextAI\WriteTextAI\Api\Data\ProductKeywordIdeasFiltersInterface;
use WriteTextAI\WriteTextAI\Api\Data\ProductKeywordIdeasFiltersInterfaceFactory;
use WriteTextAI\WriteTextAI\Api\Data\ProductKeywordIdeasFiltersSearchResultsInterfaceFactory;
use WriteTextAI\WriteTextAI\Api\ProductKeywordIdeasFiltersRepositoryInterface;
use WriteTextAI\WriteTextAI\Model\ResourceModel\ProductKeywordIdeasFilters as ResourceProductKeywordIdeasFilters;
use WriteTextAI\WriteTextAI\Model\ResourceModel\ProductKeywordIdeasFilters\CollectionFactory
    as ProductKeywordIdeasFiltersCollectionFactory;

class ProductKeywordIdeasFiltersRepository implements ProductKeywordIdeasFiltersRepositoryInterface
{
    /**
     * @var ProductKeywordIdeasFilters
     */
    protected $searchResultsFactory;

    /**
     * @var ResourceProductKeywordIdeasFilters
     */
    protected $resource;

    /**
     * @var ProductKeywordIdeasFiltersCollectionFactory
     */
    protected $productKeywordIdeasFiltersCollectionFactory;

    /**
     * @var CollectionProcessorInterface
     */
    protected $collectionProcessor;

    /**
     * @var ProductKeywordIdeasFiltersInterfaceFactory
     */
    protected $productKeywordIdeasFiltersFactory;

    /**
     * @param ResourceProductKeywordIdeasFilters $resource
     * @param ProductKeywordIdeasFiltersInterfaceFactory $productKeywordIdeasFiltersFactory
     * @param ProductKeywordIdeasFiltersCollectionFactory $productKeywordIdeasFiltersCollectionFactory
     * @param ProductKeywordIdeasFiltersSearchResultsInterfaceFactory $searchResultsFactory
     * @param CollectionProcessorInterface $collectionProcessor
     */
    public function __construct(
        ResourceProductKeywordIdeasFilters $resource,
        ProductKeywordIdeasFiltersInterfaceFactory $productKeywordIdeasFiltersFactory,
        ProductKeywordIdeasFiltersCollectionFactory $productKeywordIdeasFiltersCollectionFactory,
        ProductKeywordIdeasFiltersSearchResultsInterfaceFactory $searchResultsFactory,
        CollectionProcessorInterface $collectionProcessor
    ) {
        $this->resource = $resource;
        $this->productKeywordIdeasFiltersFactory = $productKeywordIdeasFiltersFactory;
        $this->productKeywordIdeasFiltersCollectionFactory = $productKeywordIdeasFiltersCollectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->collectionProcessor = $collectionProcessor;
    }

    /**
     * @inheritDoc
     */
    public function save(
        ProductKeywordIdeasFiltersInterface $productKeywordIdeasFilters
    ) {
        try {
            $this->resource->save($productKeywordIdeasFilters);
        } catch (\Exception $exception) {
            throw new CouldNotSaveException(__(
                'Could not save the productKeywordIdeasFilters: %1',
                $exception->getMessage()
            ));
        }
        return $productKeywordIdeasFilters;
    }

    /**
     * @inheritDoc
     */
    public function get($productKeywordIdeasFiltersId)
    {
        $productKeywordIdeasFilters = $this->productKeywordIdeasFiltersFactory->create();
        $this->resource->load($productKeywordIdeasFilters, $productKeywordIdeasFiltersId);
        if (!$productKeywordIdeasFilters->getId()) {
            throw new NoSuchEntityException(__(
                'ProductKeywordIdeasFilters with id "%1" does not exist.',
                $productKeywordIdeasFiltersId
            ));
        }
        return $productKeywordIdeasFilters;
    }

    /**
     * @inheritDoc
     */
    public function getList(
        \Magento\Framework\Api\SearchCriteriaInterface $criteria
    ) {
        $collection = $this->productKeywordIdeasFiltersCollectionFactory->create();
        
        $this->collectionProcessor->process($criteria, $collection);
        
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);
        
        $items = [];
        foreach ($collection as $model) {
            $items[] = $model;
        }
        
        $searchResults->setItems($items);
        $searchResults->setTotalCount($collection->getSize());
        return $searchResults;
    }

    /**
     * @inheritDoc
     */
    public function delete(
        ProductKeywordIdeasFiltersInterface $productKeywordIdeasFilters
    ) {
        try {
            $productKeywordIdeasFiltersModel = $this->productKeywordIdeasFiltersFactory->create();
            $this->resource->load(
                $productKeywordIdeasFiltersModel,
                $productKeywordIdeasFilters->getProductkeywordideasfiltersId()
            );
            $this->resource->delete($productKeywordIdeasFiltersModel);
        } catch (\Exception $exception) {
            throw new CouldNotDeleteException(__(
                'Could not delete the ProductKeywordIdeasFilters: %1',
                $exception->getMessage()
            ));
        }
        return true;
    }

    /**
     * @inheritDoc
     */
    public function deleteById($productKeywordIdeasFiltersId)
    {
        return $this->delete($this->get($productKeywordIdeasFiltersId));
    }
}
