<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Model;

use WriteTextAI\WriteTextAI\Model\UserProductsFactory;
use WriteTextAI\WriteTextAI\Api\UserProductsRepositoryInterface;
use WriteTextAI\WriteTextAI\Model\Api\Session as ApiSession;
use WriteTextAI\WriteTextAI\Helper\Data;

class UserProductsManager
{

    public const ACTION_TYPE_GALLERY_IMAGES = 'gallery_images';

    /**
     * @var UserProductsFactory
     */
    protected $userProductsFactory;

    /**
     * @var UserProductsRepositoryInterface
     */
    protected $userProductsRepository;

    /**
     * @var ApiSession
     */
    protected $apiSession;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * Constructor
     *
     * @param UserProductsFactory $userProductsFactory
     * @param UserProductsRepositoryInterface $userProductsRepository
     * @param ApiSession $apiSession
     * @param Data $helper
     */
    public function __construct(
        UserProductsFactory $userProductsFactory,
        UserProductsRepositoryInterface $userProductsRepository,
        ApiSession $apiSession,
        Data $helper
    ) {
        $this->userProductsFactory = $userProductsFactory;
        $this->userProductsRepository = $userProductsRepository;
        $this->apiSession = $apiSession;
        $this->helper = $helper;
    }

    /**
     * Get user's last text types selected
     *
     * @param string $actionType
     *
     * @return array
     */
    public function getUserProductsImagesSelected($productId)
    {
        $userId = $this->getUser()->getId();

        $userProducts = $this->userProductsFactory->create();

        $existing = $userProducts->getCollection()
            ->addFieldToFilter('user_id', $userId)
            ->addFieldToFilter('product_id', $productId)
            ->getFirstItem();

        $selected = [];
        if ($existing->getId()) {
            if ($existing->getImagesSelected()) {
                $selected = explode(",", $existing->getImagesSelected());
            }
        }
        return $selected;
    }

    /**
     * Get user's last text types selected
     *
     * @param string $actionType
     *
     * @return array
     */
    public function getUserProductsImagesSelectedAvailable($productId)
    {
        $userId = $this->getUser()->getId();

        $userProducts = $this->userProductsFactory->create();

        $existing = $userProducts->getCollection()
            ->addFieldToFilter('user_id', $userId)
            ->addFieldToFilter('product_id', $productId)
            ->getFirstItem();

        return $existing->getId() ? true : false;
    }

    /**
     * Save user products images selected
     *
     * @param array $fields
     * @param string $actionType
     *
     * @return void
     */
    public function saveUserProductsImagesSelected($images, $productId)
    {
        $userId = $this->getUser()->getId();

        $userProducts = $this->userProductsFactory->create();

        $existing = $userProducts->getCollection()
            ->addFieldToFilter('user_id', $userId)
            ->addFieldToFilter('product_id', $productId)
            ->getFirstItem();

        if (is_array($images)) {
            $images = implode(",", $images);
        }

        if ($existing->getId()) {
            // Update the existing record
            $existing->setImagesSelected($images);
            $userProducts = $existing;
        } else {
            // Record not found, create a new one
            $userProducts->setData([
                "user_id" => $userId,
                "product_id" => $productId,
                "images_selected" => $images
            ]);
        }

        $this->userProductsRepository->save($userProducts);
    }

    /**
     * Get current user
     *
     * @return \Magento\User\Model\User
     */
    private function getUser()
    {
        return $this->apiSession->getCurrentUser();
    }

    /**
     * Delete all settings of current user
     *
     * @return void
     */
    public function deleteAll()
    {
        $userId = $this->getUser()->getId();

        $userProducts = $this->userProductsFactory->create();

        $userProducts->getCollection()
            ->addFieldToFilter('user_id', $userId)
            ->walk('delete');
    }
}
