<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

declare(strict_types=1);

namespace WriteTextAI\WriteTextAI\Model;

use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;
use WriteTextAI\WriteTextAI\Api\Data\UserProductsInterface;
use WriteTextAI\WriteTextAI\Api\Data\UserProductsInterfaceFactory;
use WriteTextAI\WriteTextAI\Api\Data\UserProductsSearchResultsInterfaceFactory;
use WriteTextAI\WriteTextAI\Api\UserProductsRepositoryInterface;
use WriteTextAI\WriteTextAI\Model\ResourceModel\UserProducts as ResourceUserProducts;
use WriteTextAI\WriteTextAI\Model\ResourceModel\UserProducts\CollectionFactory as UserProductsCollectionFactory;

class UserProductsRepository implements UserProductsRepositoryInterface
{

    /**
     * @var ResourceUserProducts
     */
    protected $resource;

    /**
     * @var CollectionProcessorInterface
     */
    protected $collectionProcessor;

    /**
     * @var UserProductsInterfaceFactory
     */
    protected $userProductsFactory;

    /**
     * @var UserProducts
     */
    protected $searchResultsFactory;

    /**
     * @var UserProductsCollectionFactory
     */
    protected $userProductsCollectionFactory;

    /**
     * @param ResourceUserProducts $resource
     * @param UserProductsInterfaceFactory $userProductsFactory
     * @param UserProductsCollectionFactory $userProductsCollectionFactory
     * @param UserProductsSearchResultsInterfaceFactory $searchResultsFactory
     * @param CollectionProcessorInterface $collectionProcessor
     */
    public function __construct(
        ResourceUserProducts $resource,
        UserProductsInterfaceFactory $userProductsFactory,
        UserProductsCollectionFactory $userProductsCollectionFactory,
        UserProductsSearchResultsInterfaceFactory $searchResultsFactory,
        CollectionProcessorInterface $collectionProcessor
    ) {
        $this->resource = $resource;
        $this->userProductsFactory = $userProductsFactory;
        $this->userProductsCollectionFactory = $userProductsCollectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->collectionProcessor = $collectionProcessor;
    }

    /**
     * @inheritDoc
     */
    public function save(UserProductsInterface $userProducts)
    {
        try {
            $this->resource->save($userProducts);
        } catch (\Exception $exception) {
            throw new CouldNotSaveException(__(
                'Could not save the userProducts: %1',
                $exception->getMessage()
            ));
        }
        return $userProducts;
    }

    /**
     * @inheritDoc
     */
    public function get($userProductsId)
    {
        $userProducts = $this->userProductsFactory->create();
        $this->resource->load($userProducts, $userProductsId);
        if (!$userProducts->getId()) {
            throw new NoSuchEntityException(__('User_products with id "%1" does not exist.', $userProductsId));
        }
        return $userProducts;
    }

    /**
     * @inheritDoc
     */
    public function getList(
        \Magento\Framework\Api\SearchCriteriaInterface $criteria
    ) {
        $collection = $this->userProductsCollectionFactory->create();
        
        $this->collectionProcessor->process($criteria, $collection);
        
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);
        
        $items = [];
        foreach ($collection as $model) {
            $items[] = $model;
        }
        
        $searchResults->setItems($items);
        $searchResults->setTotalCount($collection->getSize());
        return $searchResults;
    }

    /**
     * @inheritDoc
     */
    public function delete(UserProductsInterface $userProducts)
    {
        try {
            $userProductsModel = $this->userProductsFactory->create();
            $this->resource->load($userProductsModel, $userProducts->getUserProductsId());
            $this->resource->delete($userProductsModel);
        } catch (\Exception $exception) {
            throw new CouldNotDeleteException(__(
                'Could not delete the User_products: %1',
                $exception->getMessage()
            ));
        }
        return true;
    }

    /**
     * @inheritDoc
     */
    public function deleteById($userProductsId)
    {
        return $this->delete($this->get($userProductsId));
    }
}
