<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Model;

use WriteTextAI\WriteTextAI\Model\UserWebTokenFactory;
use WriteTextAI\WriteTextAI\Api\UserWebTokenRepositoryInterface;

class UserWebTokenManager
{
    /**
     * @var UserWebTokenFactory
     */
    protected $userWebTokenFactory;

    /**
     * @var UserWebTokenRepositoryInterface
     */
    protected $userWebTokenRepository;

    /**
     * Constructor
     *
     * @param UserWebTokenFactory $userWebTokenFactory
     * @param UserWebTokenRepositoryInterface $userWebTokenRepository
     */
    public function __construct(
        UserWebTokenFactory $userWebTokenFactory,
        UserWebTokenRepositoryInterface $userWebTokenRepository
    ) {
        $this->userWebTokenFactory = $userWebTokenFactory;
        $this->userWebTokenRepository = $userWebTokenRepository;
    }

    /**
     * Get user web token
     *
     * @param int $userId
     * @return string
     */
    public function getUserWebToken($userId)
    {
        $userWebToken = $this->userWebTokenFactory->create();
        $userWebToken = $userWebToken->getCollection()
            ->addFieldToFilter('user_id', $userId)
            ->getFirstItem();

        if ($userWebToken->getId()) {
            return $userWebToken->getToken();
        } else {
            return '';
        }
    }

    /**
     * Set user web token
     *
     * @param int $userId
     * @param string $token
     * @return void
     */
    public function setUserWebToken($userId, $token)
    {
        $userWebToken = $this->userWebTokenFactory->create();

        $existing = $userWebToken->getCollection()
            ->addFieldToFilter('user_id', $userId)
            ->getFirstItem();

        if ($existing->getId()) {
            // Update the existing record
            $existing->setToken($token);
            $userWebToken = $existing;
        } else {
            // Record not found, create a new one
            $userWebToken->setData([
                "user_id" => $userId,
                "token" => $token
            ]);
        }

        $this->userWebTokenRepository->save($userWebToken);
    }

    /**
     * Delete user web token
     *
     * @param int $userId
     * @return void
     */
    public function deleteUserWebToken($userId)
    {
        $userWebToken = $this->userWebTokenFactory->create();

        $existing = $userWebToken->getCollection()
            ->addFieldToFilter('user_id', $userId)
            ->getFirstItem();

        if ($existing->getId()) {
            $this->userWebTokenRepository->delete($existing);
        }
    }

    /**
     * Delete all user web token
     *
     * @return void
     */
    public function deleteAllUserWebToken()
    {
        $userWebToken = $this->userWebTokenFactory->create();
        $userWebToken->getCollection()->walk('delete');
    }
}
