<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Plugin\Component;

use Magento\Framework\View\LayoutInterface;
use WriteTextAI\WriteTextAI\Model\UserFieldsSelectedManager;
use WriteTextAI\WriteTextAI\ViewModel\UserFieldsSelected;

class MassAction
{

    /**
     * @var LayoutInterface
     */
    protected $layout;

    /**
     * @var UserFieldsSelected
     */
    protected $userFieldsSelected;

    /**
     * Constructor
     *
     * @param LayoutInterface $layout
     * @param UserFieldsSelected $userFieldsSelected
     */
    public function __construct(
        LayoutInterface $layout,
        UserFieldsSelected $userFieldsSelected
    ) {
        $this->layout = $layout;
        $this->userFieldsSelected = $userFieldsSelected;
    }

    /**
     * Modify mass action popup content
     *
     * @param \Magento\Ui\Component\MassAction $subject
     * @param string $key
     * @param array $value
     * @return array
     */
    public function beforeSetData(\Magento\Ui\Component\MassAction $subject, $key, $value)
    {
        if ($key != 'config' ||
            $subject->getName() != 'listing_massaction' ||
            !isset($value['actions'])
        ) {
            return;
        }
        
        foreach ($value['actions'] as &$action) {
            if ($action['type'] == 'generate' && stripos($action['url'], 'writetextai/grid/massGenerate') !== false) {
                $hideSettings = $this->userFieldsSelected->getUserFieldsSelected(
                    UserFieldsSelectedManager::ACTION_TYPE_HIDE_BULK_SETTINGS
                );
                $hideSettings = filter_var($hideSettings, FILTER_VALIDATE_BOOLEAN);

                if ($hideSettings) {
                    unset($action['confirm']);
                }

                if (isset($action['confirm']['message'])) {
                    $action['confirm']['message'] = $this->getConfirmContent();
                    $action['confirm']['button'] = $this->getGenerateButtonContent();
                }
            }
            if ($action['type'] == 'transfer' && stripos($action['url'], 'writetextai/grid/massTransfer') !== false) {
                if (isset($action['confirm']['message'])) {
                    $action['confirm']['message'] = $this->getTransferConfirmContent();
                }
            }
        }

        return [$key, $value];
    }

    /**
     * Get generate button content
     *
     * @return string
     */
    protected function getGenerateButtonContent()
    {
        $block = $this->layout->createBlock(\WriteTextAI\WriteTextAI\Block\Adminhtml\Grid\GenerateButton::class);
        return $block->toHtml();
    }

    /**
     * Get confirm content
     *
     * @return string
     */
    protected function getConfirmContent()
    {
        $block = $this->layout->createBlock(\WriteTextAI\WriteTextAI\Block\Adminhtml\Grid\Prompt::class);
        return $block->toHtml();
    }

    /**
     * Get transfer confirm content
     *
     * @return string
     */
    protected function getTransferConfirmContent()
    {
        $block = $this->layout->createBlock(\WriteTextAI\WriteTextAI\Block\Adminhtml\Grid\TransferPrompt::class);
        return $block->toHtml();
    }
}
