<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Setup\Patch\Data;

use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\Patch\DataPatchInterface;
use Magento\Authorization\Model\ResourceModel\Role\Collection;
use Magento\Authorization\Model\RulesFactory;
use Magento\Authorization\Model\Acl\AclRetriever;

class Country implements DataPatchInterface
{
    public const ADMIN_ROLE = 'WriteText.ai Administrator';

    public const EDITOR_ROLE = 'WriteText.ai Editor';

    public const CONTRIBUTOR_ROLE = 'WriteText.ai Contributor';

    /**
     * @var ModuleDataSetupInterface
     */
    protected $moduleDataSetup;

    /**
     * @var Collection
     */
    protected $roleCollection;

    /**
     * @var RulesFactory
     */
    protected $rulesFactory;

    /**
     * @var AclRetriever
     */
    protected $aclRetriver;

    /**
     * @param ModuleDataSetupInterface $moduleDataSetup
     * @param Collection $roleCollection
     * @param RulesFactory $rulesFactory
     * @param AclRetriever $aclRetriever
     */
    public function __construct(
        ModuleDataSetupInterface $moduleDataSetup,
        Collection $roleCollection,
        RulesFactory $rulesFactory,
        AclRetriever $aclRetriever
    ) {
        $this->moduleDataSetup = $moduleDataSetup;
        $this->roleCollection = $roleCollection;
        $this->rulesFactory = $rulesFactory;
        $this->aclRetriver = $aclRetriever;
    }

    /**
     * @inheritdoc
     */
    public function apply()
    {
        $this->moduleDataSetup->getConnection()->startSetup();
        $this->updateRolePermissions();
        $this->moduleDataSetup->getConnection()->endSetup();
    }

    /**
     * @inheritdoc
     */
    public static function getDependencies()
    {
        return [];
    }

    /**
     * @inheritdoc
     */
    public function getAliases()
    {
        return [];
    }

    /**
     * Update role permissions
     *
     * @return void
     */
    private function updateRolePermissions()
    {
        $roles = $this->roleCollection->addFieldToFilter('role_name', ['in' => [
            static::ADMIN_ROLE,
            static::EDITOR_ROLE,
            static::CONTRIBUTOR_ROLE
        ]])->load();

        foreach ($roles as $role) {
            $resource = $this->aclRetriver->getAllowedResourcesByRole($role->getId());

            array_push($resource, 'WriteTextAI_WriteTextAI::country');

            $this->rulesFactory->create()
                ->setRoleId($role->getId())
                ->setResources($resource)
                ->saveRel();
        }
    }
}
