<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Ui\DataProvider\Products;

use Magento\Framework\Api\FilterBuilder;
use Magento\Framework\Api\Search\ReportingInterface;
use Magento\Framework\Api\Search\SearchCriteriaBuilder;
use Magento\Framework\Api\Search\FilterGroupBuilder;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\View\Element\UiComponent\DataProvider\DataProvider;
use Magento\Store\Model\ScopeInterface;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use WriteTextAI\WriteTextAI\Model\Api\Session as ApiSession;
use WriteTextAI\WriteTextAI\Model\GeneratedText;
use WriteTextAI\WriteTextAI\Model\OptionSource\Filter\Fields;
use Magento\Framework\UrlInterface;
use WriteTextAI\WriteTextAI\Model\UserFieldsSelectedManager;
use Magento\User\Model\ResourceModel\User\CollectionFactory as UserCollectionFactory;
use WriteTextAI\WriteTextAI\Model\OptionSource\Filter\AiStatus;
use WriteTextAI\WriteTextAI\Helper\Filters as FiltersHelper;
use Magento\Catalog\Helper\Output as OutputHelper;
use Magento\Catalog\Api\ProductRepositoryInterface;
use WriteTextAI\WriteTextAI\Helper\Data as HelperData;
use WriteTextAI\WriteTextAI\Model\OptionSource\Filter\Stores as StoresFilterOptions;
use WriteTextAI\WriteTextAI\Api\Data\AiProductInterface;
use WriteTextAI\WriteTextAI\ViewModel\Premium;
use WriteTextAI\WriteTextAI\ViewModel\UserFieldsSelected;
use WriteTextAI\WriteTextAI\Helper\Fields as FieldHelper;
use WriteTextAI\WriteTextAI\Model\Config\Source\DefaultValues\Store;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;
use WriteTextAI\WriteTextAI\Helper\Product as ProductHelper;

class ListingDataProvider extends DataProvider
{
    /**
     * @var GeneratedText
     */
    protected $generatedText;

    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var ApiSession
     */
    protected $apiSession;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var UrlInterface
     */
    protected $urlBuilder;

    /**
     * @var UserFieldsSelected
     */
    protected $userFieldsSelected;

    /**
     * @var UserCollectionFactory
     */
    protected $userCollectionFactory;

    /**
     * @var FiltersHelper
     */
    protected $filtersHelper;

    /**
     * @var OutputHelper
     */
    protected $outputHelper;

    /**
     * @var ProductRepositoryInterface
     */
    protected $productRepository;

    /**
     * @var HelperData
     */
    protected $helperData;

    /**
     * @var FilterGroupBuilder
     */
    protected $filterGroupBuilder;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * @var FieldHelper
     */
    protected $fieldHelper;

    /**
     * @var TimezoneInterface
     */
    protected $timezone;
    
    /**
     * @var ProductHelper
     */
    protected $productHelper;

    /**
     * Constructor
     *
     * @param string $name
     * @param string $primaryFieldName
     * @param string $requestFieldName
     * @param ReportingInterface $reporting
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param RequestInterface $request
     * @param FilterBuilder $filterBuilder
     * @param GeneratedText $generatedText
     * @param ApiManager $apiManager
     * @param ApiSession $apiSession
     * @param ScopeConfigInterface $scopeConfig
     * @param UrlInterface $urlBuilder
     * @param UserFieldsSelected $userFieldsSelected
     * @param UserCollectionFactory $userCollectionFactory
     * @param FiltersHelper $filtersHelper
     * @param OutputHelper $outputHelper
     * @param ProductRepositoryInterface $productRepository
     * @param HelperData $helperData
     * @param FilterGroupBuilder $filterGroupBuilder
     * @param Premium $premium
     * @param FieldHelper $fieldHelper
     * @param TimezoneInterface $timezone
     * @param array $meta
     * @param array $data
     */
    public function __construct(
        $name,
        $primaryFieldName,
        $requestFieldName,
        ReportingInterface $reporting,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        RequestInterface $request,
        FilterBuilder $filterBuilder,
        GeneratedText $generatedText,
        ApiManager $apiManager,
        ApiSession $apiSession,
        ScopeConfigInterface $scopeConfig,
        UrlInterface $urlBuilder,
        UserFieldsSelected $userFieldsSelected,
        UserCollectionFactory $userCollectionFactory,
        FiltersHelper $filtersHelper,
        OutputHelper $outputHelper,
        ProductRepositoryInterface $productRepository,
        HelperData $helperData,
        FilterGroupBuilder $filterGroupBuilder,
        Premium $premium,
        FieldHelper $fieldHelper,
        TimezoneInterface $timezone,
        ProductHelper $productHelper,
        array $meta = [],
        array $data = []
    ) {
        $this->generatedText = $generatedText;
        $this->apiManager = $apiManager;
        $this->apiSession = $apiSession;
        $this->scopeConfig = $scopeConfig;
        $this->urlBuilder = $urlBuilder;
        $this->userFieldsSelected = $userFieldsSelected;
        $this->userCollectionFactory = $userCollectionFactory;
        $this->filtersHelper = $filtersHelper;
        $this->outputHelper = $outputHelper;
        $this->productRepository = $productRepository;
        $this->helperData = $helperData;
        $this->filterGroupBuilder = $filterGroupBuilder;
        $this->premium = $premium;
        $this->fieldHelper = $fieldHelper;
        $this->timezone = $timezone;
        $this->productHelper = $productHelper;

        parent::__construct(
            $name,
            $primaryFieldName,
            $requestFieldName,
            $reporting,
            $searchCriteriaBuilder,
            $request,
            $filterBuilder,
            $meta,
            $data
        );
    }

    /**
     * @inheritdoc
     */
    public function addFilter(\Magento\Framework\Api\Filter $filter)
    {
        $filters = $this->request->getParam('filters');
        $storeId = !isset($filters['store_id']) ? Store::DEFAULT_STORE_ID : $filters['store_id'];

        if ($filter->getField() == 'writetextai_status') {
            $filters = $filters['writetextai_status'];
            $filters['store_id'] = $storeId;
            $status = $filters['status'];

            switch ($status) {
                case AiStatus::WRITETEXTAI:
                    if (isset($filters['ai_fields']) && isset($filters['ai_status'])) {
                        $productIds = $this->getFilterByAiStatus($filters);
                        $filter = $this->filterBuilder->setField('entity_id')
                            ->setValue($productIds)
                            ->setConditionType('in')
                            ->create();
                        $this->searchCriteriaBuilder->addFilter($filter);
                    }
                    break;
                case AiStatus::NO_ACTIVITY:
                    if (isset($filters['no_activity_days'])) {
                        $dates = $this->filtersHelper->getLastNumberOfDays($filters['no_activity_days']);
                        $actions = [
                            AiProductInterface::GENERATED_AT,
                            AiProductInterface::TRANSFERRED_AT,
                            AiProductInterface::EDITED_AT
                        ];
                        foreach ($actions as $action) {
                            $filterLt = $this->filterBuilder->setField($action)
                                ->setValue($dates['start_date'])
                                ->setConditionType('lt')
                                ->create();
                            $filterNull = $this->filterBuilder->setField($action)
                                ->setConditionType('null')
                                ->create();
                            $filtergroup = $this->filterGroupBuilder
                                ->addFilter($filterLt)
                                ->addFilter($filterNull)
                                ->create();
                            $this->searchCriteriaBuilder->addFilterGroup($filtergroup);
                        }
                    }
                    break;
                case AiStatus::NOT_GENERATED:
                    if (isset($filters['ai_fields'])) {
                        $filters['ai_status'] = [
                            AiStatus::GENERATED,
                            AiStatus::TRANSFERRED,
                            AiStatus::EDITED,
                            AiStatus::REVIEWED
                        ];
                        $productIds = $this->getFilterByAiStatus($filters);
                        if (!empty($productIds)) {
                            $filter = $this->filterBuilder->setField('entity_id')
                                ->setValue($productIds)
                                ->setConditionType('nin')
                                ->create();
                            $this->searchCriteriaBuilder->addFilter($filter);
                        }
                    }
                    break;
                case 'review_status':
                    if (isset($filters['review_status'])) {
                        $productIds = $this->getFilterByReviewStatus($filters);
                        $filter = $this->filterBuilder->setField('entity_id')
                            ->setValue($productIds)
                            ->setConditionType('in')
                            ->create();
                        $this->searchCriteriaBuilder->addFilter($filter);
                    }
                    break;
            }
        } elseif ($filter->getField() == 'store_id' && (int)$storeId !== Store::DEFAULT_STORE_ID) {
            $website = $this->productHelper->getWebsiteByStoreId($storeId);
            $filterWebsite = $this->filterBuilder->setField('websites')
                ->setValue($website->getId())
                ->setConditionType('finset')
                ->create();
            $this->searchCriteriaBuilder->addFilter($filterWebsite);
            $this->searchCriteriaBuilder->addFilter($filter);
        } else {
            $this->searchCriteriaBuilder->addFilter($filter);
        }
    }

    /**
     * @inheritdoc
     */
    public function getData()
    {
        // set default store filter to prevent duplicate products
        $filters = $this->request->getParam('filters');
        if (!isset($filters['store_id'])) {
            $storeId = Store::DEFAULT_STORE_ID;
            $this->addFilter(
                $this->filterBuilder->setField('store_id')->setValue($storeId)->setConditionType('eq')->create()
            );
        } else {
            $storeId = $filters['store_id'];
        }

        // get language
        $localeCode =  $this->scopeConfig->getValue(
            'general/locale/code',
            ScopeInterface::SCOPE_STORE,
            $storeId
        );
        $languageCode = explode('_', $localeCode)[0];

        $this->isLanguageSupported($languageCode);

        $languageCode = strtoupper($languageCode);
        
        $data = parent::getData();

        $ids = implode(",", array_column($data['items'], 'entity_id'));
        $generated = $this->generatedText->getGeneratedByStoreId($ids, $storeId);
        
        $customOpenGraph = $this->helperData->getCustomOpenGraph();

        foreach ($data['items'] as &$item) {
            $entityId = $item['entity_id'];
            $generatedKeys = [
                Fields::PAGE_TITLE => 'ai_page_title',
                Fields::PAGE_DESCRIPTION => 'ai_page_description',
                Fields::PRODUCT_DESCRIPTION => 'ai_product_description',
                Fields::EXCERPT => 'ai_short_product_description',
                Fields::OPEN_GRAPH => 'ai_open_graph',
            ];

            $defaultKeys = [
                'product_name' => 'default_product_name',
                'visibility' => 'default_visibility',
                'status' => 'default_status',
                'mg_page_title' => 'default_mg_page_title',
                'mg_page_description' => 'default_mg_page_description',
                'mg_product_description' => 'default_mg_product_description',
                'mg_short_product_description' => 'default_mg_short_product_description',
                'thumbnail' => 'default_thumbnail'
            ];

            if (isset($generated[$entityId])) {
                foreach ($generatedKeys as $generatedKey => $itemKey) {
                    if (isset($generated[$entityId][$generatedKey])) {
                        $item[$itemKey] = $generated[$entityId][$generatedKey];
                    }
                }
            }

            $product = $this->productRepository->getById($item['entity_id'], false, $storeId);

            $mediaGallery = $product->getMediaGalleryEntries();
            if (!empty($mediaGallery)) {
                $mediaGalleryIds = [];
                foreach ($mediaGallery as $mediaGalleryEntry) {
                    if (!$mediaGalleryEntry->getDisabled()) {
                        $mediaGalleryIds[] = $mediaGalleryEntry->getId();
                    }
                }
                $imageArray = implode(",", $mediaGalleryIds);
                if ($imageArray !== $item['image_array']) {
                    $item['image_array'] = $imageArray;
                }
            }

            $mappingSettings = $this->helperData->getMappingSettings();
            $keys = [
                'product_name' => 'name',
                'visibility' => 'visibility',
                'status' => 'status',
                'thumbnail' => 'thumbnail',
                'mg_page_title' => $mappingSettings['page_title'],
                'mg_page_description' => $mappingSettings['page_description'],
                'mg_product_description' => $mappingSettings['product_description'],
                'mg_short_product_description' => $mappingSettings['short_product_description']
            ];

            // handle store switcher
            foreach ($defaultKeys as $itemKey => $defaultKey) {
                $isUsingDefault = $this->fieldHelper->isUsingDefaultValues($product, $storeId, $keys[$itemKey]);

                if ($storeId !== Store::DEFAULT_STORE_ID && $isUsingDefault) {
                    $item[$itemKey] = $item[$defaultKey];
                }
            }

            if ($customOpenGraph && !$item['mg_open_graph']) {
                $isUsingDefault = $this->fieldHelper->isUsingDefaultValues(
                    $product,
                    $storeId,
                    $mappingSettings['open_graph']
                );
                
                if ($storeId !== Store::DEFAULT_STORE_ID && $isUsingDefault) {
                    $item['mg_open_graph'] = $item['default_mg_open_graph'];
                }
            }

            // This operation runs a significant number of queries, approximately 250 or more.
            $item = $this->decodeHtmlEntities($product, $item, $mappingSettings);

            $item['language'] = $languageCode;

            if (($item['transferred_at'])) {
                $item['transferred_at'] = $this->timezone->date($item['transferred_at'])->format('Y-m-d H:i:s');
            }

            if (($item['edited_at'])) {
                $item['edited_at'] = $this->timezone->date($item['edited_at'])->format('Y-m-d H:i:s');
            }
            
            $item['edit_url'] = $this->urlBuilder->getUrl(
                'writetextai/edit/index',
                ['id' => $item['entity_id'], 'store_id' => $storeId]
            );

            $item['websites'] = explode(',', $item['websites']);
        }

        /**
         * add all ids for tracking of product positions
         * in prev / next buttons in edit page
         */
        $allIds = $this->getAllIds();
        $data['ids'] = $allIds;
        
        $hideBulkSettings = $this->userFieldsSelected->getUserFieldsSelected(
            UserFieldsSelectedManager::ACTION_TYPE_HIDE_BULK_SETTINGS
        );
        $enableComparisonPopup = $this->userFieldsSelected->getUserFieldsSelected(
            UserFieldsSelectedManager::ACTION_TYPE_ENABLE_COMPARISON_POPUP
        );

        $data['custom_open_graph'] = $customOpenGraph;

        $data['grid_settings'] = [
            UserFieldsSelectedManager::ACTION_TYPE_HIDE_BULK_SETTINGS => filter_var(
                $hideBulkSettings,
                FILTER_VALIDATE_BOOLEAN
            ),
            UserFieldsSelectedManager::ACTION_TYPE_ENABLE_COMPARISON_POPUP => filter_var(
                $enableComparisonPopup,
                FILTER_VALIDATE_BOOLEAN
            ),
            'url' => $this->urlBuilder->getUrl('writetextai/grid/savesettings')
        ];

        $data['users'] = $this->getUsers();

        $user = $this->getUser();
        $data['current_user'] = [
            'id' => $user->getId(),
            'name' => $user->getFirstName() . ' ' . $user->getLastName(),
            'email' => $user->getEmail()
        ];

        $data['is_premium'] = $this->premium->getIsPremium();
        $data['total_credits'] = $this->premium->getTotalCredits();

        return $data;
    }

    /**
     * Check if language is supported
     *
     * @param string $languageCode
     * @return void
     * @throws \Exception
     */
    private function isLanguageSupported($languageCode)
    {
        if (!in_array($languageCode, StoresFilterOptions::SUPPORTED_LANGUAGES)) {
            // phpcs:ignore Generic.Files.LineLength.TooLong
            throw new \WriteTextAI\WriteTextAI\Exception\UnsupportedLanguageException(__('WriteText.ai beta is currently available for certain languages only. Please install this extension in a website with a supported language (EN, DA, DE, NO, SV, FR, ES, PT, NL, CA, IT) to proceed with beta testing.'));
        }
    }

    /**
     * Decode html entities
     *
     * @param \Magento\Catalog\Model\Product $product
     * @param array $item
     * @param array $mappingSettings
     * @return array
     */
    private function decodeHtmlEntities($product, $item, $mappingSettings)
    {
        $keys = [
            'product_name' => 'name',
            'mg_page_title' => 'page_title',
            'mg_page_description' => 'page_description',
            'mg_product_description' => 'product_description',
            'mg_short_product_description' => 'short_product_description',
            'mg_open_graph' => 'open_graph',
            'ai_page_title' => 'page_title',
            'ai_page_description' => 'page_description',
            'ai_product_description' => 'product_description',
            'ai_short_product_description' => 'short_product_description',
            'ai_open_graph' => 'open_graph',
        ];

        foreach ($keys as $key => $mapping) {
            if (isset($item[$key])) {
                $item[$key] = $this->outputHelper->productAttribute(
                    $product,
                    $item[$key],
                    $mappingSettings[$mapping] ?? $mapping
                );
            }
        }

        return $item;
    }

    /**
     * Get current user
     *
     * @return \Magento\User\Model\User
     */
    private function getUser()
    {
        return $this->apiSession->getCurrentUser();
    }

    /**
     * Get filter by WriteText.ai status
     *
     * @param array $filters
     * @return array
     */
    public function getFilterByReviewStatus($filters)
    {
        $storeId = !isset($filters['store_id']) ? self::DEFAULT_STORE_ID : $filters['store_id'];
        $locale =  $this->scopeConfig->getValue(
            'general/locale/code',
            ScopeInterface::SCOPE_STORE,
            $storeId
        );

        $statuses = [];
        $reviewStatus = $filters['review_status'];
        if (in_array('for_rewrite', $reviewStatus)) {
            $statuses[] = 'EditForRewrite';
            $statuses[] = 'EditForRewriteAndCorrection';
        }
        if (in_array('fact_check', $reviewStatus)) {
            $statuses[] = 'EditForCorrection';
            $statuses[] = 'EditForRewriteAndCorrection';
        }
        if (in_array('for_rewrite', $reviewStatus) && in_array('fact_check', $reviewStatus)) {
            $statuses[] = 'EditForRewriteAndCorrection';
        }

        $localeExplode = explode("_", $locale);
        $params = [
            "type" => "Product",
            "storeId" => $storeId,
            "language" => $localeExplode[0] ?? '',
            "fields" => $filters['ai_fields'],
            "status" => $statuses
        ];

        $result = $this->apiManager->filterReviewStatus($params);
        $recordIds = array_column($result['records'], 'recordId');
        if (isset($result['continuationToken'])) {
            do {
                $params['continuationToken'] = $result['continuationToken'];
                $result = $this->apiManager->getGeneratedStatus($params);
                $recordIds = array_merge($recordIds, array_column($result['records'], 'recordId'));
            } while (isset($result['continuationToken']));
        }

        return $recordIds;
    }

    /**
     * Get filter by WriteText.ai status
     *
     * @param array $filters
     * @return array
     */
    public function getFilterByAiStatus($filters)
    {
        $storeId = !isset($filters['store_id']) ? Store::DEFAULT_STORE_ID : $filters['store_id'];
        $locale =  $this->scopeConfig->getValue(
            'general/locale/code',
            ScopeInterface::SCOPE_STORE,
            $storeId
        );

        $params = [
            "type" => "Product",
            "storeId" => $storeId,
            "language" => str_replace('_', '-', $locale),
            "fields" => $filters['ai_fields'],
            "status" => $filters['ai_status']
        ];

        $result = $this->apiManager->getGeneratedStatus($params);
        $recordIds = array_column($result['records'], 'recordId');
        if (isset($result['continuationToken'])) {
            do {
                $params['continuationToken'] = $result['continuationToken'];
                $result = $this->apiManager->getGeneratedStatus($params);
                $recordIds = array_merge($recordIds, array_column($result['records'], 'recordId'));
            } while (isset($result['continuationToken']));
        }

        return $recordIds;
    }

    /**
     * Get all ids in grid preserving sort and filter
     *
     * @return array
     */
    protected function getAllIds()
    {
        $ids = [];
        $collection = clone $this->getSearchResult();
        $collection->load();
        $select = $collection->getSelect()
            ->reset(\Magento\Framework\DB\Select::COLUMNS)
            ->columns(['entity_id'])
            ->reset(\Magento\Framework\DB\Select::LIMIT_COUNT)
            ->reset(\Magento\Framework\DB\Select::LIMIT_OFFSET);

        $connection = $this->getSearchResult()->getResource()->getConnection();
        $ids = $connection->fetchCol($select);

        return $ids;
    }

    /**
     * Get admin users
     *
     * @return array
     */
    private function getUsers()
    {
        $collection = $this->userCollectionFactory->create()
            ->addFieldToSelect([
                'firstname',
                'lastname'
            ]);

        $users = [];
        foreach ($collection as $user) {
            $users[] = [
                'id' => $user->getUserId(),
                'name' => $user->getFirstname() . ' ' . $user->getLastname()
            ];
        }

        return $users;
    }
}
