<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\ViewModel;

use WriteTextAI\WriteTextAI\Helper\Data as DataHelper;
use Magento\Catalog\Model\ResourceModel\Eav\AttributeFactory;

class AttributeScope implements \Magento\Framework\View\Element\Block\ArgumentInterface
{
    /**
     * @var DataHelper
     */
    protected $dataHelper;

    /**
     * @var AttributeFactory
     */
    protected $attributeFactory;

    /**
     * Constructor
     *
     * @param DataHelper $dataHelper
     * @param AttributeFactory $attributeFactory
     */
    public function __construct(
        DataHelper $dataHelper,
        AttributeFactory $attributeFactory
    ) {
        $this->dataHelper = $dataHelper;
        $this->attributeFactory = $attributeFactory;
    }

    /**
     * Get attribute scope
     *
     * @param string $field
     *
     * @return array
     */
    public function getAttrScope($field)
    {
        $html = '';
        $mappingSettings = $this->dataHelper->getMappingSettings();

        if (isset($mappingSettings[$field])) {
            $attributeCode = $mappingSettings[$field];
            $attribute = $this->attributeFactory->create();
            $attribute->loadByCode(\Magento\Catalog\Model\Product::ENTITY, $attributeCode);

            if ($attribute->isScopeGlobal()) {
                $html = __('[GLOBAL]');
            } elseif ($attribute->isScopeWebsite()) {
                $html = __('[WEBSITE]');
            } elseif ($attribute->isScopeStore()) {
                $html = __('[STORE VIEW]');
            }
        }

        return $html;
    }

    public function isAttributeGlobal($scope)
    {
        return $scope == '[GLOBAL]';
    }
}
