<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\ViewModel;

use WriteTextAI\WriteTextAI\Helper\Data;
use WriteTextAI\WriteTextAI\Model\KeywordIdeaFiltersManager;

class Locations implements \Magento\Framework\View\Element\Block\ArgumentInterface
{
    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var KeywordIdeaFiltersManager
     */
    protected $keywordIdeaFiltersManager;

    /**
     * Constructor
     *
     * @param Data $helper
     * @param KeywordIdeaFiltersManager $keywordIdeaFiltersManager
     */
    public function __construct(
        Data $helper,
        KeywordIdeaFiltersManager $keywordIdeaFiltersManager
    ) {
        $this->helper = $helper;
        $this->keywordIdeaFiltersManager = $keywordIdeaFiltersManager;
    }

    /**
     * Get selected country
     *
     * @return string
     */
    public function getSelectedCountries()
    {
        return $this->helper->getSelectedCountries();
    }

    /**
     * Get available countries
     *
     * @return array
     */
    public function getAvailableCountries()
    {
        $countries = $this->keywordIdeaFiltersManager->getAvailableCountries();

        usort($countries, function ($item1, $item2) {
            return $item1['location_name'] <=> $item2['location_name'];
        });

        return $countries;
    }

    /**
     * Get options
     *
     * @return array
     */
    public function getCountryOptions()
    {
        $locations = $this->getAvailableCountries();

        $options = [];

        foreach ($locations as $location) {
            $options[] = [
                'value' => $location['country_iso_code'],
                'label' => $location['location_name']
            ];
        }

        return $options;
    }

    /**
     * Get pre-selected country
     *
     * @return array
     */
    public function getPreSelectedCountries()
    {
        return [$this->keywordIdeaFiltersManager->getCountryCode(
            \WriteTextAI\WriteTextAI\Model\Config\Source\DefaultValues\Store::DEFAULT_STORE_ID
        )];
    }
}
