<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\ViewModel;

use Magento\Framework\AuthorizationInterface;

class Permissions implements \Magento\Framework\View\Element\Block\ArgumentInterface
{
    public const TRANSFER_ACL_RESOURCE = 'WriteTextAI_WriteTextAI::transfer';

    public const GENERATE_ACL_RESOURCE = 'WriteTextAI_WriteTextAI::generate';

    public const KEYWORDS_ACL_RESOURCE = 'WriteTextAI_WriteTextAI::add_keywords';

    public const PRODUCT_DETAILS_ACL_RESOURCE = 'WriteTextAI_WriteTextAI::product_details';

    public const EDIT_ACL_RESOURCE = 'WriteTextAI_WriteTextAI::edit';

    public const ATTR_ACL_RESOURCE = 'WriteTextAI_WriteTextAI::modify_attributes';

    public const TONE_STYLE_ACL_RESOURCE = 'WriteTextAI_WriteTextAI::modify_tone_style';

    public const WORDS_ACL_RESOURCE = 'WriteTextAI_WriteTextAI::modify_words';

    public const COUNTRY_ACL_RESOURCE = 'WriteTextAI_WriteTextAI::country';

    /**
     * @var AuthorizationInterface
     */
    protected $authorization;

    /**
     * Constructor
     *
     * @param AuthorizationInterface $authorization
     */
    public function __construct(
        AuthorizationInterface $authorization
    ) {
        $this->authorization = $authorization;
    }

    /**
     * Check if transfer button is allowed
     *
     * @return bool
     */
    public function isTransferAllowed()
    {
        return $this->authorization->isAllowed(self::TRANSFER_ACL_RESOURCE);
    }

    /**
     * Get generate permission
     *
     * @return bool
     */
    public function isGenerateAllowed()
    {
        return $this->authorization->isAllowed(self::GENERATE_ACL_RESOURCE);
    }

    /**
     * Get keywords permission
     *
     * @return bool
     */
    public function isKeywordsAllowed()
    {
        return $this->authorization->isAllowed(self::KEYWORDS_ACL_RESOURCE);
    }

    /**
     * Get product details permission
     *
     * @return bool
     */
    public function isProductDetailsAllowed()
    {
        return $this->authorization->isAllowed(self::PRODUCT_DETAILS_ACL_RESOURCE);
    }

    /**
     * Get edit permission
     *
     * @return bool
     */
    public function isEditAllowed()
    {
        return $this->authorization->isAllowed(self::EDIT_ACL_RESOURCE);
    }

    /**
     * Get attribute permission
     *
     * @return bool
     */
    public function isAttrAllowed()
    {
        return $this->authorization->isAllowed(self::ATTR_ACL_RESOURCE);
    }

    /**
     * Get tone and style permission
     *
     * @return bool
     */
    public function isToneStyleAllowed()
    {
        return $this->authorization->isAllowed(self::TONE_STYLE_ACL_RESOURCE);
    }

    /**
     * Get words permission
     *
     * @return bool
     */
    public function isWordsAllowed()
    {
        return $this->authorization->isAllowed(self::WORDS_ACL_RESOURCE);
    }

    /**
     * Get country permission
     *
     * @return bool
     */
    public function isCountryAllowed()
    {
        return $this->authorization->isAllowed(self::COUNTRY_ACL_RESOURCE);
    }
}
