<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\ViewModel;

use WriteTextAI\WriteTextAI\Model\ApiManager;

class Premium implements \Magento\Framework\View\Element\Block\ArgumentInterface
{
    public const UNLOCK_PREMIUM_URL = 'https://platform.writetext.ai/premium';

    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var bool
     */
    protected $isPremium;

    /**
     * @var int
     */
    protected $totalCredits = 0;

    /**
     * Constructor
     *
     * @param ApiManager $apiManager
     */
    public function __construct(
        ApiManager $apiManager
    ) {
        $this->apiManager = $apiManager;
    }

    /**
     * Get is premium
     *
     * @return bool
     */
    public function isPremium()
    {
        if ($this->isPremium !== null) {
            return $this->isPremium;
        }

        return $this->getIsPremium();
    }

    /**
     * Set is premium
     *
     * @param bool $isPremium
     * @return void
     */
    protected function setIsPremium($isPremium)
    {
        $this->isPremium = $isPremium;
    }
    
    /**
     * Is premium
     *
     * @return bool
     */
    public function getIsPremium()
    {
        $credit = $this->apiManager->getCredit();
        $isPremium = isset($credit['isPremium']) ? (bool) $credit['isPremium'] : false;
        $this->setTotalCredits(isset($credit['availableCredits']) ? (int) $credit['availableCredits'] : 0);
        $this->setIsPremium($isPremium);

        return $isPremium;
    }

    /**
     * Get unclock premium url
     *
     * @return string
     */
    public function getUnlockPremiumUrl()
    {
        return self::UNLOCK_PREMIUM_URL;
    }

    /**
     * Get ads
     *
     * @return array
     */
    public function getAds()
    {
        return $this->apiManager->getAds();
    }

    /**
     * Set total credits
     *
     * @param int $totalCredits
     */
    public function setTotalCredits($totalCredits)
    {
        $this->totalCredits = $totalCredits;
    }

    /**
     * Get total credits
     *
     * @return int
     */
    public function getTotalCredits()
    {
        return $this->totalCredits;
    }
}
