/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'Magento_Ui/js/form/element/ui-select',
    'mage/translate',
    'Magento_Ui/js/lib/key-codes',
    'Magento_Ui/js/modal/modal'
], function ($, Component, $t, keyCodes) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            selectedPlaceholders: {
                defaultPlaceholder: $t('Search or type the country...')
            },
            originalValue: '',
            filterOptions: true,
            countryModal: null,
            countryModalSelector: '',
            formSelector: '',
            isLoading: false,
            errorMessage: $t('Please select a country.'),
            showError: false,
            gridButtonSelector: '.page-header-actions .country.action',
            contentSelector: '.writetextai-country-modal',
            classes: {
                modalClass: 'writetextai-grid-country',
                overlayClass: 'modals-overlay modals-overlay-writetextai-country'
            },
            isSet: false
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super().observe([
                'isLoading',
                'showError',
                'originalValue'
            ]);

            return this;
        },

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            this.initModal();

            this.value.subscribe(function (value) {
                self.showError(value.length < 1);
            });
        },

        /** @inheritdoc */
        filterOptionsKeydown: function (data, event) {
            var key = keyCodes[event.keyCode];

            if (key !== 'enterKey') {
                return this._super(data, event);
            }
        },

        /**
         * Initialize history log modal.
         */
        initModal: function () {
            this.isSet = self.value().length > 0 ? true : false;

            /* set preselected country */
            if (self.value().length < 1) {
                self.value(self.preSelected);
            }

            this.originalValue([...this.value()]);

            var modalOptions = {
                title: $.mage.__("Select your target country"),
                subTitle: $.mage.__('WriteText.ai will attempt to localize the generated text for your selected country/countries.'),
                modalClass: this.classes.modalClass,
                overlayClass: this.classes.overlayClass,
                autoOpen: !this.isSet,
                clickableOverlay: this.canEdit,
                buttons: [],
                closed: function () {
                    if (!self.isSet && self.canEdit) {
                        self.submitForm($(self.formSelector));
                    }
                    self.value([...self.originalValue()]);
                }
            };

            if (this.canEdit) {
                modalOptions.buttons.push({
                    text: $.mage.__('Save'),
                    class: 'action-primary',
                    click: function () {
                        self.submitForm($(self.formSelector));
                    }
                });
            } else {
                modalOptions.modalCloseBtn = false;
            }

            this.countryModal = $(this.countryModalSelector).modal(modalOptions);

            $(this.contentSelector).removeClass('no-display');
            $(this.gridButtonSelector).prop('disabled', false);
        },

        /**
         * Submit form.
         *
         * @param {Element} form
         * @returns {void}
         */
        submitForm: function (form) {
            self.showError(false);

            if (this.value().length < 1) {
                self.showError(true);
                return;
            }

            this.ajaxSubmit(form);
        },

        /**
         * Send form ajax request.
         *
         * @param {Element} form
         * @returns {void}
         */
        ajaxSubmit: function (form) {
            var formData = new FormData(form[0]),
                settings = {};

            self.isLoading(true);

            formData.append('country_id', this.value());

            settings = {
                url: form.attr('action'),
                data: formData,
                type: 'post',
                dataType: 'json',
                processData: false,
                contentType: false
            };

            $.ajax(settings)
                .done(function (response) {
                    if (response.success) {
                        self.originalValue([...self.value()]);
                        self.isSet = true;
                        $(self.contentSelector).modal('closeModal');
                    }
                })
                .always(function () {
                    self.isLoading(false);
                });
        }
    });
});
