/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'uiComponent',
    'WriteTextAI_WriteTextAI/js/model/edit',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords',
    'WriteTextAI_WriteTextAI/js/model/edit/attributes',
    'WriteTextAI_WriteTextAI/js/model/edit/textfields',
    'WriteTextAI_WriteTextAI/js/model/edit/mark-reviewed',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'WriteTextAI_WriteTextAI/js/model/edit/audience',
    'WriteTextAI_WriteTextAI/js/model/edit/custom-tone-style',
    'WriteTextAI_WriteTextAI/js/model/edit/ideas-filters',
    'WriteTextAI_WriteTextAI/js/model/edit/settings',
    'WriteTextAI_WriteTextAI/js/model/edit/reference-products',
    'WriteTextAI_WriteTextAI/js/model/edit/error-messages',
    'WriteTextAI_WriteTextAI/js/model/edit/generate/keyword-analysis',
    'WriteTextAI_WriteTextAI/js/grid/reload',
    'WriteTextAI_WriteTextAI/js/model/grid/notifications',
    'WriteTextAI_WriteTextAI/js/model/history-log',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/utils/edit/gallery',
    'WriteTextAI_WriteTextAI/js/model/edit/review-status',
    'WriteTextAI_WriteTextAI/js/signalr',
    'WriteTextAI_WriteTextAI/js/grid/progressbar',
    'WriteTextAI_WriteTextAI/js/edit/reference-products',
    'WriteTextAI_WriteTextAI/js/edit/keywords',
    'WriteTextAI_WriteTextAI/js/modal/modal',
    'Magento_Ui/js/modal/modal',
    'mage/calendar',
    'accordion'
], function (
    $,
    Component,
    editData,
    keywords,
    attributes,
    textfields,
    markReviewed,
    product,
    audience,
    customToneStyle,
    ideasFilters,
    settings,
    referenceProducts,
    errorMessagesModel,
    genKeywordAnalysis,
    reloadGrid,
    notifications,
    historyLog,
    totalCredits,
    premium,
    galleryUtils,
    reviewStatus,
    signalRComponent,
    progressbarComponent,
    referenceProductsComponent,
    keywordsComponent
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            modalSelector: '',
            modal: null,
            accordionsInitialized: false,
            tooltipsInitialized: false,
            widgetsInitialized: false,
            ajaxRequests: 0,
            pageTitleAjaxRequests: 0,
            pageDescriptionAjaxRequests: 0,
            productDescriptionAjaxRequests: 0,
            shortDescriptionAjaxRequests: 0,
            openGraphAjaxRequests: 0,
            galleryAjaxRequests: 0,
            analysisAjaxRequests: 0,
            audienceAjaxRequests: 0,
            productId: '',
            storeId: '',
            isLoading: false,
            selectors: {
                mainModalSelector: '.writetextai-edit-modal.writetextai-edit-main',
                mainModalClass: 'writetextai-edit-modal writetextai-edit-main',
                accordionSelector: '.writetextai-edit-modal-wrapper .accordions',
                tooltipSelector:
                    '.writetextai-edit-modal-wrapper .admin__field-tooltip-action',
                reviewStatusPopupSelector: '.review_status-tooltip .tooltip-toggle',
                collapsibleSelector:
                    '.writetextai-toolbar .custom-action-select-wrap',
                toggleSelectAllSelector:
                    '.writetextai-toolbar input[name=select_all]',
                targetCheckboxSelector:
                    '.writetextai-edit-modal-wrapper .fieldset.text-editor input[type=checkbox]'
            },
            statusRunning: 'Running',
            statuses: []
        },

        gettingData: null,

        /** @inheritdoc */
        initObservable: function () {
            this._super().observe([
                'productId',
                'storeId',
                'statuses',
                'isLoading'
            ]);

            return this;
        },

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            this.initModal();

            $(document).on({
                ajaxSend: this._onAjaxSend.bind(this),
                ajaxComplete: this._onAjaxComplete.bind(this)
            });

            editData.currentProduct.subscribe(function (currentProduct) {
                self.productId(currentProduct.productId);
                self.storeId(currentProduct.storeId);
                $(self.modalSelector).editModal('openModal');
            });

            notifications.statuses.subscribe(function (statuses) {
                this.statuses(statuses);

                let productId = self.productId(),
                    disabled = statuses.some((status) => {
                        let isQueued =
                            status.status === self.statusRunning &&
                            (status.queuedIds.indexOf(productId) !== -1 ||
                                status.completedIds.indexOf(productId) !== -1);

                        return isQueued;
                    });

                if (disabled) {
                    self.showLoader();
                    self.showAllFieldLoaders();
                } else {
                    self.hideLoader();
                    self.hideAllFieldLoaders();
                }
            }, this);
        },

        /**
         * Initialize edit modal.
         */
        initModal: function () {
            var prev = 1,
                next = 1;

            this.modal = $(this.modalSelector).editModal({
                type: 'slide',
                title: 'WriteText.ai',
                modalClass: self.selectors.mainModalClass,
                overlayClass: 'modals-overlay writetextai-edit-modal-overlay',
                innerScroll: true,
                buttons: [
                    {
                        text: $.mage.__('View'),
                        class: 'link-preview action',
                        attr: {},
                        click: function () {
                            self.linkPreview(this);
                        }
                }, {
                    text: $.mage.__('History log'),
                    class: 'history-log action',
                    attr: {},
                    click: function () {
                        self.openHistoryLog();
                    }
                }, {
                    text: $.mage.__('Previous'),
                    class: 'action-previous action-default',
                    attr: {
                        'data-prev': prev,
                        'data-bind': 'disable: 1'
                    },
                    click: function () {
                        self.showPrevious();
                    }
                }, {
                    text: $.mage.__('Next'),
                    class: 'action-next action-default',
                    attr: { 'data-next': next },
                    click: function () {
                        self.showNext();
                    }
                }
                ],
                opened: function () {
                    self.afterOpen();
                },
                closed: function () {
                    self.afterClose();
                }
            });

            $(self.selectors.mainModalSelector + ' .page-main-actions .page-actions-buttons')
                .append('<div class="writetextai-edit-loading-mask show"></div>');
        },

        /**
         * Reload grid.
         */
        gridReload: function () {
            var showLoader = false;

            reloadGrid.reloadUIComponent(
                'products_grid_listing.products_grid_listing_data_source',
                showLoader
            );
        },

        /**
         * Open preview.
         *
         * @param {Object} modal
         * @returns {void}
         */
        linkPreview: function (modal) {
            $.each(modal.buttons, function (index, button) {
                if ($(button).is('.link-preview')) {
                    let link = $(button).data('link');

                    window.open(link, '_blank');
                }
            });
        },

        /**
         * Open history log modal.
         */
        openHistoryLog: function () {
            $('.writetextai-history-log-modal').modal('openModal');
        },

        /**
         * Show previous product.
         *
         * @returns {void}
         */
        showPrevious: function () {
            var prevProductId = this.getProductByIndex('prev');

            self.resetPage();

            if (prevProductId) {
                self.productId(prevProductId);
                self.getModalData();
            }
        },

        /**
         * Show next product.
         *
         * @returns {void}
         */
        showNext: function () {
            var nextProductId = this.getProductByIndex('next');

            self.resetPage();

            if (nextProductId) {
                self.productId(nextProductId);
                self.getModalData();
            }
        },

        /**
         * Disable generating product.
         */
        disableGeneratingProduct: function (productId) {
            var disabled = self.statuses().some((status) => {
                let isQueued =
                    status.status === self.statusRunning &&
                    (status.queuedIds.indexOf(productId) !== -1 ||
                        status.completedIds.indexOf(productId) !== -1);

                return isQueued;
            });

            if (disabled) {
                self.showLoader();
            } else {
                self.hideLoader();
            }
        },

        /**
         * After open edit modal actions.
         */
        afterOpen: function () {
            progressbarComponent.prototype.seeLess();
            $('.modal-content').trigger('contentUpdated');

            if (!this.widgetsInitialized) {
                this.initAccordion();
                this.initTooltip();
                editData.opened(true);
            }

            this.ajaxRequests = 0;
            this.pageTitleAjaxRequests = 0;
            this.pageDescriptionAjaxRequests = 0;
            this.productDescriptionAjaxRequests = 0;
            this.shortDescriptionAjaxRequests = 0;
            this.openGraphAjaxRequests = 0;
            this.galleryAjaxRequests = 0;
            this.analysisAjaxRequests = 0;
            this.audienceAjaxRequests = 0;
            this.getModalData();

            genKeywordAnalysis.views(0);
        },

        /**
         * After closing edit modal actions.
         */
        afterClose: function () {
            self.resetPage();
            self.gridReload();
            editData.opened(false);
            signalRComponent.prototype.hideMessage();
            self.showPreloader();
        },

        /**
         * Get edit modal data.
         */
        getModalData: function () {
            var productId = self.productId();

            self.showPreloader();

            errorMessagesModel.messages([]);

            self.disableGeneratingProduct(productId);

            historyLog.update(false);

            if (self.gettingData !== null) {
                self.gettingData.abort();
            }

            self.gettingData = $.ajax({
                url: self.editUrl,
                dataType: 'json',
                data: {
                    form_key: window.FORM_KEY,
                    id: productId,
                    store_id: self.storeId()
                },
                showWriteTextAILoader: true,
                hideWriteTextTitleLoader: true,
                success: function (response) {
                    if (response.success) {
                        self.updateModal(response);
                    } else {
                        errorMessagesModel.messages.push(response.message);
                    }
                    
                    self.hidePreloader();
                },
                error: function (xhr, status, error) {
                    if (status !== 'abort') {
                        errorMessagesModel.messages.push(error);
                    }
                }
            });
        },

        /**
         * Update edit modal data.
         *
         * @param {Object} data
         * @returns {void}
         */
        updateModal: function (data) {
            $(self.selectors.mainModalSelector + ' .product-name').html(
                data.product_name
            );
            $(self.selectors.mainModalSelector + ' .sku').text(data.sku);

            self.updatePremium(data);
            self.updateTotalCredits(data);
            self.updateButtons(data);
            self.updateTextFields(data);
            self.updateTextfieldsStatuses(data);
            self.updateUseDefault(data);
            self.updatePronouns(data);
            self.updateKeywords(data);
            self.updateAttributes(data);
            self.updateHistoryLog(data);
            self.updateNewTextFields(data);
            self.updateMarkReviewed(data);
            self.updateProduct(data);
            self.updateAudience(data);
            self.updateCustomToneStyle(data);
            self.updateIdeasFilters(data);
            self.updateSelectedFields(data);
            self.updateSettings(data);
            self.updateReviewStatus(data);
            self.bindStoreSwitcher();
            galleryUtils.updateImages(
                data.gallery,
                data.images,
                data.images_selected,
                data.images_selected_available
            );

            $('.writetextai-edit-modal-wrapper').trigger('contentUpdated');
        },

        /**
         * Bind store switcher.
         */
        bindStoreSwitcher: function () {
            $('.writetextai-store-scope li a').on('click', function () {
                self.resetPage();
                self.storeId($(this).data('value'));
                self.getModalData();
                $('.writetextai-store-scope .dropdown').collapsible(
                    'deactivate'
                );
            });
        },

        /**
         * @param {Object} data
         * @returns {void}
         */
        updateReviewStatus: function (data) {
            reviewStatus.reviewStatus(data.review_status);
        },

        /**
         * @param {Object} data
         * @returns {void}
         */
        updatePremium: function (data) {
            premium.isPremium(data.is_premium);
        },

        /**
         * @param {Object} data
         * @returns {void}
         */
        updateTotalCredits: function (data) {
            totalCredits.totalCredits(data.total_credits);
        },

        /**
         * Update use default data.
         * 
         * @param {*} data 
         */
        updateUseDefault: function (data) {
            textfields.useDefault({
                pageTitle: data.use_default.page_title,
                pageDescription: data.use_default.page_description,
                productDescription: data.use_default.product_description,
                productShortDescription: data.use_default.short_product_description,
                openGraph: data.use_default.open_graph
            });
        },

        /**
         * @param {Object} data
         * @returns {void}
         */
        updatePremium: function (data) {
            premium.isPremium(data.is_premium);
        },

        /**
         * @param {Object} data
         * @returns {void}
         */
        updateSettings: function (data) {
            /**
             * Important: Always updates styles first before tones
             * to prioritize in redering
             */
            settings.disallowedCombinations(data.disallowed_combinations);
            settings.styles(data.styles);
            settings.tones(data.tones);
            settings.audience(data.audience);
            settings.descMin(data.desc_min);
            settings.descMax(data.desc_max);
            settings.excMin(data.exc_min);
            settings.excMax(data.exc_max);
            settings.credits(data.credits);
            settings.formalLanguageSupport(data.formal_language_support);
            settings.formalLanguages(data.formal_languages);
            settings.guideSteps(data.guide_steps);
        },

        /**
         * @param {Object} data
         * @returns {void}
         */
        updateCustomToneStyle: function (data) {
            customToneStyle.customTone(data.custom_tone);
            customToneStyle.customStyle(data.custom_style);
        },
        
        /**
         * @param {Object} data
         * @returns {void}
         */
        updateSelectedFields: function (data) {
            textfields.selectedFields([]);
            for (let key in data.selected_fields) {
                if (data.selected_fields.hasOwnProperty(key)) {
                    textfields.selectedFields.push(data.selected_fields[key]);
                }
            }
        },

        /**
         * @param {Object} data
         * @returns {void}
         */
        updateTextfieldsStatuses: function (data) {
            textfields.statuses({
                pageTitleGenerateStatus: data.textfields_statuses.page_title_generated,
                pageDescriptionGenerateStatus: data.textfields_statuses.page_description_generated,
                productDescriptionGenerateStatus: data.textfields_statuses.product_description_generated,
                productShortDescriptionGenerateStatus: data.textfields_statuses.short_product_description_generated,
                openGraphGenerateStatus: data.textfields_statuses.open_graph_generated,
                pageTitleTransferStatus: data.textfields_statuses.page_title_transferred,
                pageDescriptionTransferStatus: data.textfields_statuses.page_description_transferred,
                productDescriptionTransferStatus: data.textfields_statuses.product_description_transferred,
                productShortDescriptionTransferStatus: data.textfields_statuses.short_product_description_transferred,
                openGraphTransferStatus: data.textfields_statuses.open_graph_transferred
            });
        },

        /**
         * @param {Object} data
         * @returns {void}
         */
        updateIdeasFilters: function (data) {
            ideasFilters.selectedCountry(data.keyword_ideas_filters.location);
            ideasFilters.sortAndFilters(data.keyword_ideas_filters);
        },

        /**
         * @param {Object} data
         * @returns {void}
         */
        updateAudience: function (data) {
            audience.selectedAudience(data.selected_audience.shift());
            audience.suggestedAudience(data.suggested_audience);
        },

        /**
         * @param {Object} data
         * @returns {void}
         */
        updateProduct: function (data) {
            product.productId(data.product_id);
            product.storeId(data.store_id);
            product.storeScope(data.store_scope);
            product.language(data.language.toLowerCase());
            product.stores(data.stores);
            product.directiveRegex(data.directiveRegex);
            product.descriptionDirectiveExists(
                data.product_description_directive_exists
            );
            product.shortDescriptionDirectiveExists(
                data.product_short_description_directive_exists
            );
            product.refresh(data.refresh);
        },

        /**
         * @param {Object} data
         * @returns {void}
         */
        updateMarkReviewed: function (data) {
            markReviewed.reviewed(data.reviewed);
        },

        /**
         * @param {Object} data
         * @returns {void}
         */
        updateNewTextFields: function (data) {
            textfields.pageTitle(data.text_fields.page_title);
            textfields.pageDescription(data.text_fields.page_description);
            textfields.productDescription(data.text_fields.product_description);
            textfields.productShortDescription(
                data.text_fields.short_product_description
            );
            textfields.openGraph(data.text_fields.open_graph);
            textfields.originalPageTitle(data.text_fields.page_title);
            textfields.originalPageDescription(
                data.text_fields.page_description
            );
            textfields.originalProductDescription(
                data.text_fields.product_description
            );
            textfields.originalProductShortDescription(
                data.text_fields.short_product_description
            );
            textfields.originalOpenGraph(data.text_fields.open_graph);
        },

        /**
         * @param {Object} data
         * @returns {void}
         */
        updatePronouns: function (data) {
            textfields.highlightPronouns(
                JSON.parse(data.highlight_pronouns.toLowerCase())
            );
        },

        /**
         * @param {Object} data
         * @returns {void}
         */
        updateKeywords: function (data) {
            keywords.targetKeywords(data.keywords.texts);
            keywords.selectedSemanticKeywords(data.keywords.selected);
            keywords.highlighted(JSON.parse(data.highlighted.toLowerCase()));
            keywords.language(data.language);
        },

        /**
         * @param {Object} data
         * @returns {void}
         */
        updateAttributes: function (data) {
            attributes.otherProductDetails(data.other_details);
            attributes.otherProductDetailsChecked(data.other_details_checked);
            attributes.productAttributes([]);
            for (let key in data.product_attributes) {
                if (data.product_attributes.hasOwnProperty(key)) {
                    attributes.productAttributes.push(
                        data.product_attributes[key]
                    );
                }
            }
            /**
             * Always update product attribute list first before the selected
             * to prioritize preference over global settings
             */
            attributes.productAttributesSelected(data.product_attributes_selected);
        },

        /**
         * @param {Object} data
         * @returns {void}
         */
        updateTextFields: function (data) {
            textfields.mgPageTitle(data.product_meta_title);
            textfields.mgProductDescription(data.product_description);
            textfields.mgProductShortDescription(
                data.product_short_description
            );
            textfields.mgPageDescription(data.product_meta_description);
            textfields.mgOpenGraph(data.open_graph);
        },

        /**
         * @param {Object} data
         * @returns {void}
         */
        updateHistoryLog: function (data) {
            historyLog.productId(data.product_id);
            historyLog.storeId(data.store_id);
            historyLog.users([]);

            for (let key in data.users) {
                if (data.users.hasOwnProperty(key)) {
                    historyLog.users.push(data.users[key]);
                }
            }
        },

        /**
         * @param {Object} data
         * @returns {void}
         */
        updateButtons: function (data) {
            /**
             * next
             */
            if (this.getProductByIndex('next')) {
                $(this.selectors.mainModalSelector + ' .action-next').prop(
                    'disabled',
                    false
                );
            } else {
                $(this.selectors.mainModalSelector + ' .action-next').prop(
                    'disabled',
                    true
                );
            }

            /**
             * prev
             */
            if (this.getProductByIndex('prev')) {
                $(this.selectors.mainModalSelector + ' .action-previous').prop(
                    'disabled',
                    false
                );
            } else {
                $(this.selectors.mainModalSelector + ' .action-previous').prop(
                    'disabled',
                    true
                );
            }

            /**
             * link preview
             */
            $(this.selectors.mainModalSelector + ' .link-preview').data(
                'link',
                data.link_preview_url
            );

            /**
             * preview changes
             */
            $(this.selectors.mainModalSelector + ' .preview-button').data(
                'link',
                data.preview_changes_url
            );

            /**
             * Toggle preview visibility
             */
            if (data.visibility) {
                $(this.selectors.mainModalSelector + ' .link-preview').removeClass('no-display');
                $(this.selectors.mainModalSelector + ' .preview-button').removeClass('no-display');
                $(this.selectors.mainModalSelector + ' .preview-button + .tooltip-wrapper').removeClass('no-display');
            } else {
                $(this.selectors.mainModalSelector + ' .link-preview').addClass('no-display');
                $(this.selectors.mainModalSelector + ' .preview-button').addClass('no-display');
                $(this.selectors.mainModalSelector + ' .preview-button + .tooltip-wrapper').addClass('no-display');
            }
        },

        /**
         * Reset edit modal.
         */
        resetPage: function () {
            var data = {
                product_name: '',
                text_fields: {
                    page_title: '',
                    page_description: '',
                    product_description: '',
                    short_product_description: '',
                    open_graph: ''
                },
                product_meta_title: '',
                product_description: '',
                product_short_description: '',
                product_meta_description: '',
                open_graph: ''
            };

            $(self.selectors.mainModalSelector + ' .product-name').html('');
            $(self.selectors.mainModalSelector + ' .sku').html('');

            self.updateTextFields(data);
            self.updateNewTextFields(data);

            historyLog.update(false);
            markReviewed.reviewed(false);
            referenceProducts.referenceProductChecked(false);
            referenceProducts.referenceProducts([]);
            referenceProducts.referenceProductTotal(0);
            referenceProductsComponent.prototype.resetProductsLoaded();
            referenceProductsComponent.prototype.resetValue();
            keywordsComponent.prototype.resetKeywordIdeasLoaded();
            product.storeScope('');
            keywords.suggestedKeywords([]);
        },

        /**
         * Initialize edit modal accordions.
         */
        initAccordion: function () {
            $(self.selectors.accordionSelector).each(function () {
                var active = [],
                    activeValues = {
                        'keywords-attributes': [0, 1],
                        'fieldset': [0, 1, 2, 3, 4],
                        'writetextai-gallery': [0]
                },
                    elementClasses = $(this).attr('class').split(/\s+/);
                    
                $.each(elementClasses, function (index, className) {
                    if (activeValues.hasOwnProperty(className)) {
                        active = activeValues[className];
                    }
                });

                $(this).accordion({
                    active: active,
                    collapsible: true,
                    openedState: '_show',
                    closedState: '_hide',
                    multipleCollapsible: true
                });
            });
        },

        /**
         * Initialize edit modal tooltips.
         */
        initTooltip: function () {
            $(self.selectors.tooltipSelector).dropdown({
                parent: '.admin__field-tooltip',
                autoclose: true
            });
            $(self.selectors.reviewStatusPopupSelector).dropdown({
                parent: '.review_status-tooltip',
                autoclose: true
            });
        },

        /**
         * Get product by index.
         */
        getProductByIndex: function (type) {
            var ids = editData.ids(),
                currentIndex = ids.indexOf(self.productId());

            if (!ids || ids.length === 0) {
                return false;
            }

            let offset = (type === 'next') ? 1 : (type === 'prev') ? -1 : 0;
            let product = ids[currentIndex + offset];

            return product || false;
        },

        /**
         * Show preloader
         */
        showPreloader: function () {
            self.isLoading(true);
        },

        /**
         * Hide preloader
         */
        hidePreloader: function () {
            self.isLoading(false);
        },

        /**
         * Show field loader
         *
         * @param {String} field
         * @returns {void}
         */
        showFieldLoader: function (field) {
            $('.writetextai-' + field + '-loading-mask').addClass('show');
        },

        /**
         * Hide field loader
         *
         * @param {String} field
         * @returns {void}
         */
        hideFieldLoader: function (field) {
            $('.writetextai-' + field + '-loading-mask').removeClass('show');
        },

        /**
         * Show all field loaders
         */
        showAllFieldLoaders: function () {
            self.showFieldLoader('page-title');
            self.showFieldLoader('page-description');
            self.showFieldLoader('product-description');
            self.showFieldLoader('short-description');
            self.showFieldLoader('open-graph');
            self.showFieldLoader('gallery');
        },

        /**
         * Hide all field loaders
         */
        hideAllFieldLoaders: function () {
            self.hideFieldLoader('page-title');
            self.hideFieldLoader('page-description');
            self.hideFieldLoader('product-description');
            self.hideFieldLoader('short-description');
            self.hideFieldLoader('open-graph');
            self.hideFieldLoader('gallery');
        },

        /**
         * Show loader
         * 
         * @param {Boolean} loadTitle
         */
        showLoader: function (loadTitle = true) {
            $('.writetextai-edit-loading-mask').addClass('show');
            if (loadTitle) {
                $('.writetextai-edit-modal .modal-title').addClass('loading');
            }
        },

        /**
         * hide loader
         */
        hideLoader: function () {
            $('.writetextai-edit-modal .modal-title').removeClass('loading');
            $('.writetextai-edit-loading-mask').removeClass('show');
        },

        /**
         * @param {Object} event
         * @param {Object} xhr
         * @param {Object} config
         * @returns {void}
         */
        _onAjaxSend: function (event, xhr, config) {
            if (this.modal && config && config.showWriteTextAILoader) {
                this.showLoader(!config.hideWriteTextTitleLoader);
                this.ajaxRequests++;

                if (config.showPageTitleLoader) {
                    self.showFieldLoader('page-title');
                    this.pageTitleAjaxRequests++;
                }
                if (config.showPageDescriptionLoader) {
                    self.showFieldLoader('page-description');
                    this.pageDescriptionAjaxRequests++;
                }
                if (config.showProductDescriptionLoader) {
                    self.showFieldLoader('product-description');
                    this.productDescriptionAjaxRequests++;
                }
                if (config.showShortDescriptionLoader) {
                    self.showFieldLoader('short-description');
                    this.shortDescriptionAjaxRequests++;
                }
                if (config.showOpenGraphLoader) {
                    self.showFieldLoader('open-graph');
                    this.openGraphAjaxRequests++;
                }
                if (config.showGalleryLoader) {
                    self.showFieldLoader('gallery');
                    this.galleryAjaxRequests++;
                }
                if (config.showAnalysisLoader) {
                    self.showFieldLoader('analysis');
                    this.analysisAjaxRequests++;
                }
                if (config.showAudienceLoader) {
                    self.showFieldLoader('audience');
                    this.audienceAjaxRequests++;
                }
            }
        },

        /**
         * @param {Object} event
         * @param {Object} xhr
         * @param {Object} config
         * @returns {void}
         */
        _onAjaxComplete: function (event, xhr, config) {
            if (this.modal && config && config.showWriteTextAILoader) {
                if (this.ajaxRequests > 0) {
                    this.ajaxRequests--;
                }

                setTimeout(function () {
                    if (self.ajaxRequests === 0) {
                        self.hideLoader();
                    }
                }, 200);

                if (config.showPageTitleLoader && this.pageTitleAjaxRequests > 0) {
                    this.pageTitleAjaxRequests--;
                }
                if (config.showPageDescriptionLoader && this.pageDescriptionAjaxRequests > 0) {
                    this.pageDescriptionAjaxRequests--;
                }
                if (config.showProductDescriptionLoader && this.productDescriptionAjaxRequests > 0) {
                    this.productDescriptionAjaxRequests--;
                }
                if (config.showShortDescriptionLoader && this.shortDescriptionAjaxRequests > 0) {
                    this.shortDescriptionAjaxRequests--;
                }
                if (config.showOpenGraphLoader && this.openGraphAjaxRequests > 0) {
                    this.openGraphAjaxRequests--;
                }
                if (config.showGalleryLoader && this.galleryAjaxRequests > 0) {
                    this.galleryAjaxRequests--;
                }
                if (config.showAnalysisLoader && this.analysisAjaxRequests > 0) {
                    this.analysisAjaxRequests--;
                }
                if (config.showAudienceLoader && this.audienceAjaxRequests > 0) {
                    this.audienceAjaxRequests--;
                }

                if (config.showPageTitleLoader && this.pageTitleAjaxRequests === 0) {
                    self.hideFieldLoader('page-title');
                }
                if (config.showPageDescriptionLoader && this.pageDescriptionAjaxRequests === 0) {
                    self.hideFieldLoader('page-description');
                }
                if (config.showProductDescriptionLoader && this.productDescriptionAjaxRequests === 0) {
                    self.hideFieldLoader('product-description');
                }
                if (config.showShortDescriptionLoader && this.shortDescriptionAjaxRequests === 0) {
                    self.hideFieldLoader('short-description');
                }
                if (config.showOpenGraphLoader && this.openGraphAjaxRequests === 0) {
                    self.hideFieldLoader('open-graph');
                }
                if (config.showGalleryLoader && this.galleryAjaxRequests === 0) {
                    self.hideFieldLoader('gallery');
                }
                if (config.showAnalysisLoader && this.analysisAjaxRequests === 0) {
                    self.hideFieldLoader('analysis');
                }
                if (config.showAudienceLoader && this.audienceAjaxRequests === 0) {
                    self.hideFieldLoader('audience');
                }
            }
        }
    });
});
