/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    "jquery",
    "ko",
    "mage/translate",
    "uiComponent",
    "underscore",
    'WriteTextAI_WriteTextAI/js/model/edit/mark-reviewed',
    'WriteTextAI_WriteTextAI/js/model/edit/generate/keyword-analysis',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/model/signalr',
    'WriteTextAI_WriteTextAI/js/utils/edit/generate',
    'WriteTextAI_WriteTextAI/js/edit/toolbar',
    'WriteTextAI_WriteTextAI/js/model/edit/invalid-image-popup',
    'WriteTextAI_WriteTextAI/js/model/edit/gallery',
    'WriteTextAI_WriteTextAI/js/model/edit/error-messages',
    "Magento_Ui/js/modal/modal"
], function (
    $,
    ko,
    $t,
    Component,
    _,
    markReviewed,
    genKeywordAnalysis,
    premium,
    signalRModel,
    generateUtils,
    toolbar,
    invalidImagePopupModel,
    gallery,
    errorMessagesModel
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            generateParams: {}
        },

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            $('.writetextai-invalid-image-popup > *').applyBindings(this);

            this.initModal();

            invalidImagePopupModel.data.subscribe(function (value) {
                this.generateParams(value);
            }, this);
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super()
                .observe([
                    'generateParams'
                ]);
            
            this.proceedable = ko.computed(function () {
                return gallery.proceedable();
            }, this);

            this.failedImages = ko.computed(function () {
                return gallery.failedImages();
            });

            return this;
        },

        initModal: function () {
            $('.writetextai-invalid-image-popup').modal({
                type: 'popup',
                responsive: true,
                title: $t('The following image/s are not supported:'),
                modalClass: 'writetextai-invalid-image-popup-modal',
                overlayClass: 'modals-overlay writetextai-invalid-image-popup-overlay',
                buttons: [{
                    text: $t('Cancel'),
                    class: 'action-secondary action-dismiss',
                    click: function () {
                        this.closeModal();
                    }
                }, {
                    text: $t('Proceed'),
                    class: 'action-primary',
                    click: function () {
                        self._generate(self.generateParams());
                        this.closeModal();
                    }
                }],
                closed: function () {
                    $('.modals-overlay.writetextai-invalid-image-popup-overlay').remove();
                },
                opened: function () {
                    $('.writetextai-invalid-image-popup-modal .modal-footer').toggle(gallery.proceedable());
                }
            });
        },

        /**
         * Generate
         */
        _generate: function (data) {
            /* do not rewrite empty fields */
            if (data.rewrite) {
                data.selected_fields = generateUtils.removeEmptyFields(
                    data.selected_fields
                );
            }

            /* empty text fields */
            generateUtils.clearFields(data.selected_fields, data.selected_images);

            /* remove error messages */
            errorMessagesModel.messages([]);

            $.ajax({
                url: self.generateUrl,
                type: 'POST',
                data: data,
                dataType: 'json',
                showWriteTextAILoader: true,
                showPageTitleLoader: data.selected_fields.includes('page_title'),
                showPageDescriptionLoader: data.selected_fields.includes('page_description'),
                showProductDescriptionLoader: data.selected_fields.includes('product_description'),
                showShortDescriptionLoader: data.selected_fields.includes('short_product_description'),
                showOpenGraphLoader: data.selected_fields.includes('open_graph'),
                showGalleryLoader: data.selected_images.length > 0,
                success: function (response) {
                    premium.isPremium(response.is_premium);
                    if (response.success) {
                        markReviewed.reviewed(response.reviewed);
                        genKeywordAnalysis.views(0);
                    } else {
                        toolbar.prototype.getGenerated(data.selected_fields);
                        errorMessagesModel.messages.push(response.message);
                        signalRModel.generating(false);

                        if (response.unauthorized) {
                            window.location.href = response.login_url;
                        }
                    }
                }
            });
        }
    });
});
