/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'uiComponent',
    'ko',
    'mage/translate',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'WriteTextAI_WriteTextAI/js/model/edit/ideas-filters',
    'WriteTextAI_WriteTextAI/js/model/edit/settings',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'WriteTextAI_WriteTextAI/js/model/edit/audience',
    'underscore'
], function (
    $,
    Component,
    ko,
    $t,
    keywords,
    product,
    ideasFilters,
    settings,
    premium,
    prompt,
    totalCredits,
    audience,
    _
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            modalSelector: '.keyword-analysis',
            toggleSelectAllSelector: '#difficulty-all',
            mainModalSelector: '.writetextai-edit-modal.writetextai-edit-main',
            /* eslint-disable-next-line max-len */
            refreshMessage: $t("Keyword data are refreshed at the start of every month. The data you're seeing might have changed since you last requested it. Click Get data to refresh your keyword data."),
            targetKeywords: [],
            suggestedKeywords: [],
            newKeyword: '',
            productId: '',
            storeId: '',
            availableCountries: [],
            selectedCountry: '',
            filterByCompetition: ['LOW', 'MEDIUM', 'HIGH'],
            defaultFilterByCompetition: ['LOW', 'MEDIUM', 'HIGH'],
            filterBySearchVolumeMinimum: 0,
            filterBySearchVolumeMaximum: null,
            competitionSort: '',
            volumeSort: '',
            currentPage: 1,
            filterBySearchVolume: '{"min": 0, "max": null}',
            defaultFilterBySearchVolume: '{"min": 0, "max": null}',
            totalPages: 0,
            pageSize: 10,
            credits: [],
            language: '',
            defaultLocation: '',
            keywordSort: '',
            canFetchData: true,
            refresh: '',
            errorMessage: '',
            isPremium: true
        },

        isSelectedCountryChange: false,
        isFilterByCompetitionChange: false,
        isFilterBySearchVolumeMinimumChange: false,
        isFilterBySearchVolumeMaximumChange: false,
        isTargetKeywordsChange: false,
        originalSelectedCountry: '',
        gettingData: null,

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            this.bindCheckboxToggle();

            $('.writetextai-keyword-analysis-content > *').applyBindings(this);

            premium.isPremium.subscribe(function (isPremium) {
                self.isPremium(isPremium);
            });

            product.productId.subscribe(function (productId) {
                self.productId(productId);
            });

            product.storeId.subscribe(function (storeId) {
                self.storeId(storeId);
            });

            product.refresh.subscribe(function (refresh) {
                self.refresh(refresh);
            });

            keywords.suggestedKeywords.subscribe(function (suggestedKeywords) {
                if (suggestedKeywords.length === 0) {
                    self.totalPages(0);
                }
                self.suggestedKeywords(suggestedKeywords);
            });



            ideasFilters.selectedCountry.subscribe(function (selectedCountry) {
                var regExp = /[a-zA-Z]/g,
                    availableCountries = self.availableCountries(),
                    defaultCountry = availableCountries.find(
                        (country) =>
                        country.country_iso_code === selectedCountry
                    );

                /* If value is a country code, then get the location code */
                if (regExp.test(selectedCountry)) {
                    selectedCountry = defaultCountry.location_code;
                }

                self.isSelectedCountryChange = true;
                self.selectedCountry(selectedCountry);
                self.originalSelectedCountry(selectedCountry);
                self.isSelectedCountryChange = false;
            });

            ideasFilters.sortAndFilters.subscribe(function (data) {
                self.canFetchData = false;
                if (data.hasOwnProperty('competition')) {
                    self.isFilterByCompetitionChange = true;
                    self.filterByCompetition(data.competition);
                    self.isFilterByCompetitionChange = false;
                }
                if (data.hasOwnProperty('search_volume_minimum')) {
                    self.isFilterBySearchVolumeMinimumChange = true;
                    self.filterBySearchVolumeMinimum(
                        data.search_volume_minimum
                    );
                    self.isFilterBySearchVolumeMinimumChange = false;
                }
                if (data.hasOwnProperty('search_volume_maximum')) {
                    self.isFilterBySearchVolumeMaximumChange = true;
                    self.filterBySearchVolumeMaximum(
                        data.search_volume_maximum
                    );
                    self.isFilterBySearchVolumeMaximumChange = false;
                }
                if (data.hasOwnProperty('competition_sort')) {
                    self.competitionSort(data.competition_sort);
                }
                if (data.hasOwnProperty('volume_sort')) {
                    self.volumeSort(data.volume_sort);
                }
                if (data.hasOwnProperty('keyword_sort')) {
                    self.keywordSort(data.keyword_sort);
                }
                self.canFetchData = true;

                self.pageSize(10);
            });

            keywords.targetKeywords.subscribe(function (targetKeywords) {
                if (targetKeywords !== undefined) {
                    self.isTargetKeywordsChange = true;
                    self.targetKeywords(targetKeywords);
                    self.isTargetKeywordsChange = false;
                }
            });

            keywords.language.subscribe(function (language) {
                self.language(language);
            });

            self.targetKeywords.subscribe(function (targetKeywords) {
                if (!self.isTargetKeywordsChange) {
                    keywords.targetKeywords(targetKeywords);
                }
            });

            self.filterBySearchVolume.subscribe(function () {
                var volumesString = self.filterBySearchVolume(),
                    volumesObject = JSON.parse(volumesString);

                self.filterBySearchVolumeMinimum(volumesObject.min);
                self.filterBySearchVolumeMaximum(volumesObject.max);

                self.pageSize(10);
                if (self.canFetchData) {
                    self.getData(true, false);
                }
            });

            self.filterByCompetition.subscribe(function () {
                self.pageSize(10);
                if (self.canFetchData) {
                    self.getData(true, false);
                }
            });

            settings.credits.subscribe(function (credits) {
                self.credits(credits);
            });
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super().observe([
                'targetKeywords',
                'newKeyword',
                'availableCountries',
                'selectedCountry',
                'suggestedKeywords',
                'productId',
                'storeId',
                'filterByCompetition',
                'filterBySearchVolumeMinimum',
                'filterBySearchVolumeMaximum',
                'competitionSort',
                'volumeSort',
                'currentPage',
                'filterBySearchVolume',
                'totalPages',
                'pageSize',
                'credits',
                'language',
                'defaultLocation',
                'keywordSort',
                'refresh',
                'errorMessage',
                'isPremium',
                'originalSelectedCountry'
            ]);

            this.keywordsCount = ko.computed(function () {
                var productName = $(this.mainModalSelector + ' .product-name').text(),
                    keywords = this.targetKeywords().filter(function (keyword) {
                        return $($.parseHTML(keyword.text)).text() !== productName;
                    }),
                    count = keywords ? keywords.length : 0;

                return count + '/' + this.rules.maxKeywords;
            }, this);

            this.getDataDisabled = ko.computed(function () {
                var countryChanged = this.selectedCountry() !== this.originalSelectedCountry(),
                    productName = $(this.mainModalSelector + ' .product-name').text(),
                    needsRefresh = this.refresh(),
                    newKeywords = this.targetKeywords().some(
                        targetKeyword =>
                        $($.parseHTML(targetKeyword.text)).text() !== productName &&
                        typeof targetKeyword.searchVolume === 'undefined' &&
                        typeof targetKeyword.difficulty === 'undefined'
                    );
                return !needsRefresh && !newKeywords && !countryChanged;
            }, this);

            this.isKeywordsMax = ko.computed(function () {
                var productName = $(this.mainModalSelector + ' .product-name').text(),
                    keywords = this.targetKeywords().filter(function (keyword) {
                        return $($.parseHTML(keyword.text)).text() !== productName;
                    }),
                    keywordsCount = keywords ? keywords.length : 0;

                return keywordsCount >= this.rules.maxKeywords;
            }, this);

            this.getCredits = ko.computed(function () {
                var credits = this.credits().keywordAnalysis,
                    creditsString = `(${credits} ${
                        credits > 1 ? 'credits' : 'credit'
                    })`;

                return creditsString;
            }, this);

            this.hasFilter = ko.computed(function () {
                var hasVolume = this.filterBySearchVolume() !== this.defaultFilterBySearchVolume,
                    hasCompetition = !_.isEqual(
                        this.filterByCompetition().sort(),
                        this.defaultFilterByCompetition.sort()
                    );

                return hasVolume || hasCompetition;
            }, this);

            return this;
        },

        /**
         * Dismiss error message.
         */
        dismissMessage: function () {
            self.errorMessage('');
        },

        /**
         * Close modal.
         */
        closeModal: function () {
            $(self.modalSelector).modal('closeModal');
        },

        /**
         * Bind filter by competition checkbox.
         */
        bindCheckboxToggle: function () {
            $(self.toggleSelectAllSelector).on('click', function () {
                if (this.checked) {
                    self.filterByCompetition(['LOW', 'MEDIUM', 'HIGH']);
                } else {
                    self.filterByCompetition([]);
                }
            });
        },

        /**
         * Add keyword to product.
         *
         * @param {object} binding
         * @param {object} event
         * @returns {Boolean|undefined}
         */
        addKeyword: function (binding, event) {
            var targetKeywords = self
                .targetKeywords()
                .map((item) => item['text']),
            keywordData = {
                text: self.newKeyword(),
                values: [],
                temp: true
            };

            if (
                event.keyCode === 13 ||
                event.keyCode === 44 ||
                event.type === 'click'
            ) {
                if (!self.isPremium()) {
                    self.closeModal();
                    return;
                }

                /* If keyword is selected from suggestions */
                if (event.type === 'click') {
                    keywordData.text = this.keyword;
                    keywordData.difficulty = this.competition;
                    keywordData.searchVolume = this.search_volume;
                }

                /* Don't add empty keywords */
                if (keywordData.text.length < 1) {
                    return;
                }

                self.errorMessage('');
                /* Validate keyword length */
                if (keywordData.text.length > self.rules.maxKeywordLength) {
                    self.errorMessage('Target keyword is too long.');
                    return;
                }
                /* Validate duplicate keyword */
                var isDuplicate = self.targetKeywords().find(keyword => {
                    return keyword.text.toLowerCase() === keywordData.text.toLowerCase();
                });
                if (isDuplicate) {
                    self.errorMessage($t('The entered keyword already exists.'));
                    return;
                }

                self.targetKeywords.push(keywordData);

                /* Clear new keyword input */
                if (event.type !== 'click') {
                    self.newKeyword('');
                }

                $.ajax({
                    url: self.keywordAddUrl,
                    type: 'POST',
                    data: {
                        product_id: self.productId(),
                        store_id: self.storeId(),
                        keywords: targetKeywords,
                        keyword: keywordData.text,
                        competition: keywordData.difficulty,
                        search_volume: keywordData.searchVolume,
                        location: self.selectedCountry(),
                        competition_filter: self.filterByCompetition(),
                        search_volume_minimum: self.filterBySearchVolumeMinimum(),
                        search_volume_maximum: self.filterBySearchVolumeMaximum(),
                        competition_sort: self.competitionSort(),
                        volume_sort: self.volumeSort(),
                        keyword_sort: self.keywordSort(),
                        page: self.currentPage(),
                        page_size: self.pageSize(),
                        nogenerate: true,
                        refresh: false
                    },
                    dataType: 'json',
                    showWriteTextAILoader: true,
                    showAnalysisLoader: true,
                    success: function (response) {
                        var keyword = {};

                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);

                        if (response.success) {
                            keyword = ko.utils.arrayFirst(
                                self.targetKeywords(),
                                function (data) {
                                    return (
                                        data.text === response.data.text &&
                                        data.temp === true
                                    );
                                }
                            );

                            if (keyword) {
                                self.targetKeywords.replace(keyword, response.data);
                            } else {
                                self.targetKeywords.push(response.data);
                            }

                            self.newKeyword('');
                            audience.selectedAudience(response.selected_audience.shift());
                            audience.suggestedAudience(response.suggested_audience);
                        } else {
                            self.errorMessage(response.message);
                            self.targetKeywords.remove(function (data) {
                                return (
                                    data.text === keywordData.text &&
                                    data.temp === true
                                );
                            });
                            premium.isPremium(response.is_premium);
                        }
                    }
                });

                if (event.keyCode === 44) {
                    return;
                }
            }

            return true;
        },

        /**
         * Get keyword length.
         */
        getKeywordLength: function () {
            return (
                this.newKeyword().length +
                '/' +
                this.rules.maxKeywordLength +
                ' ' +
                $t('Char')
            );
        },

        /**
         * Check if keyword is removable.
         *
         * @param {string} keyword
         * @returns {boolean}
         */
        keywordRemovable: function (keyword) {
            var productName = $(self.mainModalSelector + ' .product-name').text(),
                parsedKeyword =  $($.parseHTML(keyword)).text();

            return parsedKeyword !== productName;
        },

        /**
         * Remove keyword from product.
         */
        removeKeyword: function () {
            if (!self.isPremium()) {
                self.closeModal();
                return;
            }

            self.errorMessage('');
            self.targetKeywords.remove(this);

            $.ajax({
                url: self.keywordDeleteUrl,
                type: 'POST',
                data: {
                    product_id: self.productId(),
                    store_id: self.storeId(),
                    remaining_keywords: self.targetKeywords()
                        .filter(function (keyword) {
                            return keyword.text !== self.text;
                        })
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showAnalysisLoader: true,
                success: function (response) {
                    premium.isPremium(response.is_premium);
                    prompt.isPremium(response.is_premium);
                    totalCredits.totalCredits(response.total_credits);
                    
                    if (response.success) {
                        keywords.selectedSemanticKeywords(response.selected);
                        audience.selectedAudience(response.selected_audience.shift());
                        audience.suggestedAudience(response.suggested_audience);
                    } else {
                        self.errorMessage(response.message);
                        premium.isPremium(response.is_premium);
                        self.targetKeywords.push(this);
                    }
                }.bind(this)
            });
        },

        /**
         * Get keyword ideas
         *
         * @param {Boolean} nogenerate
         * @param {Boolean} refresh
         * @param {Boolean} showLoader
         */
        getData: function (nogenerate = false, refresh = true, showLoader = false) {
            var selectedCountry = self.selectedCountry();
            
            if (!self.isPremium()) {
                self.closeModal();
                return;
            }

            self.errorMessage('');
            if (refresh) {
                self.refresh(false);
            }

            if (self.gettingData !== null) {
                self.gettingData.abort();
            }

            self.gettingData = $.ajax({
                url: self.keywordIdeasUrl,
                type: 'POST',
                data: {
                    product_id: self.productId(),
                    store_id: self.storeId(),
                    keywords: self.targetKeywords(),
                    location: selectedCountry,
                    competition: self.filterByCompetition(),
                    search_volume_minimum: self.filterBySearchVolumeMinimum(),
                    search_volume_maximum: self.filterBySearchVolumeMaximum(),
                    competition_sort: self.competitionSort(),
                    volume_sort: self.volumeSort(),
                    keyword_sort: self.keywordSort(),
                    page: self.currentPage(),
                    page_size: self.pageSize(),
                    nogenerate: nogenerate,
                    refresh: refresh
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showAnalysisLoader: showLoader,
                success: function (response) {
                    premium.isPremium(response.is_premium);
                    prompt.isPremium(response.is_premium);
                    totalCredits.totalCredits(response.total_credits);
                    if (response.success) {
                        if (
                            response.ideas.length === 0 &&
                            !nogenerate &&
                            refresh
                        ) {
                            self.errorMessage(
                                /* eslint-disable-next-line max-len */
                                $t('No keyword data received from the API. Check for misspellings in your keyword, check your country selection, or use a different keyword and try again.')
                            );
                        }
                        self.suggestedKeywords(response.ideas);
                        self.totalPages(response.total_page);

                        /* Update target keywords with new data */
                        let currentKeywords = [...self.targetKeywords()];
                        let updatedKeywords = currentKeywords.map(targetKeyword => {
                            let keyword = response.keywords.find(keyword => keyword.text === targetKeyword.text);

                            if (keyword) {
                                targetKeyword.difficulty = keyword.difficulty;
                                targetKeyword.searchVolume = keyword.searchVolume;
                            }

                            return targetKeyword;
                        });
                        self.targetKeywords([]);
                        self.targetKeywords(updatedKeywords);
                        /* End update target keywords */

                        self.originalSelectedCountry(selectedCountry);
                    } else {
                        self.errorMessage(response.message);
                    }
                },
                error: function (xhr, status, error) {
                    if (status !== 'abort') {
                        self.errorMessage(error);
                    }
                }
            });
        },

        /**
         * Show more keywords.
         */
        showMore: function () {
            self.pageSize(self.pageSize() + 10);
            self.getData(true, false);
        },

        /**
         * Sort by keyword.
         */
        sortByKeyword: function () {
            self.keywordSort(true);
            self.competitionSort('');
            self.volumeSort('');
            self.getData(true, false);
        },

        /**
         * Sort by difficulty.
         */
        sortByDifficulty: function () {
            self.keywordSort('');
            self.competitionSort(!self.competitionSort());
            self.volumeSort('');
            self.getData(true, false);
        },

        /**
         * Sort by search volume.
         */
        sortBySearchVolume: function () {
            self.keywordSort('');
            self.competitionSort('');
            self.volumeSort(!self.volumeSort());
            self.getData(true, false);
        }
    });
});
