/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'uiComponent',
    'underscore',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'WriteTextAI_WriteTextAI/js/model/edit/textfields',
    'WriteTextAI_WriteTextAI/js/model/edit/generate/keyword-analysis',
    'WriteTextAI_WriteTextAI/js/model/edit/settings',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/model/edit/error-messages',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'WriteTextAI_WriteTextAI/js/model/edit/audience',
    'WriteTextAI_WriteTextAI/js/utils/edit/textfields',
    'ko',
    'mage/translate',
    'WriteTextAI_WriteTextAI/js/edit/keyword-analysis',
    'Magento_Ui/js/modal/modal'
], function (
    $,
    Component,
    _,
    keywords,
    product,
    textfields,
    genKeywordAnalysis,
    settings,
    premium,
    errorMessagesModel,
    prompt,
    totalCredits,
    audience,
    textfieldsUtils,
    ko,
    $t,
    keywordAnalysisComponent,
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            modalSelector: '.keyword-analysis',
            mainModalSelector: '.writetextai-edit-modal.writetextai-edit-main',
            productId: '',
            storeId: '',
            targetKeywords: [],
            selectedSemanticKeywords: [],
            highlighted: true,
            productDescription: '',
            productShortDescription: '',
            guideSteps: false,
            isPremium: true
        },

        isTargetKeywordsChange: false,
        isHighlightedChange: false,
        isSelectedSemanticKeywordsChange: false,
        keywordIdeasLoaded: false,

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            this.bindModal();
            this.bindEvents();

            premium.isPremium.subscribe(function (isPremium) {
                self.isPremium(isPremium);
                if (!isPremium) {
                    self.isHighlightedChange = true;
                    self.highlighted(false);
                    self.isHighlightedChange = false;
                }
            });

            textfields.productDescription.subscribe(
                function (productDescription) {
                    self.productDescription(productDescription);
                }
            );

            textfields.productShortDescription.subscribe(
                function (productShortDescription) {
                    self.productShortDescription(productShortDescription);
                }
            );

            product.productId.subscribe(function (productId) {
                self.productId(productId);
            });

            product.storeId.subscribe(function (storeId) {
                self.storeId(storeId);
            });

            keywords.highlighted.subscribe(function (highlighted) {
                self.isHighlightedChange = true;
                self.highlighted(highlighted);
                self.isHighlightedChange = false;
            });

            keywords.targetKeywords.subscribe(function (targetKeywords) {
                self.isTargetKeywordsChange = true;
                self.targetKeywords(targetKeywords);
                self.isTargetKeywordsChange = false;
            });

            keywords.selectedSemanticKeywords.subscribe(
                function (selectedSemanticKeywords) {
                    self.isSelectedSemanticKeywordsChange = true;
                    self.selectedSemanticKeywords(selectedSemanticKeywords);
                    self.isSelectedSemanticKeywordsChange = false;
                }
            );
            
            settings.guideSteps.subscribe(function (guideSteps) {
                self.guideSteps(guideSteps);
            });

            self.selectedSemanticKeywords.subscribe(
                function (selectedSemanticKeywords) {
                    if (!self.isSelectedSemanticKeywordsChange) {
                        keywords.selectedSemanticKeywords(
                            selectedSemanticKeywords
                        );
                    }
                }
            );

            self.targetKeywords.subscribe(function (targetKeywords) {
                if (!self.isTargetKeywordsChange) {
                    keywords.targetKeywords(targetKeywords);
                }
            });

            self.highlighted.subscribe(function (highlighted) {
                if (!self.isHighlightedChange) {
                    keywords.highlighted(highlighted);
                    self.saveHighlighted(highlighted);
                }
            });
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super().observe([
                'targetKeywords',
                'productId',
                'storeId',
                'highlighted',
                'productDescription',
                'productShortDescription',
                'selectedSemanticKeywords',
                'guideSteps',
                'isPremium'
            ]);

            this.keywordsCount = ko.computed(function () {
                var productName = $(this.mainModalSelector + ' .product-name').text(),
                    keywords = this.targetKeywords().filter(function (keyword) {
                        return $($.parseHTML(keyword.text)).text() !== productName;
                    }),
                    count = keywords ? keywords.length : 0;

                return count + '/' + this.rules.maxKeywords;
            }, this);

            this.semanticKeywordsCount = ko.computed(function () {
                var count = this.selectedSemanticKeywords()
                    ? this.selectedSemanticKeywords().length
                    : 0;

                return '(' + count + '/' + this.rules.maxSemanticKeywords + ')';
            }, this);

            return this;
        },

        /**
         * Bind events.
         */
        bindEvents: function () {
            $('.target-list .action-close').on('click', function (event) {
                var target = event.target,
                    parent = $(target).closest('li');

                parent.remove();
            });

            $('.semantic-list li').on('click', function () {
                $(this).toggleClass('selected');
            });
        },

        /**
         * Bind keyword analysis modal.
         */
        bindModal: function () {
            $(self.modalSelector).modal({
                type: 'slide',
                title: $t('Keyword Analysis'),
                modalClass: 'writetextai-edit-modal writetextai-edit-keyword-analysis',
                innerScroll: true,
                buttons: [],
                overlayClass: 'modals-overlay modals-overlay-writetextai-keyword',
                closed: function () {
                    $('.modals-overlay-writetextai-keyword').remove();
                },
                opened: function () {
                    $('.writetextai-edit-keyword-analysis').trigger(
                        'contentUpdated'
                    );
                    genKeywordAnalysis.views(genKeywordAnalysis.views() + 1);
                }
            });
        },

        /**
         * Open keyword analysis modal.
         */
        openKeywordAnalysis: function () {
            if (!self.isPremium()) {
                return;
            }
            
            $(self.modalSelector).modal('openModal');

            if (!self.keywordIdeasLoaded) {
                keywordAnalysisComponent.prototype.getData(true, false, true);
                self.keywordIdeasLoaded = true;
            }
        },

        /**
         * Reset keyword ideas loaded.
         */
        resetKeywordIdeasLoaded: function () {
            if (self.keywordIdeasLoaded) {
                self.keywordIdeasLoaded = false;
            }
        },

        /**
         * Check if keyword is removable.
         *
         * @param {string} keyword
         * @returns {boolean}
         */
        keywordRemovable: function (keyword) {
            var productName = $(self.mainModalSelector + ' .product-name').text(),
                parsedKeyword =  $($.parseHTML(keyword)).text();

            return parsedKeyword !== productName;
        },

        /**
         * Get product name.
         *
         * @returns {string}
         */
        getProductName: function () {
            return $(self.mainModalSelector + ' .product-name').text();
        },

        /**
         * Get the keyword density for a given keyword.
         *
         * @param {string} keyword
         * @returns {ko.computed}
         */
        keywordDensity: function (keyword) {
            return ko.computed(function () {
                return self.calculateKeywordDensity(
                    keyword,
                    self.productDescription(),
                    self.productShortDescription()
                );
            });
        },

        /**
         * Calculate the keyword density for a given keyword.
         *
         * @param {string} keyword
         * @param {string} productDescription
         * @param {string} productShortDescription
         * @returns {string} Formatted keyword density
         */
        calculateKeywordDensity: function (
            keyword,
            productDescription,
            productShortDescription
        ) {
            if (!productDescription && !productShortDescription) {
                return '';
            }

            var productDescWordCount = textfieldsUtils.getWordCount(productDescription, false),
                productShortDescWordCount = textfieldsUtils.getWordCount(productShortDescription, false),
                keywordWordCount = textfieldsUtils.getWordCount(keyword, false),
                totalWordCount = productDescWordCount + productShortDescWordCount,
                productDescKeywordCount = textfieldsUtils.countKeywordOccurences(productDescription, keyword),
                productShortDescKeywordCount = textfieldsUtils.countKeywordOccurences(productShortDescription, keyword),
                productDescKeywordOccur = productDescKeywordCount * keywordWordCount,
                productShortDescKeywordOccur = productShortDescKeywordCount * keywordWordCount,
                totalKeywordCount = productDescKeywordOccur + productShortDescKeywordOccur,
                keywordDensity = (totalKeywordCount / totalWordCount) * 100,
                parsedKeywordDensity = parseFloat(keywordDensity, 10).toFixed(2);

            return ' (' + parsedKeywordDensity + '%)';
        },

        /**
         * Save user settings for highlighted keywords.
         *
         * @param {boolean} highlighted
         * @returns {void}
         */
        saveHighlighted: function (highlighted) {
            var url = self.highlightUrl;
            
            if (!self.isPremium()) {
                return;
            }

            $.ajax({
                url: url,
                type: 'POST',
                data: {
                    highlighted: highlighted
                },
                dataType: 'json',
                showWriteTextAILoader: false,
                success: function (response) {
                    if (!response.success) {
                        console.log(response.message);
                    }
                }
            });
        },

        /**
         * Remove a keyword from the current product.
         */
        removeKeyword: function () {
            var url = self.deleteUrl,
                productId = self.productId(),
                storeId = self.storeId(),
                text = this.text,
                remainingKeywords = self
                    .targetKeywords()
                    .filter(function (keyword) {
                        return keyword.text !== text;
                    });
            
            if (!self.isPremium()) {
                return;
            }

            errorMessagesModel.messages([]);
            self.targetKeywords.remove(this);

            $.ajax({
                url: url,
                type: 'POST',
                data: {
                    product_id: productId,
                    store_id: storeId,
                    remaining_keywords: remainingKeywords
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showAnalysisLoader: true,
                success: function (response) {
                    premium.isPremium(response.is_premium);
                    prompt.isPremium(response.is_premium);
                    totalCredits.totalCredits(response.total_credits);
                    
                    if (response.success) {
                        keywords.selectedSemanticKeywords(response.selected);
                        audience.selectedAudience(response.selected_audience.shift());
                        audience.suggestedAudience(response.suggested_audience);
                    } else {
                        errorMessagesModel.messages.push(response.message);
                        premium.isPremium(response.is_premium);
                        self.targetKeywords.push(this);
                    }
                }.bind(this)
            });
        },

        /**
         * Check if keyword is selected.
         */
        isSemanticSelected: function (keyword) {
            if (self.selectedSemanticKeywords()) {
                return self.selectedSemanticKeywords().indexOf(keyword) > -1;
            } else {
                return false;
            }
        },

        /**
         * Select semantic keyword.
         *
         * @param {string} text
         * @param {array} selected
         * @param {string} keyword
         * @returns {void}
         */
        selectKeyword: function (keyword) {
            var url = self.selectUrl,
                storeId = self.storeId(),
                productId = self.productId(),
                originalSelected = self.selectedSemanticKeywords().slice(),
                selected = self.selectedSemanticKeywords(),
                productName = $(self.mainModalSelector + ' .product-name').text();
                
            if (!self.isPremium()) {
                return;
            }

            errorMessagesModel.messages([]);

            /* check if keyword is already selected */
            if (selected.indexOf(keyword) > -1) {
                /* unselect keyword */
                selected.splice(selected.indexOf(keyword), 1);
            } else {
                /* check if max semantic keywords is reached */
                if (selected.length >= self.rules.maxSemanticKeywords) {
                    return;
                }

                /* select keyword */
                selected.push(keyword);
            }

            self.selectedSemanticKeywords(selected);

            $.ajax({
                url: url,
                type: 'POST',
                data: {
                    product_id: productId,
                    store_id: storeId,
                    text: productName,
                    selected: selected
                },
                dataType: 'json',
                showWriteTextAILoader: false,
                success: function (response) {
                    premium.isPremium(response.is_premium);
                    prompt.isPremium(response.is_premium);
                    totalCredits.totalCredits(response.total_credits);

                    if (!response.success) {
                        errorMessagesModel.messages.push(response.message);
                        self.selectedSemanticKeywords(originalSelected);
                        premium.isPremium(response.is_premium);
                    }
                }
            });
        }
    });
});
