/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'uiComponent',
    'underscore',
    'WriteTextAI_WriteTextAI/js/model/edit/custom-tone-style',
    'WriteTextAI_WriteTextAI/js/model/edit/settings',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords',
    'WriteTextAI_WriteTextAI/js/model/edit/textfields',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'WriteTextAI_WriteTextAI/js/model/edit/attributes',
    'WriteTextAI_WriteTextAI/js/utils/edit/gallery'
], function (
    $,
    Component,
    _,
    customToneStyle,
    settings,
    keywords,
    textfields,
    product,
    attributes,
    galleryUtils
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            productId: '',
            storeId: '',
        },

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            $('.writetextai-restore-global-settings > *').applyBindings(this);

            product.productId.subscribe(function (productId) {
                self.productId(productId);
            });

            product.storeId.subscribe(function (storeId) {
                self.storeId(storeId);
            });
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super()
                .observe([
                    'productId',
                    'storeId'
                ]);

            return this;
        },

        /**
         * Restore global settings
         */
        restore: function () {
            var url = this.restoreUrl;

            $.ajax({
                url: url,
                type: 'GET',
                dataType: 'json',
                data: {
                    product_id: self.productId(),
                    store_id: self.storeId()
                },
                showWriteTextAILoader: true,
                success: function (response) {
                    if (!response.success) {
                        console.log(response.message);
                    } else {
                        customToneStyle.customTone(response.data.custom_tone);
                        customToneStyle.customStyle(response.data.custom_style);
                        settings.guideSteps(response.data.guide_steps);
                        settings.tones(response.data.tones);
                        settings.styles(response.data.styles);
                        settings.audience(response.data.audience);
                        keywords.highlighted(JSON.parse(response.data.highlighted.toLowerCase()));

                        textfields.highlightPronouns(
                            JSON.parse(response.data.highlight_pronouns.toLowerCase())
                        );

                        textfields.selectedFields([]);
                        for (let key in response.data.selected_fields) {
                            if (response.data.selected_fields.hasOwnProperty(key)) {
                                textfields.selectedFields.push(response.data.selected_fields[key]);
                            }
                        }

                        attributes.productAttributes([]);
                        for (let key in response.data.product_attributes) {
                            if (response.data.product_attributes.hasOwnProperty(key)) {
                                attributes.productAttributes.push(
                                    response.data.product_attributes[key]
                                );
                            }
                        }
                        
                        /**
                         * Always update product attribute list first before the selected
                         * to prioritize preference over global settings
                         */
                        attributes.productAttributesSelected(response.data.product_attributes_selected);

                        galleryUtils.updateImages(response.data.gallery, response.data.images);
                    }
                }
            });
        }
    });
});
