/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'uiComponent',
    'underscore',
    'WriteTextAI_WriteTextAI/js/model/edit',
    'WriteTextAI_WriteTextAI/js/model/edit/textfields',
    'WriteTextAI_WriteTextAI/js/model/edit/generate/settings',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords',
    'WriteTextAI_WriteTextAI/js/model/edit/settings',
    'WriteTextAI_WriteTextAI/js/utils/edit/textfields',
    'WriteTextAI_WriteTextAI/js/model/edit/transfer/settings',
    'WriteTextAI_WriteTextAI/js/model/edit/mark-reviewed',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/model/edit/generate/reference-product',
    'WriteTextAI_WriteTextAI/js/model/edit/reference-products',
    'WriteTextAI_WriteTextAI/js/model/edit/review-status',
    'WriteTextAI_WriteTextAI/js/model/edit/gallery',
    'mage/translate',
    'ko',
    'wysiwygAdapter',
    'Magento_Ui/js/modal/confirm',
    'owlCarousel',
    'accordion',
    'jquery/ui',
    'mage/adminhtml/wysiwyg/tiny_mce/setup'
], function (
    $,
    Component,
    _,
    editData,
    textfields,
    generateSettings,
    product,
    keywords,
    settings,
    textfieldsUtils,
    transferSettings,
    markReviewed,
    premium,
    genRefProduct,
    referenceProducts,
    reviewStatus,
    gallery,
    $t,
    ko,
    wysiwygAdapter,
    confirm
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    /**
     * Highlight words in a textarea
     */
    ko.bindingHandlers.highlightWords = {
        update: function (element, valueAccessor) {
            var values = valueAccessor(),
                id = $(element).prop('id'),
                allKeywords = values.keywords(),
                selectedKeywords = values.selectedSemanticKeywords(),
                formalLanguages = values.formalLanguages(),
                formalLanguageSupport = values.formalLanguageSupport(),
                language = values.language(),
                formal = values.formal() ? 'Formal' : 'Informal',
                highlightPronouns = values.highlightPronouns(),
                text = values.text(),
                formalLanguage =
                    typeof formalLanguages === 'object' && formalLanguages
                        ? formalLanguages.find((item) => item.id === formal)
                        : null,
                pronouns = formalLanguage ? formalLanguage.pronouns : [],
                escapedKeywords = '',
                escapedSkeywords = '',
                escapedPronouns = '',
                pattern = '',
                spattern = '',
                ppattern = '',
                highlightedText = '',
                isPremium = values.isPremium(),
                isHighlighted = values.highlighted(),
                mainModalSelector = '.writetextai-edit-modal.writetextai-edit-main';

            if (typeof allKeywords !== 'undefined' && allKeywords.length > 0) {
                
                /**
                 * Remove product name from the highlighted keywords
                 */
                var productName = $(mainModalSelector + ' .product-name').text();
                allKeywords = allKeywords.filter(function (keyword) {
                    return keyword != productName;
                });

                /* Escape special characters in words to avoid issues with regex */
                escapedKeywords = allKeywords.map(function (keyword) {
                    return keyword.replace(
                        /[-\/\\^$*+?.()|[\]{}]/g,
                        '\\$&'
                    );
                });
                escapedSkeywords = selectedKeywords.map(function (keyword) {
                    return keyword.replace(
                        /[-\/\\^$*+?.()|[\]{}]/g,
                        '\\$&'
                    );
                });
                escapedPronouns = pronouns.map(function (pronoun) {
                    return pronoun.replace(
                        /[-\/\\^$*+?.()|[\]{}]/g,
                        '\\$&'
                    );
                });

                /* Create a regular expression pattern to match the phrases */
                pattern = new RegExp(
                    '(\\b' + escapedKeywords.join('\\b|\\b') + '\\b)',
                    'gi'
                );
                spattern = new RegExp(
                    '(\\b' + escapedSkeywords.join('\\b|\\b') + '\\b)',
                    'gi'
                );
                ppattern = new RegExp(
                    '(\\b' + escapedPronouns.join('\\b|\\b') + '\\b)',
                    'g'
                );

                if (isPremium && isHighlighted) {
                    /* Wrap matched words with <span class="highlight"> tags */
                    highlightedText = text.replace(
                        pattern,
                        '<span class="highlight-keyword">$&</span>'
                    );

                    if (selectedKeywords.length > 0) {
                        highlightedText = highlightedText.replace(
                            spattern,
                            '<span class="highlight-semantic-keyword">$&</span>'
                        );
                    }
                }

                if (highlightPronouns &&
                    formalLanguageSupport.includes(language) &&
                    pronouns.length > 0
                ) {
                    highlightedText = highlightedText.replace(
                        ppattern,
                        '<span class="highlight-pronoun">$&</span>'
                    );
                }

                highlightedText =  '<span class="highlighted-text">' + highlightedText + '</span>';

                if (!/<[a-z][\s\S]*>/i.test(highlightedText)) {
                    highlightedText = highlightedText.replace(/(\r\n|\n\r|\r|\n)/g, '<br />');
                }

                if ([
                        'page-title-mask',
                        'page-desc-mask',
                        'open-graph-mask'
                    ].includes(element.id)
                ) {
                    $(element).html(highlightedText);
                }

                if (wysiwygAdapter.get(id)) {
                    wysiwygAdapter
                        .get(id)
                        .setContent(highlightedText);
                }
            }
        }
    };

    return Component.extend({
        defaults: {
            productId: '',
            storeId: '',
            pageTitle: '',
            pageDescription: '',
            productDescription: '',
            productShortDescription: '',
            openGraph: '',
            descMin: 0,
            descMax: 0,
            excMin: 0,
            excMax: 0,
            selectedFields: [],
            keywords: [],
            highlighted: true,
            widgetInitialized: false,
            selectedSemanticKeywords: [],
            mgPageTitle: '',
            mgPageDescription: '',
            mgProductDescription: '',
            mgProductShortDescription: '',
            mgOpenGraph: '',
            highlightPronouns: false,
            formal: false,
            language: '',
            formalLanguageSupport: [],
            formalLanguages: [],
            selectedStores: [],
            statuses: {
                pageTitleGenerateStatus: false,
                pageDescriptionGenerateStatus: false,
                productDescriptionGenerateStatus: false,
                productShortDescriptionGenerateStatus: false,
                openGraphGenerateStatus: false,
                pageTitleTransferStatus: false,
                pageDescriptionTransferStatus: false,
                productDescriptionTransferStatus: false,
                productShortDescriptionTransferStatus: false,
                openGraphTransferStatus: false
            },
            originalPageTitle: '',
            originalPageDescription: '',
            originalProductDescription: '',
            originalProductShortDescription: '',
            originalOpenGraph: '',
            isPremium: true,
            referenceProductData: {},
            referenceProductChecked: false,
            reviewStatus: []
        },

        isSelectedFieldsChange: false,
        isPageTitleChange: false,
        isPageDescriptionChange: false,
        isProductDescriptionChange: false,
        isProductShortDescriptionChange: false,
        isOpenGraphChange: false,
        wysiwygInitialized: {
            productDescription: false,
            productShortDescription: false
        },
        wordsAllowed: false,
        carouselInitialized: false,

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            $('.writetextai-textfields > *').applyBindings(this);

            reviewStatus.reviewStatus.subscribe(function (reviewStatus) {
                if (self.carouselInitialized) {
                    $(".review_status-tooltip .owl-carousel").trigger('destroy.owl.carousel');
                    $(".review_status-tooltip .owl-carousel").html('');
                }
                self.reviewStatus(reviewStatus);
                self.initReviewCarousel();
            });

            premium.isPremium.subscribe(function (isPremium) {
                self.isPremium(isPremium);
            });

            keywords.targetKeywords.subscribe(function (targetKeywords) {
                keywords = targetKeywords.map((item) => item['text']);
                self.keywords(keywords);
            });

            keywords.highlighted.subscribe(function (highlighted) {
                self.highlighted(highlighted);
            });

            keywords.selectedSemanticKeywords.subscribe(
                function (selectedSemanticKeywords) {
                    self.selectedSemanticKeywords(selectedSemanticKeywords);
                }
            );

            product.productId.subscribe(function (productId) {
                self.productId(productId);
            });

            product.storeId.subscribe(function (storeId) {
                self.storeId(storeId);
            });

            product.language.subscribe(function (language) {
                self.language(language);
            });

            textfields.pageTitle.subscribe(function (pageTitle) {
                self.isPageTitleChange = true;
                self.pageTitle(pageTitle);
                self.isPageTitleChange = false;
            });

            textfields.pageDescription.subscribe(function (pageDescription) {
                self.isPageDescriptionChange = true;
                self.pageDescription(pageDescription);
                self.isPageDescriptionChange = false;
            });

            textfields.productDescription.subscribe(
                function (productDescription) {
                    var wysiwyg = wysiwygAdapter.get(
                        'new_product_description'
                    );

                    if (productDescription !== self.productDescription()) {
                        /* prevent wysiwyg cursor from resetting */
                        self.isProductDescriptionChange = true;

                        /* Set description */
                        if (!/<[a-z][\s\S]*>/i.test(productDescription)) {
                            productDescription = productDescription.replace(/(\r\n|\n\r|\r|\n)/g, '<br />');
                        }
                        self.productDescription(productDescription);

                        /* Sync scroll */
                        $(wysiwyg.getWin()).on('scroll', function (event) {
                            $('#product-desc-mask').scrollTop(
                                event.currentTarget.scrollY
                            );
                            $('#product-desc-mask').scrollLeft(
                                event.currentTarget.scrollX
                            );
                            if (wysiwygAdapter.get('product-desc-mask') != null) {
                                wysiwygAdapter
                                    .get('product-desc-mask')
                                    .contentWindow.scrollTo(
                                        event.currentTarget.scrollX,
                                        event.currentTarget.scrollY
                                    );
                            }
                        });

                        /* Set wywiwyg content */
                        wysiwyg.setContent(productDescription);

                        /* Disable wysiwyg editor */
                        if (
                            !self.wysiwygInitialized.productDescription &&
                            $('#new_product_description').prop('readonly') &&
                            wysiwyg.getBody()
                        ) {
                            wysiwyg
                                .getBody()
                                .setAttribute('contenteditable', false);
                            self.wysiwygInitialized.productDescription = true;
                        }

                        self.isProductDescriptionChange = false;
                    }
                }
            );

            textfields.productShortDescription.subscribe(
                function (productShortDescription) {
                    var wysiwyg = wysiwygAdapter.get(
                        'new_short_description'
                    );

                    if (
                        productShortDescription !==
                        self.productShortDescription()
                    ) {
                        /* prevent wysiwyg cursor from resetting */
                        self.isProductShortDescriptionChange = true;

                        /* Set product short description */
                        if (!/<[a-z][\s\S]*>/i.test(productShortDescription)) {
                            productShortDescription = productShortDescription.replace(/(\r\n|\n\r|\r|\n)/g, '<br />');
                        }
                        self.productShortDescription(productShortDescription);

                        /* Sync scroll */
                        $(wysiwyg.getWin()).on('scroll', function (event) {
                            $('#excerpt-mask').scrollTop(
                                event.currentTarget.scrollY
                            );
                            $('#excerpt-mask').scrollLeft(
                                event.currentTarget.scrollX
                            );
                            if (wysiwygAdapter.get('excerpt-mask') != null) {
                                wysiwygAdapter
                                    .get('excerpt-mask')
                                    .contentWindow.scrollTo(
                                        event.currentTarget.scrollX,
                                        event.currentTarget.scrollY
                                    );
                            }
                        });

                        /* Set wysiwyg content */
                        wysiwyg.setContent(productShortDescription);

                        /* Disable wysiwyg editor */
                        if (
                            !self.wysiwygInitialized.productShortDescription &&
                            $('#new_short_description').prop('readonly') &&
                            wysiwyg.getBody()
                        ) {
                            wysiwyg
                                .getBody()
                                .setAttribute('contenteditable', false);
                            self.wysiwygInitialized.productShortDescription = true;
                        }

                        self.isProductShortDescriptionChange = false;
                    }
                }
            );

            textfields.openGraph.subscribe(function (openGraph) {
                self.isOpenGraphChange = true;
                self.openGraph(openGraph);
                self.isOpenGraphChange = false;
            });

            textfields.mgPageTitle.subscribe(function (mgPageTitle) {
                self.mgPageTitle(mgPageTitle);
            });

            textfields.mgPageDescription.subscribe(
                function (mgPageDescription) {
                    self.mgPageDescription(mgPageDescription);
                }
            );

            textfields.mgProductDescription.subscribe(
                function (mgProductDescription) {
                    var wysiwyg = wysiwygAdapter.get(
                        'product-description'
                    );

                    /* Set description */
                    if (mgProductDescription) {
                        if (!/<[a-z][\s\S]*>/i.test(mgProductDescription)) {
                            mgProductDescription = mgProductDescription.replace(/(\r\n|\n\r|\r|\n)/g, '<br />');
                        }
                        wysiwyg.setContent(mgProductDescription);
                    } else {
                        wysiwyg.setContent('');
                    }

                    self.mgProductDescription(mgProductDescription);
                }
            );

            textfields.mgProductShortDescription.subscribe(
                function (mgProductShortDescription) {
                    var wysiwyg = wysiwygAdapter.get(
                        'short-description'
                    );

                    if (mgProductShortDescription) {
                        /* Set description */
                        if (!/<[a-z][\s\S]*>/i.test(mgProductShortDescription)) {
                            mgProductShortDescription = mgProductShortDescription.replace(/(\r\n|\n\r|\r|\n)/g, '<br />');
                        }
                        wysiwyg.setContent(mgProductShortDescription);
                    } else {
                        wysiwyg.setContent('');
                    }
                    self.mgProductShortDescription(mgProductShortDescription);
                }
            );

            textfields.mgOpenGraph.subscribe(function (mgOpenGraph) {
                self.mgOpenGraph(mgOpenGraph);
            });

            textfields.statuses.subscribe(function (statuses) {
                self.statuses(statuses);
            });

            textfields.originalPageTitle.subscribe(function (originalPageTitle) {
                self.originalPageTitle(originalPageTitle);
            });

            textfields.originalPageDescription.subscribe(
                function (originalPageDescription) {
                    self.originalPageDescription(originalPageDescription);
                }
            );

            textfields.originalProductDescription.subscribe(
                function (originalProductDescription) {
                    self.originalProductDescription(
                        originalProductDescription
                    );
                }
            );

            textfields.originalProductShortDescription.subscribe(
                function (originalProductShortDescription) {
                    self.originalProductShortDescription(
                        originalProductShortDescription
                    );
                }
            );

            textfields.originalOpenGraph.subscribe(function (originalOpenGraph) {
                self.originalOpenGraph(originalOpenGraph);
            });

            genRefProduct.referenceProductData.subscribe(
                function (referenceProductData) {
                    self.referenceProductData(referenceProductData);
                }
            );

            referenceProducts.referenceProductChecked.subscribe(
                function (referenceProductChecked) {
                    if (!self.isPremium()) {
                        referenceProductChecked = false;
                    }
                    self.referenceProductChecked(referenceProductChecked);
                }
            );

            self.pageTitle.subscribe(function (pageTitle) {
                if (!self.isPageTitleChange) {
                    textfields.pageTitle(pageTitle);
                }
            });

            self.pageDescription.subscribe(function (pageDescription) {
                if (!self.isPageDescriptionChange) {
                    textfields.pageDescription(pageDescription);
                }
            });

            self.openGraph.subscribe(function (openGraph) {
                if (!self.isOpenGraphChange) {
                    textfields.openGraph(openGraph);
                }
            });

            textfields.selectedFields.subscribe(function (selectedFields) {
                self.isSelectedFieldsChange = true;
                self.selectedFields([]);
                for (let key in selectedFields) {
                    if (selectedFields.hasOwnProperty(key)) {
                        self.selectedFields.push(selectedFields[key]);
                    }
                }
                self.isSelectedFieldsChange = false;
            });

            textfields.highlightPronouns.subscribe(
                function (highlightPronouns) {
                    self.highlightPronouns(highlightPronouns);
                }
            );

            textfields.formal.subscribe(function (formal) {
                self.formal(formal);
            });

            self.selectedFields.subscribe(function (selectedFields) {
                if (!self.isSelectedFieldsChange) {
                    let value = Object.keys(selectedFields)
                        .filter(key => selectedFields.hasOwnProperty(key))
                        .map(key => selectedFields[key]);

                    textfields.selectedFields(value);
                    self.savePreferences('single_editor', value);
                }
            });

            self.descMin.subscribe(function (descMin) {
                generateSettings.descMin(descMin);
            });

            self.descMax.subscribe(function (descMax) {
                generateSettings.descMax(descMax);
            });

            self.excMin.subscribe(function (excMin) {
                generateSettings.excMin(excMin);
            });

            self.excMax.subscribe(function (excMax) {
                generateSettings.excMax(excMax);
            });

            editData.opened.subscribe(function (opened) {
                if (opened && !self.widgetInitialized) {
                    $('.writetextai-edit-modal .input-number').spinner({
                        numberFormat: 'n',
                        min: self.rules.minOutputWords,
                        max: self.rules.maxOutputWords,
                        disabled: !self.wordsAllowed,
                        spin: function (event, ui) {
                            var value = ui.value,
                                name = $(event.target).attr('name');
                            
                            switch (name) {
                                case 'product_description_min':
                                    self.descMin(value);
                                    break;
                                case 'product_description_max':
                                    self.descMax(value);
                                    break;
                                case 'short_description_min':
                                    self.excMin(value);
                                    break;
                                case 'short_description_max':
                                    self.excMax(value);
                                    break;
                            }
                        },
                        change: function (event) {
                            var value = event.target.value,
                                name = $(event.target).attr('name');
                            
                            switch (name) {
                                case 'product_description_min':
                                    self.descMin(value);
                                    break;
                                case 'product_description_max':
                                    self.descMax(value);
                                    break;
                                case 'short_description_min':
                                    self.excMin(value);
                                    break;
                                case 'short_description_max':
                                    self.excMax(value);
                                    break;
                            }
                        }
                    });

                    self.initEditors();
                    self.initUnsavedChanges();
                }
            });

            settings.descMin.subscribe(function (descMin) {
                self.descMin(descMin);
            });

            settings.descMax.subscribe(function (descMax) {
                self.descMax(descMax);
            });

            settings.excMin.subscribe(function (excMin) {
                self.excMin(excMin);
            });

            settings.excMax.subscribe(function (excMax) {
                self.excMax(excMax);
            });

            settings.formalLanguageSupport.subscribe(
                function (formalLanguageSupport) {
                    self.formalLanguageSupport(formalLanguageSupport);
                }
            );

            settings.formalLanguages.subscribe(function (formalLanguages) {
                self.formalLanguages(formalLanguages);
            });

            transferSettings.selectedStores.subscribe(
                function (selectedStores) {
                    self.selectedStores([]);
                    for (let key in selectedStores) {
                        if (selectedStores.hasOwnProperty(key)) {
                            self.selectedStores.push(selectedStores[key]);
                        }
                    }
                }
            );
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super().observe([
                'productId',
                'storeId',
                'pageTitle',
                'pageDescription',
                'productDescription',
                'productShortDescription',
                'openGraph',
                'descMin',
                'descMax',
                'excMin',
                'excMax',
                'selectedFields',
                'keywords',
                'highlighted',
                'selectedSemanticKeywords',
                'mgPageTitle',
                'mgPageDescription',
                'mgProductDescription',
                'mgProductShortDescription',
                'mgOpenGraph',
                'highlightPronouns',
                'formal',
                'language',
                'formalLanguageSupport',
                'formalLanguages',
                'selectedStores',
                'statuses',
                'originalPageTitle',
                'originalPageDescription',
                'originalProductDescription',
                'originalProductShortDescription',
                'originalOpenGraph',
                'isPremium',
                'referenceProductData',
                'referenceProductChecked',
                'reviewStatus'
            ]);

            this.getFieldReviewStatus = function (textType) {
                var self = this;
                var result = ko.computed(function () {
                    let statuses = [];

                    if (self.reviewStatus()) {
                        self.reviewStatus().forEach(function (store) {
                            store.reviews.forEach(function (status) {
                                if (!status.fields) {
                                    return;
                                }
    
                                let fieldReview = status.fields.filter(function (field) {
                                    return field.field === textType && field.status >= 1;
                                });
    
                                if (fieldReview.length <= 0) {
                                    return;
                                }
    
                                statuses.push(fieldReview.shift().status);
                            });
                        });
                    }

                    let uniqueStatuses = [...new Set(statuses)];
                    switch (uniqueStatuses.length) {
                        case 1: return uniqueStatuses[0];
                        case 0: return 0;
                        default: return 3;
                    }
                });
                    
                return result();
            }.bind(this);
            
            this.isAllStoreView = ko.computed(function () {
                return Number(this.storeId()) === Number(this.defaultStoreId);
            }, this);

            this.useDefault = ko.computed(function () {
                return textfields.useDefault();
            }, this);

            return this;
        },

        /**
         * Show store review
         */
        showStoreReview: function (store, textType) {
            if (store.reviews.length <= 0) {
                return false;
            }

            for (let i = 0; i < store.reviews.length; i++) {
                if (self.getStatusStatus(store.reviews[i].fields, textType) >= 1) {
                    return true;
                }
            }
        },

        /**
         * Initialize review carousel
         */
        initReviewCarousel: function () {
            $(".review_status-tooltip .owl-carousel").owlCarousel({
                nav : true,
                dots: false,
                loop: true,
                items : 1,
                mouseDrag: false,
                touchDrag: false,
                pullDrag: false,
                onInitialized: self.carouselCounter,
                onChanged: self.carouselCounter
            });

            self.carouselInitialized = true;
        },

        /**
         * Carousel counter
         */
        carouselCounter: function (event) {
            var slides = event.relatedTarget;

            if (!event.namespace) {
                return;
            }

            $('.review_status-tooltip .owl-carousel-counter').text(
                slides.relative(slides.current()) + 1
                + ' ' + $t('of') + ' '
                + slides.items().length
            );
        },

        /**
         * Get status' status
         *
         * @param {Array} fields
         * @param {String} textType
         * @returns {Number}
         */
        getStatusStatus: function (fields, textType) {
            let fieldReview = fields.filter(function (field) {
                return field.field === textType;
            });

            if (fieldReview.length <= 0) {
                return;
            }

            return fieldReview.shift().status;
        },

        /**
         * Update review status
         *
         * @param {String} textType
         * @returns {void}
         */
        updateReviewStatus: function (textType) {
            $.ajax({
                url: self.reviewUrl,
                type: 'POST',
                data: {
                    text_type: textType,
                    product_id: self.productId(),
                    store_id: self.storeId()
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                success: function (response) {
                    if (!response.success) {
                        console.log(response);
                    } else {
                        reviewStatus.reviewStatus(response.updatedReview);
                    }
                }
            });
        },

        /**
         * Save user preferences
         */
        savePreferences: function (field, selected) {
            var value = selected;
            
            if (value.length <= 0) {
                value = '';
            }

            $.ajax({
                url: self.saveUrl,
                type: 'POST',
                data: {
                    [field]: value,
                },
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (!response.success) {
                        console.log(response);
                    }
                }
            });
        },

        /**
         * Check if text is edited.
         *
         * @param {String} original
         * @param {String} edited
         * @returns {Boolean}
         */
        isEdited: function (original, edited) {
            var originalText = original;

            if (!/<[a-z][\s\S]*>/i.test(originalText)) {
                originalText = originalText.replace(/(\r\n|\n\r|\r|\n)/g, '<br />');
            }

            return originalText !== edited;
        },

        /**
         * Get reference product length
         *
         * @param {Object} data
         * @param {String} field
         * @returns {String}
         */
        getRefCount: function (data, field) {
            var char = 0,
                words = 0;

            switch (field) {
                case 'product_description':
                    char = self.getCharCount(data.product_description) + ' ' + $t('Char'),
                    words = self.getWordCount(data.product_description) + ' ' + $t('word/s');
                    break;
                case 'short_product_description':
                    char = self.getCharCount(data.short_product_description) + ' ' + $t('Char'),
                    words = self.getWordCount(data.short_product_description) + ' ' + $t('word/s');
                    break;
            }

            return '(' + char + ' | ' + words + ')';
        },

        /**
         * Get char count.
         *
         * @param {String} html
         * @param {Number} limit
         * @returns {Number}
         */
        getCharCount: function (html, limit = null) {
            return textfieldsUtils.getCharCount(html, limit);
        },

        /**
         * Get word count.
         *
         * @param {String} html
         * @returns {Number}
         */
        getWordCount: function (html) {
            return textfieldsUtils.getWordCount(html);
        },

        /**
         * Check if text is over limit.
         *
         * @param {String} html
         * @param {Number} limit
         * @returns {Boolean}
         */
        isOverLimit: function (html, limit) {
            let tempDiv = document.createElement("div");
                tempDiv.innerHTML = html;
            let renderedText = tempDiv.textContent || tempDiv.innerText;

            return renderedText.length > limit;
        },

        /**
         * Transfer text to Magento.
         */
        transfer: function (field, value) {
            confirm({
                title: $.mage.__('Transfer selected to Magento'),
                content: self.confirmContent,
                modalClass: 'writetextai-modal bulk-transfer-popup transfer edit',
                actions: {
                    confirm: function () {
                        self._transfer(field, value);
                    }
                },
                buttons: [{
                    text: $t('Cancel'),
                    class: 'action-secondary action-dismiss',
                    click: function (event) {
                        this.closeModal(event);
                    }
                }, {
                    text: $t('OK'),
                    class: 'action-primary action-accept',
                    click: function (event) {
                        if (
                            $('#edit-transfer-form').data('validator') &&
                            $('#edit-transfer-form').validation() &&
                            $('#edit-transfer-form').validation('isValid')
                        ) {
                            this.closeModal(event, true);
                        }
                    }
                }]
            });
        },

        /**
         * Transfer field generated to live
         */
        _transfer: function (field, value) {
            var url = self.transferUrl;

            $.ajax({
                url: url,
                type: 'POST',
                data: {
                    product_id: self.productId(),
                    store_id: self.storeId(),
                    field: field,
                    value: value,
                    selected_stores: self.selectedStores()
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showPageTitleLoader: field === 'page_title',
                showPageDescriptionLoader: field === 'page_description',
                showProductDescriptionLoader: field === 'product_description',
                showShortDescriptionLoader: field === 'short_product_description',
                showOpenGraphLoader: field === 'open_graph',
                success: function (response) {
                    var fields = response.fields,
                        writeTextFields = {};
                        
                    writeTextFields[field] = value;

                    if (response.success) {
                        if (response.use_default) {
                            textfields.useDefault({
                                pageTitle: response.use_default.page_title,
                                pageDescription: response.use_default.page_description,
                                productDescription: response.use_default.product_description,
                                productShortDescription: response.use_default.short_product_description,
                                openGraph: response.use_default.open_graph
                            });
                        }
                        markReviewed.reviewed(response.reviewed);
                        textfieldsUtils.updateCurrentFields(fields, [field], writeTextFields);

                        if (field === 'product_description') {
                            product.descriptionDirectiveExists(
                                response.directives.product_description
                            );
                        }
                        
                        if (field === 'short_product_description') {
                            product.shortDescriptionDirectiveExists(
                                response.directives.short_product_description
                            );
                        }
                        
                        reviewStatus.reviewStatus(response.updatedReview);
                    } else {
                        console.log(response.message);
                    }
                }
            });
        },

        /**
         * Init unsaved changes validator
         */
        initUnsavedChanges: function () {
            $(window).on('beforeunload', function () {
                var unsavedTexts = textfields.originalPageTitle() !==
                            textfields.pageTitle() ||
                        textfields.originalPageDescription() !==
                            textfields.pageDescription() ||
                        textfields.originalProductDescription() !==
                            textfields.productDescription() ||
                        textfields.originalProductShortDescription() !==
                            textfields.productShortDescription() ||
                        textfields.originalOpenGraph() !==
                            textfields.openGraph(),
                    unsavedImage =  gallery.images().some(image => {
                        return image.original_writetext_alt !== image.writetext_alt;
                    });

                if (unsavedTexts || unsavedImage) {
                    return $t('You have unsaved changes. Are you sure you want to leave this page?');
                }
            });
        },

        /**
         * Validate number.
         */
        ensureNumerical: function (data, event) {
            return textfieldsUtils.ensureNumerical(event.key);
        },

        /**
         * Sync scroll
         *
         * @param {Object} data
         * @param {Object} event
         * @param {String} elementId
         * @returns {void}
         */
        syncScroll: function (data, event, elementId) {
            var elementSelector = '#' + elementId,
                scrollAmount = event.target.scrollTop,
                scrollXAmount = event.target.scrollLeft;

            $(elementSelector).scrollTop(scrollAmount);
            $(elementSelector).scrollLeft(scrollXAmount);
        },

        /**
         * Validate min and max fields.
         */
        validateMinMax: function (data, event) {
            var target = event.target,
                value = Number(target.value),
                min = Number(target.min),
                max = Number(target.max),
                name = target.name;

            const mapping = {
                product_description_min: self.descMin,
                product_description_max: self.descMax,
                short_description_min: self.excMin,
                short_description_max: self.excMax
            };

            if (value < min || value > max) {
                const func = mapping[name],
                    val = value < min ? min : max;

                $(target).val(val);
                func(val);
            }
        },

        /**
         * Initialize wysiwyg editors.
         */
        initEditors: function () {
            var config = self.wysiwygConfig,
                disabledConfig = self.disabledWysiwygConfig,
                wtaIds = ['new_product_description', 'new_short_description'],
                mgIds = ['product-description', 'short-description'],
                maskIds = ['product-desc-mask', 'excerpt-mask'];

            wtaIds.forEach(function (id) {
                /* eslint-disable-next-line no-undef */
                var editor = new wysiwygSetup(id, config);

                editor.setup('exact');
                $(id).addClass('wysiwyg-editor').data('wysiwygEditor', editor);
            });


            maskIds.forEach(function (id) {
                /* eslint-disable-next-line no-undef */
                var editor = new wysiwygSetup(id, config);

                editor.setup('exact');
                $(id)
                    .addClass('wysiwyg-editor wysiwyg-mask')
                    .data('wysiwygEditor', editor);
            });

            mgIds.forEach(function (id) {
                /* eslint-disable-next-line no-undef */
                var editor = new wysiwygSetup(id, disabledConfig);

                editor.setup('exact');
                $(id).addClass('wysiwyg-editor').data('wysiwygEditor', editor);
            });

            wysiwygAdapter.get('new_product_description').on("change", function() {
                if (!self.isProductDescriptionChange) {
                    textfields.productDescription(self.productDescription());
                }
            });

            wysiwygAdapter.get('new_short_description').on("change", function() {
                if (!self.isProductShortDescriptionChange) {
                    textfields.productShortDescription(self.productShortDescription());
                }
            });
        },
    });
});
