/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'underscore',
    'Magento_Ui/js/grid/columns/actions',
    'WriteTextAI_WriteTextAI/js/model/edit',
    'WriteTextAI_WriteTextAI/js/model/history-log',
    'WriteTextAI_WriteTextAI/js/grid/reload',
    'WriteTextAI_WriteTextAI/js/model/grid/error-messages',
    'WriteTextAI_WriteTextAI/js/utils/grid/actions',
    'WriteTextAI_WriteTextAI/js/model/grid/notifications',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'mage/translate',
    'Magento_Ui/js/modal/confirm',
    'mage/validation'
], function (
    $,
    _,
    Column,
    editData,
    historyLog,
    reloadGrid,
    errorMessagesModel,
    gridUtils,
    notifications,
    premium,
    prompt,
    totalCredits,
    $t,
    confirm
) {
    'use strict';

    return Column.extend({
        defaults: {
            ids: [],
            users: [],
            currentUser: {},
            isPremium: true,
            transferFormSelector: '#bulk-transfer-form',
            listens: {
                ids: 'updateIds',
                users: 'updateUsers'
            }
        },

        /** @inheritdoc */
        initialize: function () {
            this._super();
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super()
                .track([
                    'ids',
                    'users',
                    'isPremium',
                    'currentUser'
                ]);

            return this;
        },

        /** @inheritdoc */
        defaultCallback: function (
            actionIndex,
            recordId,
            action
        ) {
            var product = {
                productId: recordId,
                storeId: action.store_id
            };
            
            switch (actionIndex) {
                case 'edit':
                    editData.currentProduct(product);
                    break;

                case 'transfer':
                    this.transfer(action);
                    break;
             
                default:
                    this._super(
                        actionIndex,
                        recordId,
                        action
                    );
                    break;
            }
        },

        /** @inheritdoc */
        getActionHandler: function (action) {
            if (action.premium && !this.isPremium) {
                return;
            }

            return this._super(action);
        },

        /**
         * Transfer action.
         *
         * @param {Object} action
         * @returns {void}
         */
        transfer: function (action) {
            var selector = this.transferFormSelector,
                settings = {};

            if ($(selector).length) {
                settings = gridUtils.convertForm($(selector));
            }

            _.extend(settings, {
                record_id: action.record_id,
                store_id: action.store_id
            });

            this.transferProduct(action.url, settings);
        },

        /**
         * Transfer product.
         *
         * @param {String} url
         * @param {Object} params
         * @returns {void}
         */
        transferProduct: function (url, params) {
            errorMessagesModel.messages([]);
            notifications.addNewTransferStatus(
                [params.record_id],
                this.currentUser
            );
            
            $.ajax({
                url: url,
                type: 'POST',
                data: params,
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }

                    if (!response.success) {
                        errorMessagesModel.messages.push(response.message);
                        notifications.statuses.remove(function (status) {
                            return status.id === notifications.transferTempId;
                        });
                    } else {
                        this.gridReload();
                        notifications.updateTransferStatus([params.record_id], this.currentUser.email, [], [params.record_id]);
                    }
                }.bind(this)
            });
        },

        /** @inheritdoc */
        _confirm: function (action, callback) {
            var confirmData = action.confirm,
                formSelector = this.transferFormSelector;

            if (action.index !== 'transfer') {
                this._super(action, callback);
                return;
            }

            confirm({
                title: confirmData.title,
                content: confirmData.message,
                modalClass: 'writetextai-modal bulk-transfer-popup transfer',
                actions: {
                    confirm: callback,
                    cancel: function () {
                        $('body').removeClass('_has-modal');
                    }
                },
                buttons: [{
                    text: $t('Cancel'),
                    class: 'action-secondary action-dismiss',
                    click: function (event) {
                        this.closeModal(event);
                    }
                }, {
                    text: action.label,
                    class: 'action-primary action-accept',
                    click: function (event) {
                        if (
                            $(formSelector).data('validator') &&
                            $(formSelector).validation() &&
                            $(formSelector).validation('isValid')
                        ) {
                            this.closeModal(event, true);
                            $('body').removeClass('_has-modal');
                        }
                    }
                }]
            });
        },

        /**
         * Reload grid.
         */
        gridReload: function () {
            var showLoader = false;

            reloadGrid.reloadUIComponent(
                'products_grid_listing.products_grid_listing_data_source',
                showLoader
            );
        },

        /**
         * Update ids.
         */
        updateIds: function () {
            editData.ids(this.ids);
        },

        /**
         * Update history log users.
         */
        updateUsers: function () {
            historyLog.users(this.users);
        }
    });
});
