/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'Magento_Ui/js/grid/columns/column'
], function (
    $,
    Column
) {
    'use strict';

    return Column.extend({
        defaults: {
            headerTmpl: 'WriteTextAI_WriteTextAI/grid/columns/text',
            enableComparisonPopup: true
        },

        /** @inheritdoc */
        initialize: function () {
            this._super();
            this.initTooltipPositioner();
        },

        /** @inheritdoc */
        initObservable: function () {
            return this._super().observe(['enableComparisonPopup']);
        },

        /**
         * Initialize tooltip dynamic position.
         */
        initTooltipPositioner: function () {
            $('body').on('mouseenter', '.text-content', function () {
                var rowIndex = $(this).closest('.data-row').index(),
                    rows = $(this).closest('tbody').children().length;

                if (rowIndex >= rows / 2) {
                    $(this).next('.comparison').addClass('show-top');
                } else {
                    $('.comparison').removeClass('show-top');
                }
            });
        },

        /**
         * Check if the content should be truncated.
         *
         * @param {Object} record
         * @returns {Boolean}
         */
        isTruncated: function (record) {
            var AiContent = record['ai_' + this.index.substring(3)],
                MgContent = record['mg_' + this.index.substring(3)];

            return (
                AiContent && AiContent.length > 100 ||
                MgContent && MgContent.length > 100
            );
        },

        /**
         * Get header css classes.
         */
        getCssClasses: function () {
            if (this.label.startsWith('Magento')) {
                return 'data-grid-th mg-header';
            } else if (this.label.startsWith('WriteText.ai')) {
                return 'data-grid-th ai-header';
            }
            
            return 'data-grid-th';
        },

        /**
         * Get writetextai column header label.
         */
        getAiHeader: function () {
            if (this.label.startsWith('WriteText.ai')) {
                return this.label;
            }

            return this.label.replace('Magento', 'WriteText.ai');
        },

        /** @inheritdoc */
        getLabel: function (record) {
            return record[this.index] ? record[this.index].replace(/<[^>]*>/g, '') : '';
        },

        /**
         * Get writetextai column content.
         *
         * @param {Object} record
         * @returns {String}
         */
        getAiContent: function (record) {
            var content = record['ai_' + this.index.substring(3)];

            if (this.index.startsWith('ai_')) {
                content = record[this.index];
            }
            
            content = content ? content : '';

            if (!/<[a-z][\s\S]*>/i.test(content)) {
                content = content.replace(/(\r\n|\n\r|\r|\n)/g, '<br />');
            }

            return content;
        },

        /**
         * Get magento column header label.
         */
        getMgHeader: function () {
            if (this.label.startsWith('WriteText.ai')) {
                return this.label.replace('WriteText.ai', 'Magento');
            }

            return this.label;
        },

        /**
         * Get magento column content.
         *
         * @param {Object} record
         * @param {String} col
         * @returns {String}
         */
        getMgContent: function (record) {
            var content = record[this.index];

            if (this.index.startsWith('ai_')) {
                content = record['mg_' + this.index.substring(3)];
            }
            
            content = content ? content : '';

            if (!/<[a-z][\s\S]*>/i.test(content)) {
                content = content.replace(/(\r\n|\n\r|\r|\n)/g, '<br />');
            }
            
            return content;
        },

        /**
         * Open single editor.
         *
         * @param {Object} record
         * @returns {void}
         */
        showEditPage: function (record) {
            var action = {
                provider:
                    'products_grid_listing.products_grid_listing.products_columns.actions',
                target: 'applyAction',
                params: ['edit', '${ $.$data.rowIndex }']
            };

            this.applySingleAction(record._rowIndex, action);
        }
    });
});
