/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    "jquery",
    "ko",
    "mage/translate",
    "uiComponent",
    "underscore",
    'WriteTextAI_WriteTextAI/js/model/grid/invalid-image-popup',
    'WriteTextAI_WriteTextAI/js/model/grid/upload-images',
    'WriteTextAI_WriteTextAI/js/model/grid/error-messages',
    'WriteTextAI_WriteTextAI/js/model/grid/notifications',
    'WriteTextAI_WriteTextAI/js/grid/massactions',
    'WriteTextAI_WriteTextAI/js/model/grid/invalid-image-popup',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    "Magento_Ui/js/modal/modal"
], function (
    $,
    ko,
    $t,
    Component,
    _,
    invalidImagePopupModel,
    uploadImages,
    errorMessagesModel,
    notifications,
    massactions,
    gridInvalidImagePopupModel,
    totalCredits,
    premium,
    prompt
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            defaultStoreId: 0,
            data: {},
            settings: {}
        },

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            this.initModal();

            invalidImagePopupModel.data.subscribe(function (value) {
                this.data(value);
            }, this);

            invalidImagePopupModel.settings.subscribe(function (value) {
                this.settings(value);
            }, this);
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super()
                .observe([
                    'data',
                    'settings'
                ]);
            
            this.proceedable = ko.computed(function () {
                var successImages = uploadImages.successImages(),
                    successThumbnails = uploadImages.successThumbnails(),
                    settings = this.settings(),
                    textTypes = [
                        'page_title',
                        'page_description',
                        'product_description',
                        'short_product_description',
                        'open_graph'
                    ];

                if (settings.fields && textTypes.some(textType => settings.fields.includes(textType))) {
                    return true;
                } else {
                    return successImages.length > 0 || successThumbnails.length > 0;
                }
            }, this);

            this.failedImages = ko.computed(function () {
                return uploadImages.failedImages();
            });

            this.failedThumbnails = ko.computed(function () {
                return uploadImages.failedThumbnails();
            });

            return this;
        },

        initModal: function () {
            $('.writetextai-grid-invalid-image-popup').modal({
                type: 'popup',
                responsive: true,
                title: $t('The following image/s are not supported:'),
                modalClass: 'writetextai-grid-invalid-image-popup-modal',
                overlayClass: 'modals-overlay writetextai-grid-invalid-image-popup-overlay',
                buttons: [{
                    text: $t('Cancel'),
                    class: 'action-secondary action-dismiss',
                    click: function () {
                        this.closeModal();
                    }
                }, {
                    text: $t('Proceed'),
                    class: 'action-primary',
                    click: function () {
                        self._generate(self.data(), self.settings());
                        this.closeModal();
                    }
                }],
                closed: function () {
                    $('.modals-overlay.writetextai-grid-invalid-image-popup-overlay').remove();
                },
                opened: function () {
                    $('.writetextai-grid-invalid-image-popup-modal .modal-footer').toggle(self.proceedable());
                }
            });

            $('.writetextai-grid-invalid-image-popup').removeClass('no-display');
        },

        /**
         * Generate
         */
        _generate: function (data, settings) {
            var itemsType = '',
                selections = {};
                
            notifications.addNewGenerateStatus(
                data.selected,
                invalidImagePopupModel.currentUser()
            );

            _.extend(data.params, settings || {});

            itemsType = data.excludeMode ? 'excluded' : 'selected';

            selections[itemsType] = data[itemsType];

            if (!selections[itemsType].length) {
                selections[itemsType] = false;
            }

            _.extend(selections, data.params || {});

            /* set default store filter */
            if (!selections['filters']['store_id']) {
                selections['filters']['store_id'] = this.defaultStoreId;
            }

            selections = this.removeFailedImages(selections);

            $.ajax({
                url: self.generateUrl,
                type: 'POST',
                data: selections,
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }
                    if (!response.success) {
                        errorMessagesModel.messages.push(response.message);
                        notifications.statuses.remove(function (status) {
                            return status.id === notifications.tempId;
                        });
                        if (response.unauthorized) {
                            window.location.href = response.login_url;
                        }
                        if (response.reload) {
                            massactions.prototype.gridReload();
                        }
                    } else {
                        massactions.prototype.gridReload();
                    }
                }
            });
        },

        /**
         * Remove failed images
         *
         * @param {Object} selections
         * @returns {Object}
         */
        removeFailedImages: function (selections) {
            var failedImageIds = uploadImages.failedImageIds(),
                failedProducts = uploadImages.failedProducts();

            if (failedImageIds.length) {
                selections.skip_images = failedImageIds;
            }

            if (failedProducts.length) {
                selections.skip_thumbnails = failedProducts;
            }

            return selections;
        },
    });
});
