/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'underscore',
    'ko',
    'Magento_Ui/js/grid/listing',
    'WriteTextAI_WriteTextAI/js/model/grid/notifications',
    'WriteTextAI_WriteTextAI/js/grid/reload',
    'WriteTextAI_WriteTextAI/js/model/signalr',
    'WriteTextAI_WriteTextAI/js/model/gallery',
    'WriteTextAI_WriteTextAI/js/model/edit/product'
], function (
    _,
    ko,
    Listing,
    notifications,
    reloadGrid,
    signalRModel,
    galleryModel,
    product
) {
    'use strict';

    return Listing.extend({
        defaults: {
            gridName: 'products_grid_listing.products_grid_listing_data_source',
            statusRunning: 'Running',
            statuses: [],
            rowsDisabled: []
        },

        /** @inheritdoc */
        initialize: function () {
            this._super();

            this.initObservableRowsDisabled();

            notifications.statuses.subscribe(function (statuses) {
                this.statuses(statuses);
            }, this);
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super().observe(['statuses']);

            return this;
        },

        /**
         * Defines initial order of child elements.
         *
         * @returns {Listing} Chainable.
         */
        initPositions: function () {
            /**
             * Disable sorting to resolve the issue with the show/hide functionality of the Open Graph text.
             */
            return this;
        },

        /**
         * Initialize observables for disabling of rows.
         */
        initObservableRowsDisabled: function () {
            var observable = ko.getObservable(this, 'rows');

            observable.subscribe(function (rows) {
                _.each(
                    rows,
                    function (row) {
                        this.rowsDisabled[row.entity_id] = ko.observable(false);
                        this.rowsDisabled[row.entity_id].subscribe(
                            function (value) {
                                if (value) {
                                    this.gridReload();
                                }
                            },
                            this,
                            'beforeChange'
                        );
                    },
                    this
                );
            }, this);
        },

        /**
         * Reload grid.
         */
        gridReload: function () {
            var showLoader = false,
                debounce = true;

            reloadGrid.reloadUIComponent(this.gridName, showLoader, debounce);
        },

        /**
         * Check if row is disabled.
         *
         * @param {Object} row
         * @returns {Boolean}
         */
        isRowDisabled: function (row) {
            return ko.computed(function () {
                var isSingleGenerating = signalRModel.generating() && product.productId() === row.entity_id,
                    productsUploading = galleryModel.productsUploading().includes(row.entity_id),
                    isBulkGenerating = this.statuses().some((status) => {
                        var isQueued =
                            status.status === this.statusRunning &&
                            status.queuedIds.indexOf(row.entity_id) !== -1;

                        this.rowsDisabled[row.entity_id](isQueued);
                        
                        return isQueued;
                    }, this);

                return isBulkGenerating || isSingleGenerating || productsUploading;
            }, this);
        }
    });
});
