/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'uiComponent',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    'WriteTextAI_WriteTextAI/js/model/grid/reference-products',
    'WriteTextAI_WriteTextAI/js/utils/edit/textfields',
    'mage/translate',
    'mage/mage',
    'mage/validation',
    'jquery/ui'
], function (
    $,
    ko,
    Component,
    prompt,
    referenceProducts,
    textfieldsUtils,
    $t
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            fieldsSelector: '.bulk-generate-form input[name="fields[]"]',
            tooltipSelector: '.bulk-generate-form .admin__field-tooltip-action',
            selectedFields: [],
            selectAllPromptFields: null,
            referenceProductChecked: false,
            descMax: '',
            descMin: '',
            excMax: '',
            excMin: '',
            disallowedCombinations: [],
            tonesSelected: [],
            stylesSelected: '',
            audienceSelected: [],
            selectAllTonesValue: null,
            selectAllAudienceValue: null,
            tones: [],
            styles: [],
            audience: [],
            wordsAllowed: false,
            selectedFieldsNotPremium: []
        },

        /** @inheritdoc */
        initialize: function () {
            this._super();

            $('#bulk-generate-form > *:not(.reference-product, .reference-product *)').applyBindings(this);

            self = this;

            prompt.selectedFields(self.selectedFields());
            prompt.descMin(self.descMin());
            prompt.descMax(self.descMax());
            prompt.excMin(self.excMin());
            prompt.excMax(self.excMax());

            self.selectedFields.subscribe(function (selectedFields) {
                prompt.selectedFields(selectedFields);
                self.updateSelect();
                self.savePreferences('generate', selectedFields);
            });

            self.tonesSelected.subscribe(function () {
                self.observeTonesSelected();
            });

            self.audienceSelected.subscribe(function () {
                self.observeAudienceSelected();
            });

            self.descMin.subscribe(function (descMin) {
                prompt.descMin(descMin);
            });

            self.descMax.subscribe(function (descMax) {
                prompt.descMax(descMax);
            });

            self.excMax.subscribe(function (excMax) {
                prompt.excMax(excMax);
            });

            self.excMin.subscribe(function (excMin) {
                prompt.excMin(excMin);
            });

            self.updateSelect();
            self.initTooltip();
            self.initValidations();
            self.initTriggerUpdate();
            self.initSelectAll();
            self.initAttributes();
            self.resetReferenceProduct();
            self.initSpinner();
            self.observeTonesSelected();
            self.observeAudienceSelected();

            self.referenceProductChecked.subscribe(
                function (referenceProductChecked) {
                    referenceProducts.referenceProductChecked(
                        referenceProductChecked
                    );
                    prompt.referenceProductChecked(referenceProductChecked);
                }
            );
        },

        /**
         * Initialize update trigger
         */
        initTriggerUpdate: function () {
            $('#writetext-prompt').trigger('contentUpdated');
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super().observe([
                'selectedFields',
                'selectAllPromptFields',
                'referenceProductChecked',
                'descMax',
                'descMin',
                'excMax',
                'excMin',
                'disallowedCombinations',
                'tonesSelected',
                'stylesSelected',
                'audienceSelected',
                'selectAllTonesValue',
                'selectAllAudienceValue',
                'tones',
                'styles',
                'audience',
                'selectedFieldsNotPremium'
            ]);

            this.isPremium = ko.computed(function () {
                return prompt.isPremium();
            });

            return this;
        },

        /**
         * Save user preferences
         */
        savePreferences: function (field, selected) {
            var value = selected;
            
            if (value.length <= 0) {
                value = '';
            }

            $.ajax({
                url: self.saveUrl,
                type: 'POST',
                data: {
                    [field]: value,
                },
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (!response.success) {
                        console.log(response);
                    }
                }
            });
        },

        /**
         * Select all tones.
         */
        selectAllTones: function () {
            var tones = self.tones().filter(function (tone) {
                    return !$('.bulk-generate-form [name="tones[]"][value="' + tone.id + '"]').attr('disabled');
            }).map(function (tone) {
                return tone.id;
            });

            if (this.selectAllTonesValue()) {
                self.tonesSelected([...tones]);
            } else {
                self.tonesSelected([]);
            }
        },

        /**
         * Observe tones selected.
         */
        observeTonesSelected: function () {
            if (self.tones().filter(function (tone) {
                    return !$('.bulk-generate-form [name="tones[]"][value="' + tone.id + '"]').attr('disabled');
            }).map(function (tone) {
                return tone.id;
            }).length === self.tonesSelected().length
            ) {
                this.selectAllTonesValue(true);
            } else {
                this.selectAllTonesValue(false);
            }
        },

        /**
         * Validate number.
         */
        ensureNumerical: function (data, event) {
            return textfieldsUtils.ensureNumerical(event.key);
        },

        /**
         * Select all audience.
         */
        selectAllAudience: function () {
            var audience = self.audience().map(function (item) {
                if (!$('.bulk-generate-form [name="audience[]"][value="' + item.id + '"]').attr('disabled')) {
                    return item.id;
                }
            });

            if (this.selectAllAudienceValue()) {
                self.audienceSelected(audience);
            } else {
                self.audienceSelected([]);
            }
        },

        /**
         * Observe audience selected.
         */
        observeAudienceSelected: function () {
            if (
                self.audience().length === self.audienceSelected().length &&
                self.audience().length > 0
            ) {
                this.selectAllAudienceValue(true);
            } else {
                this.selectAllAudienceValue(false);
            }
        },

        /**
         * Reset reference product
         */
        resetReferenceProduct: function () {
            referenceProducts.referenceProductChecked(
                this.referenceProductChecked()
            );
        },

        /**
         * Initialize spinner
         */
        initSpinner: function () {
            setTimeout(
                function () {
                    $('.bulk-generate-popup .input-number').spinner({
                        numberFormat: 'n',
                        min: this.minOutputWords,
                        max: this.maxOutputWords,
                        disabled: !self.wordsAllowed,
                        spin: function (event, ui) {
                            var value = ui.value,
                                name = $(event.target).attr('name');
                            
                            switch (name) {
                                case 'description_min':
                                    self.descMin(value);
                                    break;
                                case 'description_max':
                                    self.descMax(value);
                                    break;
                                case 'excerpt_min':
                                    self.excMin(value);
                                    break;
                                case 'excerpt_max':
                                    self.excMax(value);
                                    break;
                            }
                        },
                        change: function (event) {
                            var value = event.target.value,
                                name = $(event.target).attr('name');
                            
                            switch (name) {
                                case 'description_min':
                                    self.descMin(value);
                                    break;
                                case 'description_max':
                                    self.descMax(value);
                                    break;
                                case 'excerpt_min':
                                    self.excMin(value);
                                    break;
                                case 'excerpt_max':
                                    self.excMax(value);
                                    break;
                            }
                        }
                    });
                }.bind(this),
                100
            );
        },

        /**
         * Initialize select all checkbox
         */
        initSelectAll: function () {
            setTimeout(function () {
                $('.checkbox-select-all').on('change', function () {
                    var name = $(this).attr('data-name'),
                        checked = $(this).prop('checked');

                    $('.bulk-generate-form input[name="' + name + '[]"]').prop(
                        'checked',
                        checked
                    );
                });

                $('.bulk-generate-form input[type="checkbox"]')
                    .not('.checkbox-select-all')
                    .on('change', function () {
                        var name = $(this).attr('name'),
                            selector = $('.bulk-generate-form input[name="' + name + '"]'),
                            selected = 0,
                            identifier = name.replace('[]', '');

                        selector.each(function () {
                            if ($(this).is(':checked')) {
                                selected++;
                            }
                        });

                        if (selected === selector.length) {
                            $('.bulk-generate-form [data-name="' + identifier + '"]').prop('checked', true);
                        } else {
                            $('.bulk-generate-form [data-name="' + identifier + '"]').prop('checked', false);
                        }
                    });
            }, 100);
        },

        /**
         * Initialize attributes
         */
        initAttributes: function () {
            setTimeout(function () {
                var name = 'attributes[]',
                    selector = $('.bulk-generate-form input[name="' + name + '"]'),
                    checked = $('.bulk-generate-form input[name="' + name + '"]:checked'),
                    allSelector = $('.checkbox-select-all[data-field="' + name + '"]');

                if (selector.length === checked.length) {
                    $(allSelector).prop('checked', true);
                } else {
                    $(allSelector).prop('checked', false);
                }
            }, 100);
        },

        /**
         * Initialize tooltip
         */
        initTooltip: function () {
            $(self.tooltipSelector).dropdown({
                parent: '.admin__field-tooltip',
                autoclose: true
            });
        },

        /**
         * Initialize validations
         */
        initValidations: function () {
            $('#bulk-generate-form').mage('validation', {
                rules: {
                    'fields[]': {
                        required: true
                    },
                    'tones[]': {
                        required: {
                            param: true,
                            depends: function () {
                                return !$('#reference_product_checked').is(
                                    ':checked'
                                );
                            }
                        }
                    },
                    styles: {
                        required: {
                            param: true,
                            depends: function () {
                                return !$('#reference_product_checked').is(
                                    ':checked'
                                );
                            }
                        }
                    },
                    gridReferenceProduct: {
                        required: {
                            param: true,
                            depends: function () {
                                return $('#reference_product_checked').is(
                                    ':checked'
                                );
                            }
                        }
                    }
                },
                messages: {
                    'tones[]': {
                        required: 'Select at least 1 tone.'
                    }
                },
                errorPlacement: function (error, element) {
                    var parent = element.closest('.fieldset');

                    $(parent).find('.error-message').html(error);
                }
            });
        },

        /**
         * Check if combination is not allowed
         */
        isCombinationDisallowed: function (id, type) {
            var result,
                isStyleChecked = false,
                isToneChecked = false,
                isAudienceChecked = false,
                combinationHasAudience = false;

            self = this;

            /* Check if combination is available */
            result = self
                        .disallowedCombinations()
                        .find(function (combination) {
                            return combination.combination.some(function (obj) {
                                return obj.type === type && obj.id === id;
                            });
                        });

            if (!(self.disallowedCombinations() && result)) {
                return false;
            }

            switch (type) {
                case 'Tone':
                    isStyleChecked = result.combination.some(
                        function (obj) {
                            return (
                                obj.type === 'Style' &&
                                self.stylesSelected() === obj.id
                            );
                        }
                    );
                    combinationHasAudience =
                        result.combination.some(function (obj) {
                            return obj.type === 'Audience';
                        });

                    if (combinationHasAudience) {
                        isAudienceChecked =
                            self
                                .audienceSelected()
                                .indexOf(combinationHasAudience.id) !==
                            -1;

                        return isStyleChecked && isAudienceChecked;
                    }
                    return isStyleChecked;

                case 'Style':
                    isToneChecked = result.combination.some(
                        function (obj) {
                            return (
                                obj.type === 'Tone' &&
                                self.tonesSelected().indexOf(obj.id) !==
                                    -1
                            );
                        }
                    );
                    combinationHasAudience =
                        result.combination.some(function (obj) {
                            return obj.type === 'Audience';
                        });

                    if (combinationHasAudience) {
                        isAudienceChecked =
                            self
                                .audienceSelected()
                                .indexOf(combinationHasAudience.id) !==
                            -1;

                        return isToneChecked && isAudienceChecked;
                    }
                    return isToneChecked;

                case 'Audience':
                    isToneChecked = result.combination.some(
                        function (obj) {
                            return (
                                obj.type === 'Tone' &&
                                self.tonesSelected().indexOf(obj.id) !==
                                    -1
                            );
                        }
                    );
                    isStyleChecked = result.combination.some(
                        function (obj) {
                            return (
                                obj.type === 'Style' &&
                                self.stylesSelected() === obj.id
                            );
                        }
                    );

                    return isStyleChecked && isToneChecked;

                default:
                    return false;
            }
        },

        /**
         * Get tooltip text based on disallowed combination
         */
        getTooltipCombinationDisallowedText: function (id, type) {
            var result,
                isStyleChecked = false,
                isToneChecked = false,

            self = this;

            /* Check if combination is available */
            result = self
                        .disallowedCombinations()
                        .find(function (combination) {
                            return combination.combination.some(function (obj) {
                                return obj.type === type && obj.id === id;
                            });
                        });
            if (!result) {
                return '';
            }
            var textBase = '';
            switch (type) {
                case 'Tone':
                    isStyleChecked = result.combination.some(
                        function (obj) {
                            if (obj.type === 'Style' && self.stylesSelected() === obj.id) {
                                textBase = self.processToolTip(obj.id, 'Style');
                                return true;
                            }
                            return false;
                        }
                    );
                    return textBase;

                case 'Style':
                    isToneChecked = result.combination.some(
                        function (obj) {
                            if ( obj.type === 'Tone' &&
                                self.tonesSelected().indexOf(obj.id) !== -1) {
                                textBase = self.processToolTip(obj.id, 'Tone');
                                return true;
                            }
                            return false;
                        }
                    );
                    return textBase;

                default:
                    return '';
            }
        },

        /**
         * Process tooltip content
         *
         * @param {String} selector
         * @param {String} stringValue
         */
        processToolTip: function (stringValue, type) {
            var textBase = '';
            switch (type) {
                case 'Tone':
                    textBase = $t("Unavailable when the '%1' tone is selected.");
                    break;
                case 'Style':
                    textBase = $t("Unavailable when the '%1' style is selected.");
                    break;
                default:
                    textBase = $t("Unavailable when '%1' is selected.");
                    break;
            }
            return $t(textBase).replace('%1', stringValue);
        },

        /**
         * Validate min and max fields.
         */
        validateMinMax: function (data, event) {
            var target = event.target,
                value = Number(target.value),
                min = Number(target.min),
                max = Number(target.max),
                name = target.name;

            const mapping = {
                description_min: self.descMin,
                description_max: self.descMax,
                excerpt_min: self.excMin,
                excerpt_max: self.excMax
            };

            if (value < min || value > max) {
                const func = mapping[name],
                    val = value < min ? min : max;

                $(target).val(val);
                func(val);
            }
        },

        /**
         * Detect select all checkbox changes
         */
        selectAllPromptFieldsChanged: function () {
            var checked = self.selectAllPromptFields(),
                allFields = $(self.fieldsSelector)
                    .not('[value=\'all\']')
                    .map(function () {
                        return $(this).val();
                    })
                    .get();

            self.selectedFields(checked ? allFields : []);
        },

        /**
         * Update all checkboxes
         */
        updateSelect: function () {
            var selector = $(self.fieldsSelector).not('[value=\'all\']'),
                selected = 0;

            selector.each(function () {
                if ($(this).is(':checked')) {
                    selected++;
                }
            });

            if (selected === selector.length) {
                self.selectAllPromptFields(true);
            } else {
                self.selectAllPromptFields(false);
            }
        }
    });
});
