/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'uiComponent',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    'WriteTextAI_WriteTextAI/js/model/grid/reference-products'
], function ($, ko, Component, prompt, referenceProducts) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            fieldsSelector: '#bulk-transfer-form input[name="fields[]"]',
            checkboxSelector: '#bulk-transfer-form .transfer-prompt input',
            storeId: 0,
            selectAllPromptFields: null,
            selectedFields: []
        },

        /** @inheritdoc */
        initialize: function () {
            this._super();

            $('#bulk-transfer-form > *').applyBindings(this);

            self = this;

            self.initStoreId();
            self.updateSelect();

            self.selectedFields.subscribe(function (selectedFields) {
                self.updateSelect();
                self.savePreferences('transfer', selectedFields);
            });

            referenceProducts.storeId.subscribe(function (storeId) {
                self.storeId(storeId);
            });
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super()
                .observe([
                    'storeId',
                    'selectedFields',
                    'selectAllPromptFields'
                ]);

            this.isPremium = ko.computed(function () {
                return prompt.isPremium();
            });

            return this;
        },

        /**
         * Save user preferences
         */
        savePreferences: function (field, selected) {
            var value = selected;
            
            if (value.length <= 0) {
                value = '';
            }

            $.ajax({
                url: self.saveUrl,
                type: 'POST',
                data: {
                    [field]: value,
                },
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (!response.success) {
                        console.log(response);
                    }
                }
            });
        },

        /**
         * Init store id and checkboxes checked
         */
        initStoreId: function () {
            var storeId = referenceProducts.storeId();

            this.storeId(storeId);

            $(this.checkboxSelector).prop('checked', false);
            $(this.checkboxSelector + '[data-value="' + storeId + '"]').prop('checked', true);
        },

        /**
         * Check if the store checkbox is disabled
         *
         * @param {string} value
         * @returns {boolean}
         */
        isDisabled: function (value) {
            return parseInt(this.storeId()) === parseInt(value);
        },

        /**
         * Detect select all checkbox changes
         */
        selectAllPromptFieldsChanged: function () {
            var checked = self.selectAllPromptFields(),
                allFields = $(self.fieldsSelector)
                    .not('[value=\'all\']')
                    .map(function () {
                        return $(this).val();
                    })
                    .get();

            self.selectedFields(checked ? allFields : []);
        },

        /**
         * Update all checkboxes
         */
        updateSelect: function () {
            var selector = $(self.fieldsSelector).not('[value=\'all\']'),
                selected = 0;

            selector.each(function () {
                if ($(this).is(':checked')) {
                    selected++;
                }
            });

            if (selected === selector.length) {
                self.selectAllPromptFields(true);
            } else {
                self.selectAllPromptFields(false);
            }
        }
    });
});
