/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'uiComponent',
    'mage/translate',
    'Magento_Ui/js/modal/modal'
], function ($, Component, $t) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            unlockPremiumUrl: '',
            premiumPopupModal: null,
            selectors: {
                premiumPopupModalSelector: '.writetextai-premium-popup-modal',
                premiumPopupTriggerSelector: 'span.premium',
                premiumPopupOverlaySelector: '.writetextai-premium-popup-overlay',
            },
            classes: {
                modalClass: 'writetextai-premium-popup-modal',
                overlayClass: 'writetextai-premium-popup-overlay'
            }
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super().observe([]);
    
            return this;
        },
    
        /** @inheritdoc */
        initialize: function () {
            this._super();
    
            self = this;
    
            this.initModal();
        },

        /**
         * Initialize history log modal.
         */
        initModal: function () {
            this.premiumPopup = $(this.selectors.premiumPopupModalSelector).modal({
                title: $t('Unlock Premium'),
                subTitle: $t("Unlock Premium features by purchasing credits with our pay-as-you-go solution, or by subscribing to a monthly or annual plan. With Premium, you'll be able to:"),
                innerScroll: true,
                modalClass: self.classes.modalClass,
                overlayClass: self.classes.overlayClass,
                buttons: [{
                    text: $t('Unlock Premium'),
                    class: 'action-premium',
                    click: function () {
                        window.open(self.unlockPremiumUrl, '_blank');
                    }
                }],
                closed: function () {
                    $(self.selectors.premiumPopupOverlaySelector).remove();
                }
            });

            $(this.selectors.premiumPopupModalSelector).css('display', 'block');

            $(document).on('click', self.selectors.premiumPopupTriggerSelector, function (event) {
                self.premiumPopup.modal('openModal');
            });
        }
    });
});
