/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'uiComponent',
    'WriteTextAI_WriteTextAI/js/model/edit/textfields',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'WriteTextAI_WriteTextAI/js/model/grid/notifications',
    'WriteTextAI_WriteTextAI/js/model/signalr',
    'WriteTextAI_WriteTextAI/js/model/edit',
    'WriteTextAI_WriteTextAI/js/model/edit/gallery',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'signalR',
    'mage/translate'
], function (
    $,
    ko,
    Component,
    textfields,
    product,
    notifications,
    signalRModel,
    editData,
    gallery,
    premium,
    prompt,
    totalCredits,
    signalR,
    $t
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            productId: '',
            storeId: '',
            connection: null,
            retryReconnectAttempt: 0,
            enableMaxRetryAttempt: false,
            maxRetryAttempt: 10,
            accessToken: '',
            signalRUrl: '',
            singleGenerate: 0,
            bulkGenerate: 1,
            fieldStatus: 2,
            message: $t('Text generation completed'),
            showMessage: false,
            messageSuccess: false,
            successMessage: $t('Text generation completed'),
            pageTitle: '',
            pageDescription: '',
            productDescription: '',
            excerpt: '',
            openGraph: '',
            errorMessage: '',
            statusFailed: 'Failed',
            statusCompleted: 'Completed'
        },

        isGettingNewToken: false,

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            this.connectTostream();

            product.productId.subscribe(function (productId) {
                self.productId(productId);
            });

            product.storeId.subscribe(function (storeId) {
                self.storeId(storeId);
            });

            signalRModel.showMessage.subscribe(function (show) {
                self.showMessage(show);
            });

            signalRModel.message.subscribe(function (message) {
                self.message(message);
            });
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super().observe([
                'productId',
                'storeId',
                'accessToken',
                'signalRUrl',
                'message',
                'showMessage',
                'messageSuccess',
                'errorMessage'
            ]);

            this.generateComplete = ko.computed(function () {
                var fieldsGenerated = signalRModel.fieldsGenerated(),
                    fieldsValues = Object.values(fieldsGenerated),
                    hasTrue = fieldsValues.includes(true),
                    hasFalse = fieldsValues.includes(false),
                    imagesGenerating = signalRModel.imagesGenerating(),
                    imagesGenerated = signalRModel.imagesGenerated(),
                    imagesGeneratedLength = imagesGenerated ? imagesGenerated.length : 0,
                    imagesGeneratingLength = imagesGenerating ? imagesGenerating.length : 0;

                var isGenerating = signalRModel.generating(),
                    isTextsGenerating = hasTrue || hasFalse,
                    isTextsGenerated = hasTrue && !hasFalse,
                    isImagesGenerating = imagesGeneratingLength > 0 && imagesGeneratedLength > 0,
                    isImagesGenerated = imagesGeneratingLength === imagesGeneratedLength;

                if (isGenerating && (
                    (isTextsGenerating && !isImagesGenerating && isTextsGenerated) || /** texts only */
                    (!isTextsGenerating && isImagesGenerating && isImagesGenerated) || /** images only */
                    (isTextsGenerating && isImagesGenerating && isTextsGenerated && isImagesGenerated) /** texts and images */
                )) {
                    return true;
                }

                return false;
            });

            this.generateComplete.subscribe(function (generateComplete) {
                if (generateComplete) {
                    self.catchFieldsError();
                    self.getGenerated();
                }
            });

            return this;
        },

        /**
         * Catch fields error.
         */
        catchFieldsError: function () {
            var fieldsGenerated = signalRModel.fieldsGenerated(),
                fieldsValues = Object.values(fieldsGenerated),
                hasTrue = fieldsValues.includes(true),
                hasFalse = fieldsValues.includes(false),
                imagesGenerating = signalRModel.imagesGenerating(),
                imagesGenerated = signalRModel.imagesGenerated(),
                imagesGeneratedLength = imagesGenerated ? imagesGenerated.length : 0,
                imagesGeneratingLength = imagesGenerating ? imagesGenerating.length : 0;

            var isTextsGenerating = hasTrue || hasFalse,
                isImagesGenerating = imagesGeneratingLength > 0 && imagesGeneratedLength > 0;

            self.errorMessage('');
            self.messageSuccess(true);
                
            switch (true) {
                case !isTextsGenerating && isImagesGenerating:
                    self.getAltTextsError();
                    break;
                case isTextsGenerating && !isImagesGenerating:
                    self.getNormalTextsError();
                    break;
                case isTextsGenerating && isImagesGenerating:
                    self.getAllTextsError();
                    break;
                default:
                    break;
            }
        },

        /**
         * Get all texts error.
         */
        getAllTextsError: function () {
            var errorFields = signalRModel.errorFields(),
                imagesHtml = '';

            if (gallery.failedImagesWithoutThumbnail().length > 0) {
                errorFields.push('alt text');
                gallery.failedImagesWithoutThumbnail().forEach(element => {
                    imagesHtml += '<img src="' + element + '">';
                });
            }
            
            var htmlMessage = self.sprintf(
                $t('Error encountered while generating the following text: %s'),
                '<ul>' + errorFields.map(field => `<li>${field}</li>`).join('') + '</ul>'
            );

            htmlMessage += imagesHtml;
            
            if (gallery.failedImagesWithoutThumbnail().length > 0 || errorFields.length > 0) {
                signalRModel.message(self.successMessage);
                self.errorMessage(htmlMessage);
            }
        },

        /**
         * Get alt texts error.
         */
        getAltTextsError: function () {
            var errorFields = ['alt text'],
                imagesHtml = self.sprintf(
                    $t('Error encountered while generating the following text: %s'),
                    '<ul>' + errorFields.map(field => `<li>${field}</li>`).join('') + '</ul>'
                );
            
            if (gallery.failedImagesWithoutThumbnail().length > 0) {
                signalRModel.message(self.successMessage);
                gallery.failedImagesWithoutThumbnail().forEach(element => {
                    imagesHtml += '<img src="' + element + '">';
                });
                self.errorMessage(imagesHtml);
            }
        },

        /**
         * Get normal texts error.
         */
        getNormalTextsError: function () {
            var errorFields = signalRModel.errorFields(),
                selectedFields = signalRModel.selectedFields(),
                htmlMessage = self.sprintf(
                    $t('Error encountered while generating the following text: %s'),
                    '<ul>' + errorFields.map(field => `<li>${field}</li>`).join('') + '</ul>'
                );

            if (selectedFields.length > 0 && selectedFields.length === errorFields.length) {
                self.messageSuccess(false);
                signalRModel.message(signalRModel.errorMessage());
            } else if (errorFields.length > 0) {
                signalRModel.message(self.successMessage);
                self.errorMessage(htmlMessage);
            }
        },

        /**
         * @param {String} format
         * @returns {String}
         */
        sprintf: function (format) {
            var args = Array.prototype.slice.call(arguments, 1);

            return format.replace(/%s/g, function () {
                return args.shift();
            });
        },

        /**
         * Hide text generation success message.
         */
        hideMessage: function () {
            signalRModel.showMessage(false);
        },

        /**
         * Connect to signalR stream.
         */
        connectTostream: function () {
            if (self.accessToken()) {
                self.connection = new signalR.HubConnectionBuilder()
                    .withUrl(self.signalRUrl(), {
                        accessTokenFactory: () => self.accessToken()
                    })
                    .build();
                self.connection
                    .start()
                    .then(() => self.onConnected(self.connection))
                    .catch((error) => self.onConnectionError(error));

                self.bindConnectionMessageGenerate(self.connection);

                $(window).on('focus', function () {
                    if (
                        self.connection != null &&
                        self.connection.connectionState != "Connected" &&
                        self.connection.connectionState != "Connecting"
                    ) {
                        self.connection.start();
                    }
                });
            }
        },

        /**
         * Connection is established actions.
         *
         * @param {object} connection
         * @returns {void}
         */
        onConnected: function (connection) {
            connection.send('broadcastMessage', '_SYSTEM_', 'Connected');
        },

        /**
         * Connection error actions.
         *
         * @param {object} error
         * @returns {void}
         */
        onConnectionError: function (error) {
            var doConnect = true;

            if (error.statusCode === 401 && !self.isGettingNewToken) {
                self.getNewAccessToken();
            }

            if (self.enableMaxRetryAttempt && self.retryReconnectAttempt >= self.maxRetryAttempt) {
                doConnect = false;
            }

            if (doConnect) {
                if (self.retryReconnectAttempt === 0) {
                    self.connectTostream();
                } else {
                    setTimeout(function () {
                        self.connectTostream(); /* attempt to connect every 5 seconds */
                    }, 5000);
                }

                self.retryReconnectAttempt++;
            }
        },

        /**
         * Get new access token.
         */
        getNewAccessToken: function () {
            var url = self.getNewAccessTokenUrl;
        
            self.isGettingNewToken = true;
            $.ajax({
                url: url,
                type: 'GET',
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (response.success) {
                        self.accessToken(response.access_token);
                    }
                }
            }).always(function () {
                self.isGettingNewToken = false;
            });
        },

        /**
         * Bind connection messages.
         *
         * @param {Object} connection
         * @returns {void}
         */
        bindConnectionMessageGenerate: function (connection) {
            if (connection === null) {
                /* bypass */
            }

            connection.on('broadcastMessage', self.messageCallback);
            connection.onclose(self.onConnectionError);
        },

        /**
         * Parse message.
         *
         * @param {String} encodedName
         * @param {String} encodedMsg
         * @returns {Object}
         */
        createMessageEntry: function (encodedName, encodedMsg) {
            return {
                encodedName: encodedName,
                encodedMsg: JSON.parse(encodedMsg)
            };
        },

        /**
         * Update text fields.
         *
         * @param {String} field
         * @param {String} value
         * @returns {void}
         */
        updateField: function (field, value) {
            switch (field) {
                case self.pageTitle:
                    textfields.pageTitle(value);
                    break;
                case self.pageDescription:
                    textfields.pageDescription(value);
                    break;
                case self.productDescription:
                    // textfields.productDescription(value);
                    break;
                case self.excerpt:
                    // textfields.productShortDescription(value);
                    break;
                case self.openGraph:
                    textfields.openGraph(value);
                    break;
            }
        },

        /**
         * Get text field value.
         *
         * @param {String} field
         * @returns {String}
         */
        getField: function (field) {
            var value = '';

            switch (field) {
                case self.pageTitle:
                    value = textfields.pageTitle();
                    break;
                case self.pageDescription:
                    value = textfields.pageDescription();
                    break;
                case self.productDescription:
                    value = textfields.productDescription();
                    break;
                case self.excerpt:
                    value = textfields.productShortDescription();
                    break;
                case self.openGraph:
                    value = textfields.openGraph();
                    break;
            }

            return value;
        },

        /**
         * Get index field.
         *
         * @param {String} field
         * @returns {String}
         */
        getIndexField: function (field) {
            var value = '';

            switch (field) {
                case self.pageTitle:
                    value = 'page_title';
                    break;
                case self.pageDescription:
                    value = 'page_description';
                    break;
                case self.productDescription:
                    value = 'product_description';
                    break;
                case self.excerpt:
                    value = 'excerpt';
                    break;
                case self.openGraph:
                    value = 'open_graph';
                    break;
            }

            return value;
        },

        /**
         * Process stream message.
         *
         * @param {String} name
         * @param {String} message
         * @returns {void}
         */
        messageCallback: function (name, message) {
            var encodedName = name,
                encodedMsg = message
                    .replace(/&/g, '&amp;')
                    .replace(/</g, '&lt;')
                    .replace(/>/g, '&gt;'),
                messageEntry = self.createMessageEntry(encodedName, encodedMsg),
                streamType = messageEntry.encodedMsg.type,
                streamFieldType = messageEntry.encodedMsg.field,
                streamStop = messageEntry.encodedMsg.stop,
                streamPartialText = messageEntry.encodedMsg.partialText,
                streamRecordId = messageEntry.encodedMsg.recordId,
                streamStoreId = messageEntry.encodedMsg.storeId,
                streamIndex = messageEntry.encodedMsg.index,
                indexField = '',
                streamStatus = '',
                fieldsGenerated = {},
                productId = self.productId(),
                storeId = self.storeId();

            if (streamType === self.fieldStatus) {
                indexField = self.getIndexField(streamFieldType);
                streamStatus = messageEntry.encodedMsg.status;

                if (streamStatus === self.statusFailed) {
                    signalRModel.errorFields.push(streamFieldType);
                    signalRModel.errorMessage(messageEntry.encodedMsg.error);
                }

                if (streamFieldType !== "alt text") {
                    fieldsGenerated = signalRModel.fieldsGenerated();
                    fieldsGenerated[indexField] = true;
                    signalRModel.fieldsGenerated(fieldsGenerated);
                    if (productId === streamRecordId && storeId === streamStoreId) {
                        var statuses = textfields.statuses();
                        switch (indexField) {
                            case 'page_title':
                                $('.writetextai-page-title-loading-mask').removeClass('show');
                                statuses['pageTitleGenerateStatus'] = true;
                                break;
                            case 'page_description':
                                $('.writetextai-page-description-loading-mask').removeClass('show');
                                statuses['pageDescriptionGenerateStatus'] = true;
                                break;
                            case 'product_description':
                                $('.writetextai-product-description-loading-mask').removeClass('show');
                                statuses['productDescriptionGenerateStatus'] = true;
                                break;
                            case 'excerpt':
                                $('.writetextai-short-description-loading-mask').removeClass('show');
                                statuses['productShortDescriptionGenerateStatus'] = true;
                                break;
                            case 'open_graph':
                                $('.writetextai-open-graph-loading-mask').removeClass('show');
                                statuses['openGraphGenerateStatus'] = true;
                                break;
                        }
                        
                        textfields.statuses(statuses);
                    }
                }

                if (streamFieldType === "alt text" && streamStatus === self.statusCompleted) {
                    signalRModel.imagesGenerated([...signalRModel.imagesGenerated(), ...streamRecordId.split(",")]);
                    if (productId) {
                        self.getImages(productId, storeId);
                    }
                }
            }

            if (streamType === self.singleGenerate
                && signalRModel.generating() === true
            ) {
                indexField = self.getIndexField(streamFieldType);

                if (productId === streamRecordId && storeId === streamStoreId) {
                    /* eslint-disable max-depth */

                    if (streamIndex === signalRModel.index()[indexField]) {
                        self.processSingleGenerate(
                            streamFieldType,
                            streamStop,
                            streamPartialText,
                            streamIndex
                        );
                        streamIndex += 1;
                        signalRModel.index()[indexField] = streamIndex;

                        while (
                            typeof signalRModel.messages()[indexField][
                                streamIndex
                            ] !== 'undefined'
                        ) {
                            self.processSingleGenerate(
                                signalRModel.messages()[indexField][streamIndex]
                                    .encodedMsg.field,
                                signalRModel.messages()[indexField][streamIndex]
                                    .encodedMsg.stop,
                                signalRModel.messages()[indexField][streamIndex]
                                    .encodedMsg.partialText,
                                signalRModel.messages()[indexField][streamIndex]
                                    .encodedMsg.index
                            );
                            streamIndex += 1;
                            signalRModel.index()[indexField] = streamIndex;
                        }
                    } else {
                        signalRModel.messages()[indexField][streamIndex] =
                            messageEntry;
                    }

                    if (streamStop === true) {
                        /* generate end */
                        signalRModel.index()[indexField] = 0;
                        signalRModel.messages()[indexField] = {};
                    }
                    /* eslint-enable max-depth */
                }
            }

            if (streamType === self.bulkGenerate) {
                notifications.addStreamStatus(messageEntry);
            }
        },

        /**
         * Get images.
         *
         * @param {Number} productId
         * @param {Number} storeId
         */
        getImages: function (productId, storeId) {
            $.ajax({
                url: self.getImagesUrl,
                type: 'POST',
                data: {
                    product_id: productId,
                    store_id: storeId
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showGalleryLoader: true,
                success: function (response) {
                    if (response.success) {
                        gallery.images(response.images);
                    }
                }
            });
        },

        /**
         * Process single generate.
         *
         * @param {String} streamFieldType
         * @param {Boolean} streamStop
         * @param {String} streamPartialText
         * @param {Number} streamIndex
         * @returns {void}
         */
        processSingleGenerate: function (
            streamFieldType,
            streamStop,
            streamPartialText,
            streamIndex
        ) {
            var currentValue = self.getField(streamFieldType),
                newValue = currentValue + streamPartialText;

            /* generate starts */
            if (streamIndex === 0) {
                self.updateField(streamFieldType, streamPartialText);
            }

            /* generate partial */
            if (
                streamPartialText !== null &&
                streamIndex !== null &&
                streamIndex !== 0 &&
                streamStop === false
            ) {
                self.updateField(streamFieldType, newValue);
            }
        },

        /**
         * Get generated text fields.
         */
        getGenerated: function () {
            var url = self.getGeneratedUrl,
                statuses = textfields.statuses(),
                productId = self.productId(),
                storeId = self.storeId(),
                selectedFields = signalRModel.selectedFields(),
                imagesGenerated = signalRModel.imagesGenerated();

            $.ajax({
                url: url,
                type: 'POST',
                data: {
                    product_id: productId,
                    store_id: storeId
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showPageTitleLoader: selectedFields.includes('page_title'),
                showPageDescriptionLoader: selectedFields.includes('page_description'),
                showProductDescriptionLoader: selectedFields.includes('product_description'),
                showShortDescriptionLoader: selectedFields.includes('short_product_description'),
                showOpenGraphLoader: selectedFields.includes('open_graph'),
                showGalleryLoader: imagesGenerated.length > 0,
                success: function (response) {
                    premium.isPremium(response.is_premium);
                    prompt.isPremium(response.is_premium);
                    totalCredits.totalCredits(response.total_credits);
                    
                    if (response.success) {
                        selectedFields.forEach((field) => {
                            switch (field) {
                                case 'page_title':
                                    textfields.pageTitle(
                                        response.textfields.page_title
                                    );
                                    textfields.originalPageTitle(
                                        response.textfields.page_title
                                    );
                                    statuses['pageTitleGenerateStatus'] = true;
                                    statuses['pageTitleTransferStatus'] = false;
                                    $('#page-title-mask').addClass('generated');
                                    break;
                                case 'page_description':
                                    textfields.pageDescription(
                                        response.textfields.page_description
                                    );
                                    textfields.originalPageDescription(
                                        response.textfields.page_description
                                    );
                                    statuses['pageDescriptionGenerateStatus'] = true;
                                    statuses['pageDescriptionTransferStatus'] = false;
                                    $('#page-desc-mask').addClass('generated');
                                    break;
                                case 'product_description':
                                    textfields.productDescription(
                                        response.textfields.product_description
                                    );
                                    textfields.originalProductDescription(
                                        response.textfields.product_description
                                    );
                                    statuses['productDescriptionGenerateStatus'] = true;
                                    statuses['productDescriptionTransferStatus'] = false;
                                    $('#product-desc-mask + .mce-tinymce').addClass('generated');
                                    $('#product-desc-mask + .tox-tinymce').addClass('generated');
                                    break;
                                case 'short_product_description':
                                    textfields.productShortDescription(
                                        response.textfields
                                            .short_product_description
                                    );
                                    textfields.originalProductShortDescription(
                                        response.textfields
                                            .short_product_description
                                    );
                                    statuses['productShortDescriptionGenerateStatus'] = true;
                                    statuses['productShortDescriptionTransferStatus'] = false;
                                    $('#excerpt-mask + .mce-tinymce').addClass('generated');
                                    $('#excerpt-mask + .tox-tinymce').addClass('generated');
                                    break;
                                case 'open_graph':
                                    textfields.openGraph(
                                        response.textfields.open_graph
                                    );
                                    textfields.originalOpenGraph(
                                        response.textfields.open_graph
                                    );
                                    statuses['openGraphGenerateStatus'] = true;
                                    statuses['openGraphTransferStatus'] = false;
                                    $('#open-graph-mask').addClass('generated');
                                    break;
                            }
                            textfields.statuses(statuses);
                        });

                        if (editData.opened()) {
                            signalRModel.showMessage(true);
                            self.showMessage(true);
                        }
                        signalRModel.generating(false);
                    } else {
                        console.log(response.message);
                    }
                }
            });
        }
    });
});
