/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'WriteTextAI_WriteTextAI/js/model/edit/textfields',
    'WriteTextAI_WriteTextAI/js/model/signalr',
    'WriteTextAI_WriteTextAI/js/signalr'
], function (
    $,
    textfields,
    signalRModel,
    signalRComponent
) {
    'use strict';

    /**
     * Remove empty fields.
     */
    function removeEmptyFields(fields)
    {
        var selectedFields = fields;

        selectedFields.each((field) => {
            switch (field) {
                case 'page_title':
                    if (textfields.mgPageTitle() === '') {
                        selectedFields.splice(
                            selectedFields.indexOf(field),
                            1
                        );
                    }
                    break;
                case 'page_description':
                    if (textfields.mgPageDescription() === '') {
                        selectedFields.splice(
                            selectedFields.indexOf(field),
                            1
                        );
                    }
                    break;
                case 'product_description':
                    if (textfields.mgProductDescription() === '') {
                        selectedFields.splice(
                            selectedFields.indexOf(field),
                            1
                        );
                    }
                    break;
                case 'short_product_description':
                    if (textfields.mgProductShortDescription() === '') {
                        selectedFields.splice(
                            selectedFields.indexOf(field),
                            1
                        );
                    }
                    break;
                case 'open_graph':
                    if (textfields.mgOpenGraph() === '') {
                        selectedFields.splice(
                            selectedFields.indexOf(field),
                            1
                        );
                    }
                    break;
            }
        });

        return selectedFields;
    }

    /**
     * Clear text fields.
     */
    function clearFields(selectedFields, selectedImages)
    {
        signalRComponent.prototype.hideMessage();
        signalRModel.errorFields([]);
        signalRModel.errorMessage('');
        signalRModel.selectedFields(selectedFields);
        signalRModel.imagesGenerating(selectedImages);
        signalRModel.imagesGenerated([]);
        signalRModel.fieldsGenerated({
            page_title: null,
            page_description: null,
            product_description: null,
            excerpt: null,
            open_graph: null
        });
        selectedFields.forEach((field) => {
            switch (field) {
                case 'page_title':
                    textfields.pageTitle('');
                    signalRModel.fieldsGenerated()['page_title'] = false;
                    $('#page-title-mask').removeClass('generated');
                    break;

                case 'page_description':
                    textfields.pageDescription('');
                    signalRModel.fieldsGenerated()[
                        'page_description'
                    ] = false;
                    $('#page-desc-mask').removeClass('generated');
                    break;

                case 'product_description':
                    textfields.productDescription('');
                    signalRModel.fieldsGenerated()[
                        'product_description'
                    ] = false;
                    $('#product-desc-mask + .mce-tinymce').removeClass('generated');
                    $('#product-desc-mask + .tox-tinymce').removeClass('generated');
                    break;

                case 'short_product_description':
                    textfields.productShortDescription('');
                    signalRModel.fieldsGenerated()['excerpt'] = false;
                    $('#excerpt-mask + .mce-tinymce').removeClass('generated');
                    $('#excerpt-mask + .tox-tinymce').removeClass('generated');
                    break;

                case 'open_graph':
                    textfields.openGraph('');
                    signalRModel.fieldsGenerated()['open_graph'] = false;
                    $('#open-graph-mask').removeClass('generated');
                    break;
            }
        });
        /** Ensure this is set last to prevent premature notification display. */
        signalRModel.generating(true);
    }

    return {
        removeEmptyFields: removeEmptyFields,
        clearFields: clearFields
    }
});