/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'WriteTextAI_WriteTextAI/js/model/edit/textfields',
    'mage/translate'
], function ($, textfields, $t) {
    'use strict';

    /**
     * @param {String} keyValue
     * @returns {Boolean}
     */
    function ensureNumerical(keyValue)
    {
        if (keyValue.match(/[0-9]/g)) {
            return true;
        }

        return false;
    }

    /**
     * Update current fields.
     */
    function updateCurrentFields(fields, selectedFields, writeTextFields)
    {
        var statuses = textfields.statuses();

        if (selectedFields.includes('page_title') && writeTextFields.page_title) {
            textfields.mgPageTitle(fields.page_title);
            textfields.pageTitle(writeTextFields.page_title);
            textfields.originalPageTitle(writeTextFields.page_title);
            statuses['pageTitleTransferStatus'] = true;
        }
        if (selectedFields.includes('page_description') && writeTextFields.page_description) {
            textfields.mgPageDescription(fields.page_description);
            textfields.pageDescription(writeTextFields.page_description);
            textfields.originalPageDescription(writeTextFields.page_description);
            statuses['pageDescriptionTransferStatus'] = true;
        }
        if (selectedFields.includes('product_description') && writeTextFields.product_description) {
            textfields.mgProductDescription(fields.product_description);
            textfields.productDescription(writeTextFields.product_description);
            textfields.originalProductDescription(writeTextFields.product_description);
            statuses['productDescriptionTransferStatus'] = true;
        }
        if (selectedFields.includes('short_product_description')
            && writeTextFields.short_product_description) {
            textfields.mgProductShortDescription(
                fields.short_product_description
            );
            textfields.productShortDescription(writeTextFields.short_product_description);
            textfields.originalProductShortDescription(writeTextFields.short_product_description);
            statuses['productShortDescriptionTransferStatus'] = true;
        }
        if (selectedFields.includes('open_graph')  && writeTextFields.open_graph) {
            textfields.mgOpenGraph(fields.open_graph);
            textfields.openGraph(writeTextFields.open_graph);
            textfields.originalOpenGraph(writeTextFields.open_graph);
            statuses['openGraphTransferStatus'] = true;
        }
        textfields.statuses(statuses);
    }

    /**
     * Get char count.
     *
     * @param {String} html
     * @param {Number} limit
     * @param {Boolean} unit
     * @returns {Number}
     */
    function getCharCount(html = '', limit = null, unit = true)
    {
        var br2nl = html.replace(/<br\s*\/?>/g, '\n'),
            parseHtml = $($.parseHTML(br2nl)).text(),
            noHtmlTags = parseHtml.replace(/<[^>]*>/g, ''),
            count = noHtmlTags ? noHtmlTags.length : 0;

        if (!unit) {
            return count;
        }
        
        return count + (limit ? '/' + limit : '') + ' ' + $t('Char');
    }

    /**
     * Get word count.
     *
     * @param {String} html
     * @param {Boolean} unit
     * @returns {Number}
     */
    function getWordCount(html = '', unit = true)
    {
        var br2nl = html.replace(/<br\s*\/?>/g, '\n'),
            parseHtml = $($.parseHTML(br2nl)).text(),
            noHtmlTags = parseHtml.replace(/<[^>]*>/g, ''),
            words = noHtmlTags.toLowerCase().match(/\b(?:[\w'‘’′]+(?:[.-][\w'‘’′]+)*|[\w'‘’′]+)\b/g),
            count = words ? words.length : 0;

        if (!unit) {
            return count;
        }

        return count + ' ' + $t('word/s');
    }

    /**
     * Count keyword occurences.
     *
     * @param {String} html
     * @param {String} keyword
     * @returns {Number}
     */
    function countKeywordOccurences(html, keyword)
    {
        var br2nl = html.replace(/<br\s*\/?>/g, '\n'),
            parseHtml = $($.parseHTML(br2nl)).text(),
            noHtmlTags = parseHtml.replace(/<[^>]*>/g, '');
        var escapedString = keyword.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
        return (noHtmlTags.match(new RegExp(escapedString, "mgi")) || []).length;
    }

    return {
        ensureNumerical: ensureNumerical,
        updateCurrentFields: updateCurrentFields,
        getCharCount: getCharCount,
        getWordCount: getWordCount,
        countKeywordOccurences: countKeywordOccurences
    };
});
